/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * Este proyecto procesa la informacion pasada por GRUB como parametro a
 * la rutina principal del kernel.
 *
 * Dentro de esta informacion, resalta el mapa de memoria. Este permite
 * determinar las regiones de memoria RAM que se encuentran disponibles.
 *
 * Este kernel busca la region de memoria de mayor tamano que encuentre
 * por encima del limite de 1 MB y por encima de la memoria que ocupa
 * el kernel.
 *
 * El codigo para buscar la region de memoria se encuentra en el archivo
 * multiboot.h
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */
void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para buscar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	unsigned int n; /* Numero entero */
	int i;

	/* Recuperar los valores del encabezado multiboot definidos en el
	 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	cls();

	printf("Inicio del kernel: %x\n", multiboot_header.kernel_start);
	printf("Fin del segmento de datos: %x\n", multiboot_header.data_end);
	printf("Fin del segmento BSS: %x\n", multiboot_header.bss_end);
	printf("Punto de entrada del kernel: %x\n", multiboot_header.entry_point);

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	printf("Info flags: %b\n", info->flags);

	// si flags[0] = 1, los campos mem_lower y mem_upper son validos
	if (test_bit(info->flags, 0)) {
		printf("Mem lower: %d\n", info->mem_lower);
		printf("Mem upper: %d\n", info->mem_upper);
	}

	// si flags[1] = 1, el campo boot_device es valido
	if (test_bit(info->flags, 1)) {
		printf("Boot device: %x\n", info->boot_device);
	}

	// si flags[2] = 1, el campo cmdline es valido
	if (test_bit(info->flags, 2)) {
		printf("cmdline: %s\n", info->cmdline);
	}

	/* si flags[3] = 1, se especificaron modulos que deben ser cargados junto
	 * con el kernel*/
	if (test_bit(info->flags, 3)) {
		printf("Modules available!. Start: %u Count: %u\n", info->mods_addr,
				info->mods_count);
		mod_info_t * mod_info = (mod_info_t*)info->mods_addr;
		for (i=0; i<info->mods_count; i++) {
			printf("start: %u end: %u cmdline: %s\n", mod_info->mod_start,
					mod_info->mod_end, mod_info->string);
			mod_info++;
		}
	}

	/* Buscar la region de memoria disponible, que se encuentra
	 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Se encontro memoria disponible? */
	if (memory_length > 0) {
		printf("Available memory found at %u. Length: %dK\n",
				memory_start, memory_length / 1024);
	}else {
		printf("No available memory found!\n");
	}

}
