/*
 * multiboot.c
 * Contiene rutinas asociadas con la especificacion multiboot.
 */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Rutina: get_available_memory
 * Descripcion: Determina la region de memoria de mayor tamanio que se
 * encuentra por encima del kernel. Usa la informacion enviada por el
 * cargador de arranque compatible con la especificacion multiboot.
 * Entrada:
 * 	multiboot_header_t : Variable que contiene el encabezado multiboot
 *  multiboot_info_t * : Apuntador a la estructura de informacion
 *  					 multiboot pasada al kernel por el cargador de arranque
 *  unsigned int *    : Referencia a la variable en la cual se almacenara la
 *  				     direccion de memoria de la region disponible
 *  unsigned int *    : Referencia a la variable en la cual se almacenara
 *						 el tamanio de la region disponible encontrada
 * Salida:
 * 	Ninguno.
 * En caso de no encontrar una region disponible, los dos parametros
 * unsigned int * se establecen a cero.
 * */
void get_available_memory(multiboot_header_t multiboot_header,
	multiboot_info_t *info,
	unsigned int * memory_start, unsigned int * memory_length) {
/* Variables temporales para hallar la region de memoria disponible */
	unsigned int tmp_start;
	unsigned int tmp_length;
	int mod_count;

	unsigned int mods_end; /* Almacena la direccion de memoria final
	del ultimo modulo cargado, o 0 si no se cargaron modulos. */

	/*
	printf("Inicio del kernel: %x\n", multiboot_header.kernel_start);
	printf("Fin del segmento de datos: %x\n", multiboot_header.data_end);
	printf("Fin del segmento BSS: %x\n", multiboot_header.bss_end);
	printf("Punto de entrada del kernel: %x\n", multiboot_header.entry_point);
	*/

	/* si flags[3] = 1, se especificaron modulos que deben ser cargados junto
	 * con el kernel*/

	mods_end = 0;

	if (test_bit(info->flags, 3)) {
		mod_info_t * mod_info;
		/*
		printf("Modules available!. Start: %u Count: %u\n", info->mods_addr,
				info->mods_count);
		*/
		for (mod_info = (mod_info_t*)info->mods_addr, mod_count=0;
				mod_count <info->mods_count;
				mod_count++, mod_info++) {
			/*
			printf("[%d] start: %u end: %u cmdline: %s \n", mod_count,
					mod_info->mod_start, mod_info->mod_end,
					mod_info->string);*/
			if (mod_info->mod_end > mods_end) {
				/* Los modulos se redondean a limites de 4 KB, redondear
				 * la direccion final del modulo a un limite de 4096 */
				mods_end = mod_info->mod_end + (mod_info->mod_end % 4096);
			}
		}
	}

	//printf("Mods end: %u\n", mods_end);

	/* si flags[6] = 1, los campos mmap_length y mmap_addr son validos */

	/* Revisar las regiones de memoria, y extraer la region de memoria
	 * de mayor tamano, maracada como disponible, cuya direccion base sea
	* mayor o igual a la posicion del kernel en memoria.
	*/

	*memory_start = 0;
	*memory_length = 0;

	if (test_bit(info->flags, 6)) {
		memory_map_t *mmap;

		printf ("mmap_addr = 0x%x, mmap_length = 0x%x\n",
			   (unsigned) info->mmap_addr, (unsigned) info->mmap_length);
		for (mmap = (memory_map_t *) info->mmap_addr;
			(unsigned int) mmap < info->mmap_addr + info->mmap_length;
			mmap = (memory_map_t *) ((unsigned int) mmap
									 + mmap->entry_size
									 + sizeof (mmap->entry_size))) {
		 printf (" size = 0x%x, base_addr = 0x%x%x,"
				 " length = 0x%x%x, type = 0x%x\n",
				  mmap->entry_size,
				  mmap->base_addr_high,
				  mmap->base_addr_low,
				  mmap->length_high,
				  mmap->length_low,
				  mmap->type);

	  /* Verificar si esta region de memoria cumple con las condiciones
	   * para ser considerada "memoria disponible".
	   *
	   * Importante: Si se supone un procesador de 32 bits, los valores
	   * de la parte alta de base y length (base_addr_high y
	   * length_high) son cero. Por esta razon se pueden ignorar y solo
	   * se usan los 32 bits menos significativos de base y length.
	   *
	   * Para que una region de memoria sea considerada "memoria
	   * disponible", debe cumplir con las siguientes condiciones.
	   *
	   * 1. Estar ubicada en una posicion de memoria mayor o igual que
	   * 	1 MB.
	   * 2. Tener su atributo 'type' en 1 = memoria disponible.
	   * */
		 /* La region esta marcada como disponible y su direccion base
		  * esta por encima de la posicion del kernel en memoria ?*/
		 if (mmap->type == 1 &&
			 mmap->base_addr_low >= multiboot_header.kernel_start) {
			 tmp_start = mmap->base_addr_low;
			 tmp_length = mmap->length_low;

			 /* Verificar si el kernel se encuentra en esta region */
			 if (multiboot_header.bss_end >= tmp_start &&
					 multiboot_header.bss_end <= tmp_start + tmp_length) {
				 //printf("Kernel is on this region!. Base: %u\n", tmp_start);
				 /* El kernel se encuentra en esta region. Tomar el inicio
				  * de la memoria disponible en la posicion en la cual
				  * finaliza el kernel
				 */
				 tmp_start = multiboot_header.bss_end;

				 /* Ahora verificar si ser cargaron modulos junto con el
				  * kernel. Estos modulos se cargan en regiones continuas
				  * al kernel.
				  * Si es asi, la nueva posicion inicial de la memoria
				  * disponible es la posicion en la cual terminan los modulos
				  * */
				 if (mods_end > 0 &&
						 mods_end >= tmp_start &&
						 mods_end <= tmp_start + tmp_length) {
						//printf("Adding module space...\n");
						tmp_start = mods_end;
				 }

				 /* Restar al espacio disponible.*/
				 tmp_length -= tmp_start - mmap->base_addr_low;
				 if (tmp_length > *memory_length) {
					 *memory_start = tmp_start;
					 *memory_length = tmp_length; /* Tomar el espacio */
				 }
			 }else {
				 /* El kernel no se encuentra en esta region, verificar si
				  * su tamano es mayor que la region mas grande encontrada
				  * hasta ahora
				  */
				 if (tmp_length > *memory_length) {
					 *memory_start = tmp_start;
					 *memory_length = tmp_length; /* Tomar el espacio */
				 }
			 }
		 }
		} //endfor
	}
}

