/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * Este proyecto implementa la funcionalidad necesaria para cargar una nueva
 * tabla global de descriptores (GDT). Esta tabla controla la ejecucion del
 * procesador en modo protegido.
 * Debido a que este kernel es cargado y ejecutado por un cargador de arranque
 * compatible con la Especificacion Multiboot, es necesario	crear una tabla
 * global de descriptores tan pronto como sea posible.
 *
 * Las estructuras de datos y las rutinas necesarias se definen en el archivo
 * include/pm.h y se implementan en el archivo src/pm.c
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <pm.h>

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */


void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	char * linea;

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Si no se encuentra memoria disponible, terminar */
	if (memory_length <= 0) {
		printf("No available memory found!\n");
		return;
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);

	/* Pasar a modo protegido, usando la GDT configurada en pm.c */
	setup_gdt();

	printf("Kernel finished \n");

}
