/*
 * pm.c
 * Este archivo contiene la variables y la implementacion de las rutinas
 * necesarias para el paso a modo protegido desde codigo en C
 */

#include <pm.h>
#include <stdio.h>
#include <stdlib.h>

/* Tabla global de descriptores (GDT). Debe estar alineado 8 bytes (Intel).
 * La GDT es un arreglo en el cual cada entrada es un descriptor de segmento.
 * El formato de los descriptores de segmento se define en el include/pm.h
 * */

segment_descriptor gdt[MAX_GDT_ENTRIES] __attribute__((aligned(8)));

/* El apuntador al GDT que se utiliza en la instruccion lgdt. */
gdt_ptr gdt_pointer;

/* Variables que almacenan los selectores a los segmentos de codigo y datos
 * del kernel. Sus valores se establecen en la rutina setup_gdt */
unsigned short kernel_code_selector;
unsigned short kernel_data_selector;


/* Tiene el siguiente formato:
 * 47           31                    0
 * +----------------------------------+
 * |  limite    |    base             |
 * +----------------------------------+
 *
 * en donde:
 * base = direccion lineal del gdt, que corresponde a la direccion
 * de memoria de la variable gdt.
 *
 * limite = tamanio de la GDT.
 * */

/*
 * Rutina: allocate_gdt_entry
 * Descripcion: Esta rutina permite obtener un indice de descriptor de segmento
 * libre dentro de la GDT, diferente al descriptor nulo.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	int : indice de la entrada disponible en el GDT
 * */
int allocate_gdt_entry(void) {
	int i;
	for (i=1; i<MAX_GDT_ENTRIES; i++) {
		if (gdt[i].present == 0) {
			gdt[i].present = 1; /* Marcar como 'presente' */
			return i;
		}
	}
	return 0;
}

/*
 * Rutina: free_gdt_descriptor
 * Descripcion: Esta rutina permite liberar un descriptor de segmento
 * de la GDT.
 * Entrada:
 * 	unsigned short : sub-indice del descriptor a liberar
 * Salida:
 * 	Ninguno.
 * */
static __inline__ void free_gdt_descriptor(int index) {

	/* Validar que no se trate de liberar el descriptor de segmento de
	 * codigo o datos del kernel */

	if (index == kernel_code_selector >> 3
			|| index == kernel_data_selector >> 3) {
		return;
	}

	/* Simplemente establecer el descritor a nulo. */
	gdt[index] = null_segment_descriptor();
}

/*
 * Rutina: setup_gdt_descriptor
 * Descripcion: Esta rutina publica permite establecer una entrada en la GDT
 * con un nuevo descriptor, diferente a la entrada nula y a las entradas para
 * los descriptores de segmento de codigo y datos para el kernel.
 * Entrada:
 * 	unsigned short: indice del descriptor a configurar
 *  segment_descriptor : Descriptor a almacenar dentro de la GDT
 * Salida:
 * 	unsigned short: selector del descriptor configurado.
 * */
 unsigned short setup_gdt_descriptor(unsigned short index,
		segment_descriptor descriptor) {
	if (index == 0  || index == (kernel_code_selector >>3)
			|| index == (kernel_data_selector >> 3)) {
		return;
	}
	gdt[index] = descriptor;
	return (index<<3 | descriptor.dpl);
}

/*
 * Rutina: setup_gdt
 * Descripcion: Esta rutina se encarga de configurar la tabla global de
 * descriptores (GDT) y cargarla mediante la instruccion LGDT. Luego pasa de
 * nuevo a modo protegido.
 * Dado a que la carga de la GDT solo se puede realizar en codigo ensamblador,
 * se utiliza inline assembly.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_gdt(void) {
	int i;

	/* Variables para verificar los valores pasados por inline assembly */
	unsigned int gdt_ptr_location;
	short cs_value, ds_value;

	printf("Setting up the GDT...\n");

	/* Primero inicializar el registro del GDT que se va a cargar mediante
	 * la directiva de ensamblador lgdt */

	/* Tamano del GDT - 1 en bytes */
	gdt_pointer.limit = sizeof(segment_descriptor)*MAX_GDT_ENTRIES - 1;

	/* Direccion lineal de 32 bits en donde se encuentra la GDT,
	 * es decir la direccion de memoria del arreglo gdt. */
	gdt_pointer.base = (unsigned int) &gdt;

	printf("Location of GDT pointer: %u\n", &gdt_pointer);
    printf("GDT base address: %u\n", gdt_pointer.base);
    printf("GDT limit: %u\n", gdt_pointer.limit);

	for (i=0; i<MAX_GDT_ENTRIES; i++) {
			free_gdt_descriptor(i);
	}


  /* Definir el selector para el descriptor de segmento de codigo del kernel */
	kernel_code_selector = KERNEL_CODE_SELECTOR;

	/* Segmento de codigo del kernel */
	gdt[kernel_code_selector >> 3] = code32_segment_descriptor(0, 0xFFFFFFFF,
			RING0_DPL);

    /* Definir el selector para el descriptor de segmento de datos del kernel */
	kernel_data_selector = KERNEL_DATA_SELECTOR;
	/* Segmento de datos del kernel. */
	gdt[kernel_data_selector >> 3] = data32_segment_descriptor(0, 0xFFFFFFFF,
			RING0_DPL);

	/*
	 * Cargar la GDT: Se realiza con Inline Assembly.
	 * Cantidad de parametros de inline assembly: 7
	 * Importante: Aunque el paso a modo protegido se puede realizar
	 *  con menos parametros, se especifican 7 para luego verificar si se
	 *  pasaron correctamente por inline assembly.
	 * Parametros de salida (para verificacion)
	 *  0 : apuntador a la localizacion de gdt_pointer en ax -> gdt_ptr_location
	 *  1 : Valor de cs en bx -> cs_value
	 *  2 : Valor de ds en cx -> ds_value
	 * Parametros de entrada
	 *  3 : Selector de segmento de codigo (inmediato)
	 *  4 : Direccion lineal en memoria de gdt_pointer -> eax
	 *  5 : kernel_code_selector -> ebx (para verificacion)
	 *  6 : kernel_data_selector -> ecx
	 *
	 *
	 *  Tenga en cuenta que un cargador compatible con la Especificacion
	 *  Multiboot ya ha activado el bit PE de CR0. No obstante, para que
	 *  el codigo sea mas consistente con los pasos sugeridos por Intel
	 *  para pasar a modo protegido, se activa de nuevo este bit.
	 *
	 * */
		inline_assembly(
					"lgdtl (%3)						 	\n\t\
					movl %%cr0, %%edx						\n\t\
					orl $1, %%edx							\n\t\
					movl %%edx, %%cr0						\n\t\
					pushl %5								\n\t\
					pushf									\n\t\
					pushl %4								\n\t\
					pushl $1f								\n\t\
					iret									\n\t\
		1:			 										\n\t\
					popl %%ecx								\n\t\
					movw %%cx, %%ds							\n\t\
					movw %%cx, %%es							\n\t\
					movw %%cx, %%fs							\n\t\
					movw %%cx, %%gs							\n\t\
					movw %%cx, %%ss							\n\t\
					"
					: "=a"(gdt_ptr_location), "=b"(cs_value), "=c"(ds_value)
					: "am"(&gdt_pointer),
					  "b"(kernel_code_selector & 0x0000FFFF),
					  "c"(kernel_data_selector & 0x0000FFFF)
					: "dx"
					);


	printf("GDT loaded successfully!\n");
	/*
	 * Ahora verificar que los operandos de salida corresponden a los
	 * operandos de entrada enviados a inline assembly
	 * */

	/* Deberia imprimir el mismo valor: la direccion de memoria de
	 * gdt_pointer */
	printf("Returned GDT pointer location: %u = %u\n", gdt_ptr_location,
														&gdt_pointer);

	/* Deberia imprimir un valor igual a kernel_code_selector */
	printf("CS = %x\n", cs_value);

	/* Deberia imprimir un valor igual a kernel_code_selector */
	printf("DS = %x\n", ds_value);
}

/*
 * Rutina: print_gdt_descriptor
 * Descripcion: Imprime un descriptor de la GDT.
 * Entrada:
 * 	int : Entrada a imprimir
 * Salida:
 * 	Ninguno.
 * */
void print_gdt_descriptor(int index) {
	/* Verificar que se va a imprimir una entrada valida */
	if (index >= MAX_GDT_ENTRIES) {
		printf("[%s] FATAL: Entry %d out of range!\n",
						__func__, index);
		while(1);
	}
	printf("Global Descriptor table. Entry %d\n", index);
	printf("limit low: %x\n", gdt[index].limit_low);
	printf("limit high: %x\n", gdt[index].limit_high);

	printf("base low: %x\n", gdt[index].base_low);
	printf("Base middle: %x\n", gdt[index].base_middle);
	printf("Base high: %x\n", gdt[index].base_high);

	printf("Type : %b\n", gdt[index].type);
	printf("Code, data or system: %u\n", gdt[index].code_or_data);
	printf("DPL: %x\n", gdt[index].dpl);
	printf("Present: %u\n", gdt[index].present);
	printf("Available: %u\n", gdt[index].available);
	printf("Reserved: %u\n", gdt[index].reserved);
	printf("D/B (big) bit: %u\n", gdt[index].big);
	printf("Granularity: %u\n", gdt[index].granularity);
	printf("Size of descriptor: %d\n", sizeof(segment_descriptor));
}
