/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * Este proyecto implementa la funcionalidad necesaria para cargar una nueva
 * tabla de descriptores de interrupcion (IDT). Esta tabla controla la gestion
 * de interrupciones en la arquitectura x86. Las interrupciones pueden ser
 * de diferentes tipos:
 * 	Excepciones ( condiciones de error del procesador )
 *  Interrupciones generadas por los dispositivos de Entrada/Salida
 *  Interrupciones por software, generadas con la instruccion ' int # ' en
 *  codigo ensamblador.
 *
 *  Este kernel define la tabla de descriptores de interrupcion (IDT) para las
 *  256 interrupciones de la arquitectura Intel, y las rutinas de servicio
 *  de interrupcion (isr) que las procesan. Vea el archivo readme.txt para
 *  mas detalles.
 *
 * Las estructuras de datos y las rutinas necesarias se definen en el archivo
 * include/idt.h y se implementan en los archivos src/idt.c y src/isr.S
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <pm.h>
#include <idt.h>

/* Manejador de prueba.
 * Esta funcion recibe el control cuando se invoca la interrupcion a la
 * cual se ha configurado para atender. */
void test_handler(interrupt_state * state) {
	printf("Interrupt Handler called! number: %d=0%x\n",state->number,state->number);
}

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */


void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	char * linea;

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Si no se encuentra memoria disponible, terminar */
	if (memory_length <= 0) {
		printf("No available memory found!\n");
		return;
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);

	/* Pasar a modo protegido, usando la GDT configurada en pm.c */
	setup_gdt();

	/* Configurar la IDT en idt.c */
	setup_idt();

	printf("Kernel started! \n");

	/* Prueba: Establecer un manejador para la interrupcion 128 (0x80), y
	 * luego invocar la interrupcion. El manejador deberia ser llamado. */

	install_interrupt_handler(0x80, test_handler);
	install_interrupt_handler(0x42, test_handler);

	/* Finalmente probar que la IDT se ha configurado correctamente,
	 * lanzando la interrupcion previamente configurada. */
	printf("Launching interrupt...\n");
	inline_assembly("int $0x80");

	/* Si existe un manejador asociado, la ejecucion deberia retornar a este
	 * punto. Lanzar de nuevo la interrupcion: */
	printf("Launching interrupt again...\n");
	inline_assembly("int $0x80");

	printf("Launching another interrupt......\n");
	inline_assembly("int $0x42");

	/* Las interrupciones no se pueden habilitar aun, ya que se generaria
	 * una excepcion cuando el timer de la arquitectura X86 lance la
	 * primera interrupcion (unos cuantos milisegundos luego de habilitar
	 * las interrupciones).  */
	//sti();

	/*
	 * Finalmente lanzar una interrupcion que no tiene manejador asociado. Esto
	 * deberia bloquear el kernel.
	 * */
	printf("Launching unhandled interrupt...\n");
	inline_assembly("int $0x0");

	/* Este mensaje no deberia aparecer...*/
	printf("Kernel finished \n");


}
