/*
 * idt.h
 * Este archivo contiene las definicion de las estructuras de datos
 * y las rutinas necesarias para configurar la tabla de descriptores de
 * interrupcion, IDT.
 */

#ifndef IDT_H_
#define IDT_H_

#include <pm.h>
#include <asm.h>
#include <stdio.h>

/* Este valor define el numero de entrada de la tabla de descriptores de
interrupcion (IDT) */

#define MAX_IDT_ENTRIES 256

/* Constante para el tipo de descriptor 'interrupt_gate' */
#define INTERRUPT_GATE_TYPE 0x0E

/*Estructura que define el estado del procesador al recibir una interrupcion
 * o una excepcion */

typedef struct interrupt_state {
	unsigned int gs, fs, es, ds;
	unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;
	unsigned int number, error_code;
	unsigned int old_eip, old_cs, old_eflags, old_esp, old_ss;
} interrupt_state;

/* Definicion de tipo para las rutinas de manejo de interrupcion */
typedef void (*interrupt_handler)(interrupt_state *);

/* Constante para definir un manejador de interrupcion vacio*/
#define NULL_INTERRUPT_HANDLER (interrupt_handler)0

/* Cada entrada de la Tabla Descriptores de Interrupcion (IDT) tiene los
  siguientes campos (Ver documento de Intel: en la seccion IDT Descriptors) */

struct idt_descriptor{
	unsigned offset_low	 : 16, /* Bits 0 .. 15 del offset */
		     selector	 : 16, /* Selector del segmento de codigo
								con la rutina*/
		     always0	 : 8, /* Reservado: 0*/
		     type	 	 : 8, /* tipo de descriptor*/
		     offset_high : 16; /* Bits 16 .. 31 del offset */
} __attribute__ ((packed));

typedef struct idt_descriptor idt_descriptor;

/* Estructura de datos para el apuntador a la IDT, para ser usada en
 * la instruccion lidt que permite cargar la IDT. */

struct idt_pointer{
	unsigned short limit;
	unsigned int base;
} __attribute__ ((packed));

typedef struct idt_pointer idt_ptr;

/*
 * Rutina: idt_descriptor_32
 * Descripcion: Esta rutina permite crear un descriptor de idt de 32 bits.
 * Entrada:
 * 	unsigned int : Selector del GDT a partir del cual se puede determinar
 * 					el segmento de codigo dentro del cual se encuentra
 * 					la rutina de manejo de interrupcion.
 * 	unsigned int:  Desplazamiento en el cual se encuentra la rutina de manejo
 * 					de interrupcion dentro del segmento especificado por el
 * 					selector.
 *  unsigned char:  Nivel de privilegios del descriptor
 *  unsigned char:  Tipo de descriptor
 * Salida:
 * 	idt_descriptor : Descriptor de idt.
 * */
static __inline__ idt_descriptor idt_descriptor_32(unsigned int selector,
		unsigned int offset, unsigned char dpl, unsigned char type) {

	  return (idt_descriptor) {
		  (offset & 0x0000FFFF),	/* offset 0..15*/
		  selector & 0x0000FFFF,					/* selector */
		  0,						/* always 0 */
		  (1<<7) | ((dpl & 0x03) << 5) | type & 0x0F, /* tipo de descriptor */
		  ((offset >> 16) & 0x0000FFFF) /* offset 16..31*/
	  };
}

/*
 * Rutina: setup_idt
 * Descripcion: Esta rutina permite configurar la Tabla de Descriptores de
 * Interrupcion (IDT).
 * Al final de la funcion se usa inline assembly para cargar el registro idtr
 * del procesador con los valores de la IDT configurada.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 *  */
void setup_idt(void);

/*
 * Rutina: cli
 * Descripcion: Permite deshabilitar las interrupciones mediante la instruccion
 * 'cli' de ensamblador.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ void cli(void) {
	inline_assembly("cli");
}

/*
 * Rutina: cli
 * Descripcion: Permite habilitar las interrupciones mediante la instruccion
 * 'sti' de ensamblador.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ void sti(void) {
	inline_assembly("sti");
}

/*
 * Rutina: sampe_dpl
 * Descripcion: Esta rutina permite determinar si dos selectores
 * se encuentran en el mismo nivel de privilegios.
 * Entrada:
 * 	unsigned short, unsigned short: Selectores a comparar.
 * Salida:
 *  int: 1 = los selectores tienen igual DPL, 0 = los selectores tienen
 *  diferente DPL.
 * */
static __inline__ int same_dpl(unsigned short sel1, unsigned short
		sel2) {
	/* Comparar los bits 0 y 1 de los selectores. Si son iguales,
	 * los selectores tienen el mismo DPL.
	 * Recuerde que el formato de un selector es:
	 * bits 0 - 1: DPL
	 * bit 2: Indicador de tabla (0 = gdt, 1 = ldt)
	 * bits 3-15 : indice del selector.
	 * Asi, al realizar la operacion 'and' con el valor 0x03, se enmascaran
	 * solo los bits correspondientes al DPL. Luego los dos valores se
	 * comparan para determinar si son iguales. */
	return ((sel1 & 0x03) == (sel2 & 0x03));
}

/*
 * Rutina: dump_interrupt_state
 * Descripcion: Esta rutina permite mostrar el estado en el cual
 * se encuentra el procesador cuando ocurrio una interrupcion.
 * Entrada:
 *  interrupt_state * : Apuntador al marco de pila de interrupcion.
 * Salida:
 * 	Ninguno.
 * typedef struct interrupt_state {
	unsigned int gs, fs, es, ds;
	unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;
	unsigned int number, error_code;
	unsigned int old_eip, old_cs, old_eflags, old_esp, old_ss;
} interrupt_state;
 * */

static __inline__ dump_interrupt_state(interrupt_state * state) {
	unsigned int current_cs;
	printf("Interrupt state: \n");
	printf("======================================\n");
	printf("gs: %x fs: %x es: %x ds: %x\n", state->gs, state->fs, state->es,
			state->ds);
	printf("edi: %u esi: %u ebp: %u esp: %u ebx: %u edx: %u ecx: %u eax: %u\n",
			state->edi, state->esi, state->ebp, state->esp, state->ebx,
			state->edx, state->ecx, state->eax);
	printf("Number=%d Error code: %d\n", state->number, state->error_code);
	printf("old eip: %u old cs: %x\n", state->old_eip, state->old_cs);

	inline_assembly("push %%cs; pop %%eax" :"=a"(current_cs)::);
	printf("Current cs: %u\n", current_cs);

	/* Verificar si ocurrio un cambio de nivel de privilegios */
	if (current_cs != state->old_cs && !same_dpl(current_cs, state->old_cs)) {
		printf("Old DPL: %d Old ss: %x Old esp: %u\n",
				state->old_cs & 0x03,
				state->old_ss,
				state->old_esp);
	}

	printf("EFLAGS: %b\n", state->old_eflags);
	printf("======================================\n");
}

/*
 * Rutina: install_interrupt_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de interrupcion
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	unsigned char: vector (numero) de interrupcion a configurar el manejador
 *  interrupt_handler : Rutina para manejar la interrupcion especificada
 * Salida:
 * 	Ninguno. */
int install_interrupt_handler(unsigned char, interrupt_handler);

/*
 * Rutina: uninstall_interrupt_handler
 * Descripcion: Esta rutina permite quitar un  manejador de interrupcion
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	unsigned char: vector (numero) de interrupcion de la cual se desea quitar su manejador
 * Salida:
 * 	Ninguno. */
void uninstall_interrupt_handler(unsigned char);

/* Macro para definir mas facilmente la referencia a las rutinas de servicio de
	interrupcion (isr)implementadas en isr.S.
	Este macro recibe como parametro el numero de isr y se expande a:
	extern void isr#(void);
	donde # es el numero de isr*/

#define ISR(id) \
 extern void isr##id(void);

ISR(0);  //"Division By Zero"
ISR(1);  //"Debug"
ISR(2);  //"Non Maskable Interrupt"
ISR(3);  //"Breakpoint"
ISR(4);  //"Into Detected Overflow"
ISR(5);  //"Out of Bounds"
ISR(6);  //"Invalid Opcode"
ISR(7);  //"No Coprocessor"
ISR(8);  //"Double Fault"
ISR(9);  //"Coprocessor Segment Overrun"
ISR(10); //"Bad TSS"
ISR(11); //"Segment Not Preent"
ISR(12); //"Stack Fault"
ISR(13); //"General Protection Fault"
ISR(14); //"Page Fault"
ISR(15); //"Unknown Interrupt"
ISR(16); //"Coprocessor Fault"
ISR(17); //"Alignment Check"
ISR(18); //"Machine Check"
ISR(19); //"Reserved"
ISR(20); //"Reserved"
ISR(21); //"Reserved"
ISR(22); //"Reserved"
ISR(23); //"Reserved"
ISR(24); //"Reserved"
ISR(25); //"Reserved"
ISR(26); //"Reserved"
ISR(27); //"Reserved"
ISR(28); //"Reserved"
ISR(29); //"Reserved"
ISR(30); //"Reserved"
ISR(31); //"Reserved"
ISR(32)
ISR(33)
ISR(34)
ISR(35)
ISR(36)
ISR(37)
ISR(38)
ISR(39)
ISR(40)
ISR(41)
ISR(42)
ISR(43)
ISR(44)
ISR(45)
ISR(46)
ISR(47)
ISR(48)
ISR(49)
ISR(50)
ISR(51)
ISR(52)
ISR(53)
ISR(54)
ISR(55)
ISR(56)
ISR(57)
ISR(58)
ISR(59)
ISR(60)
ISR(61)
ISR(62)
ISR(63)
ISR(64)
ISR(65)
ISR(66)
ISR(67)
ISR(68)
ISR(69)
ISR(70)
ISR(71)
ISR(72)
ISR(73)
ISR(74)
ISR(75)
ISR(76)
ISR(77)
ISR(78)
ISR(79)
ISR(80)
ISR(81)
ISR(82)
ISR(83)
ISR(84)
ISR(85)
ISR(86)
ISR(87)
ISR(88)
ISR(89)
ISR(90)
ISR(91)
ISR(92)
ISR(93)
ISR(94)
ISR(95)
ISR(96)
ISR(97)
ISR(98)
ISR(99)
ISR(100)
ISR(101)
ISR(102)
ISR(103)
ISR(104)
ISR(105)
ISR(106)
ISR(107)
ISR(108)
ISR(109)
ISR(110)
ISR(111)
ISR(112)
ISR(113)
ISR(114)
ISR(115)
ISR(116)
ISR(117)
ISR(118)
ISR(119)
ISR(120)
ISR(121)
ISR(122)
ISR(123)
ISR(124)
ISR(125)
ISR(126)
ISR(127)
ISR(128)
ISR(129)
ISR(130)
ISR(131)
ISR(132)
ISR(133)
ISR(134)
ISR(135)
ISR(136)
ISR(137)
ISR(138)
ISR(139)
ISR(140)
ISR(141)
ISR(142)
ISR(143)
ISR(144)
ISR(145)
ISR(146)
ISR(147)
ISR(148)
ISR(149)
ISR(150)
ISR(151)
ISR(152)
ISR(153)
ISR(154)
ISR(155)
ISR(156)
ISR(157)
ISR(158)
ISR(159)
ISR(160)
ISR(161)
ISR(162)
ISR(163)
ISR(164)
ISR(165)
ISR(166)
ISR(167)
ISR(168)
ISR(169)
ISR(170)
ISR(171)
ISR(172)
ISR(173)
ISR(174)
ISR(175)
ISR(176)
ISR(177)
ISR(178)
ISR(179)
ISR(180)
ISR(181)
ISR(182)
ISR(183)
ISR(184)
ISR(185)
ISR(186)
ISR(187)
ISR(188)
ISR(189)
ISR(190)
ISR(191)
ISR(192)
ISR(193)
ISR(194)
ISR(195)
ISR(196)
ISR(197)
ISR(198)
ISR(199)
ISR(200)
ISR(201)
ISR(202)
ISR(203)
ISR(204)
ISR(205)
ISR(206)
ISR(207)
ISR(208)
ISR(209)
ISR(210)
ISR(211)
ISR(212)
ISR(213)
ISR(214)
ISR(215)
ISR(216)
ISR(217)
ISR(218)
ISR(219)
ISR(220)
ISR(221)
ISR(222)
ISR(223)
ISR(224)
ISR(225)
ISR(226)
ISR(227)
ISR(228)
ISR(229)
ISR(230)
ISR(231)
ISR(232)
ISR(233)
ISR(234)
ISR(235)
ISR(236)
ISR(237)
ISR(238)
ISR(239)
ISR(240)
ISR(241)
ISR(242)
ISR(243)
ISR(244)
ISR(245)
ISR(246)
ISR(247)
ISR(248)
ISR(249)
ISR(250)
ISR(251)
ISR(252)
ISR(253)
ISR(254)
ISR(255)

#endif /* IDT_H_ */

