/*
 * tss.h
 * Este archivo contiene las definiciones de las estructuras de datos y las
 * rutinas asociadas con la gestion de segmentos de estado de tarea.
 */

#ifndef TSS_H_
#define TSS_H_

/* Task State Segment
 * Descripcion: Esta estructura de datos representa el estado de una tarea
 * en la arquitectura x86. Consulte el manual de Intel IA-32 Volume 3
 * Systems Programming para mas detalles. */
typedef struct tss {
	unsigned short link;	/* Previous task link */
	unsigned short padding_link;	/* Espacio reservado luego de link */
	unsigned int esp0;	/* %esp para el nivel de privilegio 0 */
	unsigned short ss0;	/* %ss para el nivel de privilegio 0 */
	unsigned short padding_ss0; /* Espacio reservado luego de ss0 */
	unsigned int esp1;	/* %esp para el nivel de privilegio 1 */
	unsigned short ss1;	/* %ss para el nivel de privilegio 1 */
	unsigned short padding_ss1; /* Espacio reservado luego de ss1 */
	unsigned int esp2;	/* %esp para el nivel de privilegio 2 */
	unsigned short ss2;	/* %ss para el nivel de privilegio 2 */
	unsigned short padding_ss2; /* Espacio reservado luego de ss2 */
	unsigned int cr3;	/* %cr3  */
	unsigned int eip;	/* %eip */
	unsigned int eflags; /* EFLAGS */
	unsigned int eax;	/* %eax */
	unsigned int ecx;	/* %ecx */
	unsigned int edx;	/* %edx */
	unsigned int ebx;	/* %ebx */
	unsigned int esp;	/* %esp */
	unsigned int ebp;	/* %ebp */
	unsigned int esi;	/* %esi */
	unsigned int edi;	/* %edi */
	unsigned short es;	/* %es */
	unsigned short es_padding; /* Espacio reservado luego de es */
	unsigned short cs;	/* %cs */
	unsigned short cs_padding;	/* Espacio reservado luego de cs */
	unsigned short ss;	/* %ss */
	unsigned short ss_padding; /* Espacio reservado luego de ss */
	unsigned short ds;	/* %ds */
	unsigned short ds_padding;	/* Espacio reservado luego de ds */
	unsigned short fs;	/* %fs */
	unsigned short fs_padding; /* Espacio reservado luego de fs */
	unsigned short gs;	/* %gs */
	unsigned short gs_padding;	/* Espacio reservado luego de gs */
	unsigned short ldt_selector; /* Selector del ldt */
	unsigned short ldt_selector_padding; /* Espacio reservado*/
	unsigned short t; /* T + espacio reservado */
	unsigned short iomap_base; /* I/O map base address */
}tss;

#define NULL_TSS  \
	(tss){ \
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 \
}


#define BUSY_TASK 0x0B
#define AVAILABLE_TASK 0x09

/* Rutina: tss_descriptor_32
 * Descripcion: Esta rutina permite configurar un descriptor de TSS de 32 bits
 * Entrada:
 * 	tss * : Apuntador al TSS
 *  unsigned char: Nivel de privilegios
 * Salida:
 *  segment_descriptor : Descriptor de segmento. */
static __inline__ segment_descriptor tss_descriptor_32(
		tss * ptr,
		unsigned char dpl) {

		unsigned int base;
		unsigned int limit;

		base = (unsigned int)ptr;
		limit = sizeof(tss) - 1;

		return (segment_descriptor) {
			(limit & 0x0000FFFF),	/* limite 0..15 */
			(base & 0x0000FFFF), 	/* base 0..15 */
			(base >> 16 & 0xFF), 	/* base 16..23 */
			AVAILABLE_TASK,	 	/* Informacion de acceso */
			0,						/*  Code or data = 0, segmento de sistema */
			(dpl & 0x03), 			/* Nivel de privilegio */
			1, 						/* Presente */
			(limit >> 16 & 0x0F),	/* limite 16..19*/
			0,						/* Disponible */
			0, 						/* Reservado */
			0, 						/* 0 = 16 bits, 1 = 32 bits */
			0, 						/* 0 = bytes, 1 = 4096 bytes */
			(base >> 24 & 0xFF) 	/* base 24..31*/
		};
}

/*
 * Rutina: setup_tss
 * Descripcion: Esta rutina establece el registro de estado de tarea (TSS) para
 * el kernel, y pasa el control a la primera tarea del kernel (la rutina
 * Entrada:
 * 	Ninguno.
 * Salida:
 *  Ninguno.
 * */
void setup_tss(void);

#endif /* TSS_H_ */
