/*
 * idt.c
 *Este archivo contiene la implementacion de las rutinas necesarias para
 *configurar la tabla de descriptores de interrupcion, IDT.
 * Para ello, se configuran todas las entradas de la IDT como descriptores
 * de tipo INTERRUPT_GATE, que ejecutan la rutina isr0..isr255 correspondiente
 * que se encuentra en el segmento de codigo del kernel (ver archivo irs.S).
 *
 * De esta forma, cuando ocurre una interrupcion, se realiza el siguiente
 * proceso:
 *
 * Si la tarea que se estaba ejecutando se encontraba en un nivel
 * de privilegios diferentes, el procesador almacena en la pila
 * los valores de SS y ESP.
 * El procesador almacena en la pila EFLAGS, CS y EIP.
 * Luego el procesador obtiene el vector (numero) de interrupcion. Con este
 * numero, busca la entrada correspondiente en la IDT, la cual
 * apunta a una de las rutinas isr0 ... isr 255 configuradas previamente.
 * Las rutinas isr0 .. isr255 a su vez establecen un marco de pila
 * uniforme (ver isr.S) e invocan a la rutina interrupt_dispatcher.
 * La rutina interrupt_dispatcher se encarga de verificar si existe
 * una rutina de manejo de interrupcion y ejecutarla.
 * La rutina de manejo de interrupcion termina, y le regresa el control
 * a la rutina interrupt_dispatcher.
 * La rutina interrupt_dispatcher a su vez termina y retorna a la rutina
 * isr0 ... isr255 correspondiente.
 * La rutina isr0 ... isr255 recupera el estado del procesador al
 * estado en que se encontraba justo cuando ocurrio la interrupcion.
 * Finalmente, la rutina isr0 ... isr255 ejecuta la instruccion 'iret', para
 * retornar a la tarea que fue interrumpida.
 * El procesador se encarga automaticamente de reestablecer CS, EIP y EFLAGS
 * a partir de los valores almacenados en la pila cuando ocurrio la
 * inteerrupcion. Asi, la ejecucion de la tarea continua en el mismo punto en
 * el cual se encontraba en el momento de ocurrir la interrupcion. */

#include <idt.h>
#include <stdio.h>
#include <stdlib.h>

/* Tabla de descriptores de interrupcion (IDT) */
idt_descriptor idt[MAX_IDT_ENTRIES] __attribute__((aligned(8)));

/* El apuntador al IDT que se utiliza en la instruccion lidt */
idt_ptr idt_pointer;

/*
 * Rutina: interrupt_dispatcher
 * Descripcion: Esta rutina recibe el control de la Rutina de Servicio
 * de Interrupcion (ISR) correspondiente. Su trabajo consiste en determinar
 * el vector de interrupcion a partir del estado que recibe como parametro,
 * y de invocar la rutina de manejo de interrupcion adecuada, si existe.
 * Entrada:
 * Ninguno.
 * Salida:
 *	Ninguno.
 * */
void interrupt_dispatcher();

/*
 * Rutina: install_isr
 * Descripcion: Funcion para instalar las rutinas de servicio de interrupcion,
 * isr. Las ISR son rutinas en ensamblador que salvan el estado del procesador
 * y configuran los registros de segmento necesarios para que se
 * invoque la rutina interrupt_dispatcher, quien a su vez invoca a la rutina
 * de manejo de interrupcion correspondiente..
 * Entrada:
 * 	Ninguno.
 *
 * Salida:
 * 	Ninguno.
 * */
void install_isr(void);

/* Arreglo que almacena las referencias a las rutinas de manejo de interrupcion
 * La rutina install_interrupt_handler almacena las referencias a las rutinas
 * en este arreglo. */
interrupt_handler interrupt_handlers[MAX_IDT_ENTRIES];

/*
 * Rutina: setup_idt
 * Descripcion: Esta rutina permite configurar la Tabla de Descriptores de
 * Interrupcion (IDT).
 * Al final de la funcion se usa inline assembly para cargar el registro idtr
 * del procesador con los valores de la IDT configurada.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 *  */
void setup_idt(void) {
	int i;

	/* Configurar la base y el limite de la estructura idt_pointer
	 * que se utilizara en la instruccion lidt. */
	idt_pointer.limit = (sizeof(idt_descriptor) * MAX_IDT_ENTRIES);
	idt_pointer.base = (unsigned int) &idt;

	install_isr();

	/* Configurar las Rutinas de Servicio de Interrupcion (ISR)
	 para las excepciones x86 (las primeras 32 entradas de la IDT)
	 */

	//printf("Loading the IDT...");

	/* Y finalmente cargar la IDT con inline assembly */
	inline_assembly("lidt (%0)"
			:
			:"a"(&idt_pointer)
			:);

	//printf("OK\n");

}

/*
 * Rutina: interrupt_dispatcher
 * Descripcion: Esta rutina recibe el control de la Rutina de Servicio
 * de Interrupcion (ISR) correspondiente. Su trabajo consiste en determinar
 * el vector de interrupcion a partir del estado que recibe como parametro,
 * y de invocar la rutina de manejo de interrupcion adecuada, si existe.
 * Entrada:
 * 	Ninguno.
 * Salida:
 *	Ninguno.
 * */
void interrupt_dispatcher() {

	/* Esta variable definida en isr.S contiene el valor del apuntador
	 * actual al tope de la pila (esp)*/
	extern char * current_esp;

	interrupt_state * state;

	state = (interrupt_state *)current_esp;

	interrupt_handler handler;

	/* Buscar la rutina que maneja la interrupcion */
	handler = interrupt_handlers[state->number];

	/* Si la rutina existe, ejecutarla y pasarle como parametro los registros.*/
	if (handler != NULL_INTERRUPT_HANDLER) {
		handler(state);
	} else {
		/* En caso contrario, informar que ocurrio una interrupcion
		 * que no tiene un manejador asociado.*/
		if (state->number < 32) { /* Excepcion*/
			printf("x86 Exception [%d]. System Halted!\n", state->number);
		} else { /* Interrupcion */
			printf("Unhandled interrupt [%d]. System halted.", state->number);
		}
		/* Mostrar el estado de la interrupcion. */
		dump_interrupt_state(state);

		/* Bloquear el kernel cuando ocurre una excepcion o una
		 * interrupcion que no tiene manejador asociado.
		 * Recuerde que esta rutina se esta ejecutando con las interrupciones
		 * deshabilitadas, por lo cual no existe forma de romper el
		 * ciclo infinito que se define a continuacion.*/
		for (;;)
			;
	}
}

/*
 * Rutina: install_interrupt_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de interrupcion
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	unsigned char: vector (numero) de interrupcion a configurar el manejador
 *  interrupt_handler : Rutina para manejar la interrupcion especificada
 * Salida:
 * 	Ninguno. */
int install_interrupt_handler(unsigned char i, interrupt_handler handler) {
	if (interrupt_handlers[i] != NULL_INTERRUPT_HANDLER) {
		printf("Error! handler for routine %d was already set!\n", i);
		return -1;
	}
	interrupt_handlers[i] = handler;
}

/*
 * Rutina: uninstall_interrupt_handler
 * Descripcion: Esta rutina permite quitar un  manejador de interrupcion
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	unsigned char: vector (numero) de interrupcion de la cual se desea quitar su manejador
 * Salida:
 * 	Ninguno. */
void uninstall_interrupt_handler(unsigned char i) {
	/* Simplemente quitar la referencia a la rutina de manejo de interrupcion.*/
	if (i < MAX_IDT_ENTRIES) {
		interrupt_handlers[i] = NULL_INTERRUPT_HANDLER;
	}

}

/*
 * Rutina: install_isr
 * Descripcion: Funcion para instalar las rutinas de servicio de interrupcion,
 * isr. Las ISR son rutinas en ensamblador que salvan el estado del procesador
 * y configuran los registros de segmento necesarios para que se
 * invoque la rutina interrupt_dispatcher, quien a su vez invoca a la rutina
 * de manejo de interrupcion correspondiente.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void install_isr(void) {
	int i;

	/* Primero inicializar el arreglo de referencias a las rutinas de manejo
	 * de interrupcion */
	for (i = 0; i < MAX_IDT_ENTRIES; i++) {
		interrupt_handlers[i] = NULL_INTERRUPT_HANDLER;
	}

	/* Ahora configurar los 255 descriptores de la IDT para que apunten
	 * a las rutinas isr0 .. isr255*/
	idt[0] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr0,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[1] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr1,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[2] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr2,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[3] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr3,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[4] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr4,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[5] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr5,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[6] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr6,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[7] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr7,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[8] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr8,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[9] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr9,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[10] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr10,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[11] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr11,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[12] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr12,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[13] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr13,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[14] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr14,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[15] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr15,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[16] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr16,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[17] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr17,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[18] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr18,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[19] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr19,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[20] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr20,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[21] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr21,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[22] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr22,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[23] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr23,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[24] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr24,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[25] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr25,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[26] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr26,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[27] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr27,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[28] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr28,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[29] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr29,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[30] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr30,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[31] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr31,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[32] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr32,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[33] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr33,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[34] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr34,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[35] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr35,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[36] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr36,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[37] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr37,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[38] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr38,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[39] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr39,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[40] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr40,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[41] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr41,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[42] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr42,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[43] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr43,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[44] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr44,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[45] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr45,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[46] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr46,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[47] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr47,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[48] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr48,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[49] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr49,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[50] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr50,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[51] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr51,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[52] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr52,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[53] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr53,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[54] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr54,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[55] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr55,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[56] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr56,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[57] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr57,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[58] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr58,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[59] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr59,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[60] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr60,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[61] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr61,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[62] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr62,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[63] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr63,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[64] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr64,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[65] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr65,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[66] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr66,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[67] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr67,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[68] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr68,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[69] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr69,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[70] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr70,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[71] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr71,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[72] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr72,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[73] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr73,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[74] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr74,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[75] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr75,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[76] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr76,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[77] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr77,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[78] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr78,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[79] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr79,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[80] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr80,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[81] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr81,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[82] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr82,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[83] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr83,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[84] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr84,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[85] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr85,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[86] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr86,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[87] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr87,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[88] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr88,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[89] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr89,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[90] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr90,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[91] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr91,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[92] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr92,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[93] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr93,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[94] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr94,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[95] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr95,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[96] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr96,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[97] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr97,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[98] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr98,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[99] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr99,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[100] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr100,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[101] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr101,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[102] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr102,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[103] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr103,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[104] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr104,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[105] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr105,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[106] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr106,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[107] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr107,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[108] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr108,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[109] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr109,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[110] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr110,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[111] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr111,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[112] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr112,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[113] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr113,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[114] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr114,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[115] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr115,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[116] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr116,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[117] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr117,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[118] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr118,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[119] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr119,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[120] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr120,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[121] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr121,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[122] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr122,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[123] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr123,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[124] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr124,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[125] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr125,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[126] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr126,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[127] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr127,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[128] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr128,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[129] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr129,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[130] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr130,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[131] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr131,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[132] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr132,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[133] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr133,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[134] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr134,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[135] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr135,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[136] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr136,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[137] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr137,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[138] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr138,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[139] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr139,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[140] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr140,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[141] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr141,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[142] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr142,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[143] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr143,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[144] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr144,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[145] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr145,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[146] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr146,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[147] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr147,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[148] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr148,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[149] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr149,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[150] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr150,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[151] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr151,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[152] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr152,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[153] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr153,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[154] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr154,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[155] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr155,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[156] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr156,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[157] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr157,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[158] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr158,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[159] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr159,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[160] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr160,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[161] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr161,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[162] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr162,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[163] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr163,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[164] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr164,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[165] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr165,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[166] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr166,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[167] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr167,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[168] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr168,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[169] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr169,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[170] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr170,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[171] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr171,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[172] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr172,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[173] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr173,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[174] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr174,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[175] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr175,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[176] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr176,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[177] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr177,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[178] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr178,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[179] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr179,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[180] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr180,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[181] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr181,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[182] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr182,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[183] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr183,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[184] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr184,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[185] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr185,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[186] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr186,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[187] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr187,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[188] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr188,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[189] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr189,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[190] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr190,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[191] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr191,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[192] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr192,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[193] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr193,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[194] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr194,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[195] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr195,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[196] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr196,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[197] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr197,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[198] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr198,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[199] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr199,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[200] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr200,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[201] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr201,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[202] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr202,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[203] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr203,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[204] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr204,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[205] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr205,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[206] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr206,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[207] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr207,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[208] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr208,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[209] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr209,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[210] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr210,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[211] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr211,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[212] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr212,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[213] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr213,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[214] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr214,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[215] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr215,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[216] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr216,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[217] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr217,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[218] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr218,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[219] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr219,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[220] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr220,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[221] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr221,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[222] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr222,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[223] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr223,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[224] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr224,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[225] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr225,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[226] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr226,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[227] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr227,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[228] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr228,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[229] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr229,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[230] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr230,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[231] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr231,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[232] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr232,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[233] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr233,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[234] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr234,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[235] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr235,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[236] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr236,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[237] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr237,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[238] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr238,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[239] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr239,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[240] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr240,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[241] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr241,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[242] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr242,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[243] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr243,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[244] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr244,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[245] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr245,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[246] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr246,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[247] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr247,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[248] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr248,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[249] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr249,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[250] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr250,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[251] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr251,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[252] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr252,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[253] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr253,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[254] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr254,
			RING0_DPL, INTERRUPT_GATE_TYPE);
	idt[255] = idt_descriptor_32(KERNEL_CODE_SELECTOR, (unsigned int) isr255,
			RING0_DPL, INTERRUPT_GATE_TYPE);

}
