/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * Este proyecto implementa la funcionalidad necesaria para configurar el
 * segmento de estado de tarea (Task State Segment, TSS). Esta estructura de
 * datos es requerida por el procesador para implementar el manejo de tareas.
 *
 * Vea el archivo readme.txt para mas detalles.
 *
 * Las estructuras de datos y las rutinas necesarias se definen en el archivo
 * include/tss.h y se implementan en el archivo task.c
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <pm.h>
#include <idt.h>
#include <tss.h>

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */


void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	char * linea;

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Si no se encuentra memoria disponible, terminar */
	if (memory_length <= 0) {
		printf("No available memory found!\n");
		return;
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);

	/* Pasar a modo protegido, usando la GDT configurada en pm.c */
	setup_gdt();

	/* Configurar la IDT en idt.c */
	setup_idt();

	/* Ahora configurar el TSS. El TSS es un segmento que se utiliza para
	 * realizar el cambio de contexto de ejecucion de tareas.
	 * En este proyecto simplemente se crea un TSS vacio. Posteriormente
	 * se implementara la logica para utilizarlo.
	 * */

	setup_tss();

	/* Imprimir la tercera entrada de la GDT. Dentro de su campo 'tipo',
	 * debera estar marcada como 'ocupada' (1011), ya que esta tarea
	 * se encuentra ejecutandose.*/

	print_gdt_descriptor(3);

	/* Este mensaje nunca deberia aparecer, ya que se entra en un ciclo
	* infinito.*/
	printf("Kernel finished\n");

}

