/*
 * idt.h
 * Este archivo contiene las definicion de las estructuras de datos
 * y las rutinas necesarias para configurar la tabla de descriptores de
 * interrupcion, IDT.
 */

#ifndef IDT_H_
#define IDT_H_

#include <pm.h>

/* Este valor define el numero de entrada de la tabla de descriptores de
interrupcion (IDT) */

#define MAX_IDT_ENTRIES 255

/*Estructura que define el estado del procesador al recibir una interrupcion
 * o una excepcion */

struct interrupt_state {
	unsigned int gs, fs, es, ds;
	unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;
	unsigned int number, error_code;
	unsigned int eip, cs, eflags, old_esp, old_ss;
};


/* Cada entrada de la Tabla Descriptores de Interrupcion (IDT) tiene los
  siguientes campos (Ver documento de Intel: en la seccion IDT Descriptors) */

struct idt_descriptor_t{
	unsigned int offset_low	 : 16, /* Bits 0 .. 16 del offset */
		     selector	 : 16, /* Selector del segmento de codigo
					con la rutina*/
		     always0	 : 8, /* Reservado: 0*/
		     flags	 : 8, /* FLAGS (interrupt gate: 0x8E)*/
		     offset_high : 16; /* Bits 17 .. 31 del offset */
} __attribute__ ((packed));

typedef struct idt_descriptor_t idt_descriptor;

struct idt_pointer_t{
	unsigned short limit;
	unsigned int base;
} __attribute__ ((packed));

typedef struct idt_pointer_t idt_ptr;

/*
 * Rutina: setup_idt
 * Descripcion: Esta rutina permite configurar la Tabla de Descriptores de
 * Interrupcion (IDT).
 * Al final de la funcion se usa inline assembly para cargar el registro idtr
 * del procesador con los valores de la IDT configurada.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 *  */
void setup_idt(void);

/*
 * Rutina: setup_interrupt_gate
 * Descripcion: Funcion para configurar un descriptor de interrupcion dentro
 * de la IDT.
 * Entrada:
 * 	int 	: indice del descriptor en la IDT a configurar
 *  unsigned int : Direccion de memoria de la rutina de servicio de
 *  				interrupcion (isr)
 *  unsigned short : Selector en el GDT en el cual se describe el
 *  				segmento de codigo dentro del cual se encuentra la isr
 *  flags	: Para interrupt gate, flags siempre es 0x8E
 * Salida:
 * 	Ninguno.
 * */
void setup_interrupt_gate(int index, unsigned int offset,
	unsigned short selector, unsigned char flags);


/* Alias en C de las rutinas declaradas en codigo ensamblador*/
extern void cli(void);
extern void sti(void);

#endif /* IDT_H_ */

