/*
 * pm.h
 * Este archivo contiene las definicion de la GDT y las rutinas para su
 * manejo.
 * La GDT es la principal estructura de datos que controla la ejecucion de
 * un procesador x86 en modo protegido. La GDT contiene una serie de entradas,
 * llamadas descriptores de segmentos. Cada descriptor de segmento define
 * las direcciones base y limite de un segmento, su tipo y sus opciones de
 * acceso.
 */

#ifndef PM_H_
#define PM_H_

/* Definiciones para la Tabla Global de Descriptores (GDT) */

/*Este valor define el numero de entradas de la GDT */
#define MAX_GDT_ENTRIES  1024

/* Tipo de segmento de codigo:  = 1010 : Code, Non-Conformant,  Readable,
 * Not Accessed*/
#define CODE_SEGMENT_TYPE 0x0A

/* Tipo de segmento de datos:  = 0011 :Data, Read / Write, Accessed */
#define DATA_SEGMENT_TYPE 0x03

/* Desplazamiento en el GDT en el cual se encuentra el selector del segmento de
codigo del kernel */
#define KERNEL_CODE_SELECTOR 0x08

/* Desplazamiento en el GDT en el cual se encuentra el selector del segmento de
datos del kernel */
#define KERNEL_DATA_SELECTOR 0x10

/* Definir los niveles de privilegio para los segmentos */
#define RING0_DPL 0
#define RING1_DPL 1
#define RING2_DPL 2
#define RING3_DPL 3

/* Dado que este archivo puede ser incluido desde codigo en Assembler, incluir
 * solo las constantes definidas anteriormente. */

#ifndef ASM

/* Cada entrada de la Tabla Global de Descriptores (GDT) tiene los siguientes
 * campos (Ver documento de Intel: en la seccion General Segment Descriptor
 * Format) */

 struct segment_descriptor {
	unsigned limit_low			: 16, /* limite 0..15*/
		     base_low			: 16, /* base 0..16 */
		     base_middle		: 8,  /* base 16..23*/
		     type 				: 4,  /* Informacion de acceso */
		     code_or_data		: 1, /* 1 = codigo o datos, 0 = sistema */
		     dpl				: 2, /* Nivel de privilegio */
		     present			: 1, /* Presente */
		     limit_high			: 4, /* limite 16..19*/
		     available			: 1, /* Disponible */
		     reserved			: 1, /* Reservado */
		     big				: 1, /* 0 = 16 bits, 1 = 32 bits */
		     granularity		: 1, /* 0 = bytes, 1 = 4096 bytes */
		     base_high			: 8; /* base 24..31*/
}__attribute__((packed)); /* Evitar posible alineacion del compilador */

typedef struct segment_descriptor segment_descriptor;

/* Estructura de datos para el registro GDTR (puntero a la GDT) */
struct gdt_pointer_t {
	unsigned short limit; /* Tamanio de la GDT */
	unsigned int base; /* Direccion lineal del inicio de la GDT */
} __attribute__ ((packed));

typedef struct gdt_pointer_t gdt_ptr;

/*
 * Rutina: setup_gdt
 * Descripcion: Esta rutina se encarga de configurar la tabla global de
 * descriptores (GDT) y cargarla mediante la instruccion LGDT. Luego pasa de
 * nuevo a modo protegido.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_gdt(void);

/*
 * Rutina: get_gdt_free_descriptor
 * Descripcion: Esta rutina permite obtener un descriptor de segmento
 * libre dentro de la GDT, diferente al descriptor nulo.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	segment_descriptor * : Apuntador al descriptor de segmento obtenido.
 * */
segment_descriptor * get_gdt_free_descriptor(void);

/* Rutina: code32_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor de segmento de
 * codigo de 32 bits.
 * Entrada:
 * 	unsigned int: Base del segmento
 *  unsigned int: Limite del segmento
 *  unsigned char: Nivel de privilegios
 * Salida:
 *  segment_descriptor : Descriptor de segmento. */
static __inline__ segment_descriptor code32_segment_descriptor(
		unsigned int base,
		unsigned int limit,
		unsigned char dpl) {

		return (segment_descriptor) {
			(limit & 0x0000FFFF),	/* limite 0..15 */
			(base & 0x0000FFFF), 	/* base 0..15 */
			(base >> 16 & 0xFF), 	/* base 16..23 */
			CODE_SEGMENT_TYPE,	 	/* Informacion de acceso */
			1,						/*  Code or data */
			dpl & 0x03, 			/* Nivel de privilegio */
			1, 						/* Presente */
			(limit >> 16 & 0x0F),	/* limite 16..19*/
			0,						/* Disponible */
			0, 						/* Reservado */
			1, 						/* 0 = 16 bits, 1 = 32 bits */
			1, 						/* 0 = bytes, 1 = 4096 bytes */
			(base >> 24 & 0xFF) 	/* base 24..31*/
		};
}

/* Rutina: data32_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor de segmento de
 * datos de 32 bits.
 * Entrada:
 * 	unsigned int: Base del segmento
 *  unsigned int: Limite del segmento
 *  unsigned char: Nivel de privilegios
 * Salida:
 *  segment_descriptor : Descriptor de segmento. */
static __inline__ segment_descriptor data32_segment_descriptor(
		unsigned int base,
		unsigned int limit,
		unsigned char dpl) {

		return (segment_descriptor) {
			(limit & 0x0000FFFF),	/* limite 0..15 */
			(base & 0x0000FFFF), 	/* base 0..15 */
			(base >> 16 & 0xFF), 	/* base 16..23 */
			DATA_SEGMENT_TYPE,	 	/* Informacion de acceso */
			1,						/*  Code or data */
			dpl & 0x03, 			/* Nivel de privilegio */
			1, 						/* Presente */
			(limit >> 16 & 0x0F),	/* limite 16..19*/
			0,						/* Disponible */
			0, 						/* Reservado */
			1, 						/* 0 = 16 bits, 1 = 32 bits */
			1, 						/* 0 = bytes, 1 = 4096 bytes */
			(base >> 24 & 0xFF) 	/* base 24..31*/
		};
}

/* Rutina: null_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor desegmento nulo.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ segment_descriptor null_segment_descriptor(void) {

		return (segment_descriptor) {
			0,	/* limite 0..15 */
			0, 	/* base 0..15 */
			0, 	/* base 16..23 */
			0,	/* Informacion de acceso */
			0,	/*  Code or data */
			0, 	/* Nivel de privilegio */
			0, 	/* Presente */
			0,	/* limite 16..19*/
			0,	/* Disponible */
			0, 	/* Reservado */
			0, 	/* 0 = 16 bits, 1 = 32 bits */
			0, 	/* 0 = bytes, 1 = 4096 bytes */
			0 	/* base 24..31*/
		};
}

/*
 * Rutina: setup_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor de segmento dentro
 * de la tabla global de descriptores, GDT
 * Entrada:
 * 	int 			: Entrada de la GDT que se desea configurar
 *	unsigned int	: Base del segmento
 *	unsigned int	: Limite del segmento
 *	unsigned char	: Tipo de descriptor
 *	unsigned char	: Tipo del segmento
 *	unsigned char	: Nivel de privilegio del segmento
 * Salida:
 * 	Ninguno.
 * En caso de error, imprime un mensaje y bloquea el kernel.
 *
 * */
void setup_gdt_descriptor(int ,
			unsigned int,
			unsigned int,
			unsigned char,
			unsigned char);

/*
 * Rutina: setup_null_descriptor
 * Descripcion: Esta rutina permite establecer en nulo un descriptor de segmento
 *  dentro de la tabla global de descriptores, GDT
 * Entrada:
 * 	int 			: Entrada de la GDT que se desea establecer en nulo
 * Salida:
 * 	Ninguno.
 * En caso de error, imprime un mensaje y bloquea el kernel.
 *
 * */
void setup_null_descriptor(int);

/*
 * Rutina: print_gdt_descriptor
 * Descripcion: Imprime un descriptor de la GDT.
 * Entrada:
 * 	int : Entrada a imprimir
 * Salida:
 * 	Ninguno.
 * */
void print_gdt_descriptor(int );

#endif /* ASM */

#endif /* PM_H_ */
