/*
 * idt.c
 *
 *Este archivo contiene la implementacion de las rutinas necesarias para
 *configurar la tabla de descriptores de interrupcion, IDT.
 */

#include <idt.h>
#include <stdio.h>
#include <stdlib.h>

/* Tabla de descriptores de interrupcion  (IDT) */
idt_descriptor idt[MAX_IDT_ENTRIES] __attribute__((aligned(8)));

/* El apuntador al IDT que se utiliza en la instruccion lidt */
idt_ptr idt_pointer;

/*
 * Rutina: fault_handler
 * Descripcion: Esta rutina recibe el control de la Rutina de Servicio
 * de Interrupcion (ISR) correspondiente.
 * Entrada:
 *  struct interrupt_state * : Estructura de datos que contiene
 *  el estado de los registros del procesador
 * Salida:
 *	Ninguno.
 * */
void fault_handler(struct interrupt_state);

/* Excepciones dele procesador Intel*/
unsigned char *exceptions[] = {
	"Division By Zero",
	"Debug",
	"Non Maskable Interrupt",
	"Breakpoint",
	"Into Detected Overflow",
	"Out of Bounds",
	"Invalid Opcode",
	"No Coprocessor",
	"Double Fault",
	"Coprocessor Segment Overrun",
	"Bad TSS",
	"Segment Not Present",
	"Stack Fault",
	"General Protection Fault",
	"Page Fault",
	"Unknown Interrupt",
	"Coprocessor Fault",
	"Alignment Check",
	"Machine Check",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved"
};

/* Macro para definir mas facilmente la referencia a las rutinas de servicio de
	interrupcion (isr) para las 32 excepciones x86  implementadas en
	idt.S.
	Este macro recibe como parametro el numero de isr y se expande a:

	extern void isr#(void);

	donde # es el numero de isr*/

#define ISR(id) \
 extern void isr##id(void);

ISR(0);  //"Division By Zero"
ISR(1);  //"Debug"
ISR(2);  //"Non Maskable Interrupt"
ISR(3);  //"Breakpoint"
ISR(4);  //"Into Detected Overflow"
ISR(5);  //"Out of Bounds"
ISR(6);  //"Invalid Opcode"
ISR(7);  //"No Coprocessor"
ISR(8);  //"Double Fault"
ISR(9);  //"Coprocessor Segment Overrun"
ISR(10); //"Bad TSS"
ISR(11); //"Segment Not Preent"
ISR(12); //"Stack Fault"
ISR(13); //"General Protection Fault"
ISR(14); //"Page Fault"
ISR(15); //"Unknown Interrupt"
ISR(16); //"Coprocessor Fault"
ISR(17); //"Alignment Check"
ISR(18); //"Machine Check"
ISR(19); //"Reserved"
ISR(20); //"Reserved"
ISR(21); //"Reserved"
ISR(22); //"Reserved"
ISR(23); //"Reserved"
ISR(24); //"Reserved"
ISR(25); //"Reserved"
ISR(26); //"Reserved"
ISR(27); //"Reserved"
ISR(28); //"Reserved"
ISR(29); //"Reserved"
ISR(30); //"Reserved"
ISR(31); //"Reserved"

/*
 * Rutina: install_isr
 * Descripcion: Funcion para instalar las rutinas de servicio de interrupcion,
 * isr
 * Las ISR son rutinas en ensamblador que salvan el estado del procesador
 * y configuran los registros de segmento necesarios para que se
 * invoque el manejador de interrupcion del kernel
 * Las primeras 32 entradas corresponden a las rutinas que manejan las
 * excepciones de los procesadores x86.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void install_isr(void);

/*
 * Rutina: setup_idt
 * Descripcion: Esta rutina permite configurar la Tabla de Descriptores de
 * Interrupcion (IDT).
 * Al final de la funcion se usa inline assembly para cargar el registro idtr
 * del procesador con los valores de la IDT configurada.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 *  */
void setup_idt(void) {
	int i;

	/* Configurar la base y el limite de la estructura idt_pointer
	 * que se utilizara en la instruccion lidt. */
	idt_pointer.limit = (sizeof (idt_descriptor) * MAX_IDT_ENTRIES);
	idt_pointer.base = (unsigned int) &idt;


	/* Inicializar toda la IDT en cero */
	 for (i=0; i<MAX_IDT_ENTRIES; i++) {
		 idt[i].offset_low = 0;
		 idt[i].selector = 0;
		 idt[i].always0 = 0;
		 idt[i].flags = 0;
		 idt[i].offset_high = 0;
	 }

	/* Configurar las Rutinas de Servicio de Interrupcion (ISR)
      para las excepciones x86 (las primeras 32 entradas de la IDT)
    */
	install_isr();

	//printf("Loading the IDT...");

	/* Y finalmente cargar la IDT con inline assembly */
	inline_assembly("lidt (%%eax)"
			:
			:"a"(&idt_pointer)
			:);


	//printf("OK\n");

}

/*
 * Rutina: setup_interrupt_gate
 * Descripcion: Funcion para configurar un descriptor de interrupcion dentro
 * de la IDT.
 * Entrada:
 * 	int 	: indice del descriptor en la IDT a configurar
 *  unsigned int : Direccion de memoria de la rutina de servicio de
 *  				interrupcion (isr)
 *  unsigned short : Selector en el GDT en el cual se describe el
 *  				segmento de codigo dentro del cual se encuentra la isr
 *  flags	: Para interrupt gate, flags siempre es 0x8E
 * Salida:
 * 	Ninguno.
 * */
void setup_interrupt_gate(int index, unsigned int offset,
	unsigned short selector, unsigned char flags) {
	idt[index].offset_low = (offset & 0x0000FFFF);
	idt[index].offset_high = (offset >> 16 & 0x0000FFFF);
	idt[index].selector = selector;
	idt[index].always0 = 0;
	idt[index].flags = flags; /* 0x8E es una interrupt gate
				(ver seccion 5.11)*/
}

/*
 * Rutina: install_isr
 * Descripcion: Funcion para instalar las rutinas de servicio de interrupcion,
 * isr
 * Las ISR son rutinas en ensamblador que salvan el estado del procesador
 * y configuran los registros de segmento necesarios para que se
 * invoque el manejador de interrupcion del kernel
 * Las primeras 32 entradas corresponden a las rutinas que manejan las
 * excepciones de los procesadores x86.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void install_isr(void) {

	setup_interrupt_gate(0, (unsigned int)isr0, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(1, (unsigned int)isr1, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(2, (unsigned int)isr2, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(3, (unsigned int)isr3, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(4, (unsigned int)isr4, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(5, (unsigned int)isr5, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(6, (unsigned int)isr6, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(7, (unsigned int)isr7, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(8, (unsigned int)isr8, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(9, (unsigned int)isr9, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(10, (unsigned int)isr10, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(11, (unsigned int)isr11, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(12, (unsigned int)isr12, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(13, (unsigned int)isr13, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(14, (unsigned int)isr14, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(15, (unsigned int)isr15, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(16, (unsigned int)isr16, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(17, (unsigned int)isr17, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(18, (unsigned int)isr18, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(19, (unsigned int)isr19, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(20, (unsigned int)isr20, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(21, (unsigned int)isr21, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(22, (unsigned int)isr22, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(23, (unsigned int)isr23, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(24, (unsigned int)isr24, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(25, (unsigned int)isr25, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(26, (unsigned int)isr26, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(27, (unsigned int)isr27, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(28, (unsigned int)isr28, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(29, (unsigned int)isr29, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(30, (unsigned int)isr30, KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(31, (unsigned int)isr31, KERNEL_CODE_SELECTOR, 0x8E);

}

/*
 * Rutina: fault_handler
 * Descripcion: Esta rutina recibe el control de la Rutina de Servicio
 * de Interrupcion (ISR) correspondiente.
 * Esta rutina no retorna.
 * Entrada:
 *  struct interrupt_state  : Estructura de datos que contiene el estado de los registros
 *  del procesador
 * Salida:
 *	Ninguno.
 * */
 void fault_handler(struct interrupt_state r) {
 	if (r.number < 32) {
 	   printf("[%d] %s Exception. System Halted!\n", r.number,
 		exceptions[r.number]);
 	   for (;;);
 	}
 	printf("Unhandled exception [%d]. System halted.", r.number);
 	for (;;);
 }
