/*
 * irq.c
 * Este archivo contiene la implementacion de las rutinas necesarias para
 * manejar las solicitudes de interrupcion (IRQ) de los dispositivos de
 * entrada / salida.
 * */

#include <idt.h>
#include <interrupt.h>
#include <stdio.h>
#include <stdlib.h>

/* Primitivas basicas para el manejo de interrupciones */
//OCW2 (End of Interrupt): Codigo para escribir en el puerto de comandos del PIC
#define EOI 0x20

/* Direccion del puerto de comandos del PIC  maestro*/
#define MASTER_PIC_COMMAND_PORT 0x20

/* Direccion del puerto de comandos del PIC esclavo */
#define SLAVE_PIC_COMMAND_PORT 0xA0

/* Direccion del puerto de datos del PIC maestro*/
#define MASTER_PIC_DATA_PORT 0x21

/* Direccion del puerto de datos del PIC esclavo */
#define SLAVE_PIC_DATA_PORT 0xA1

/* Arreglo que contiene los apuntadores a las rutinas de manejo de interrupcion
 */
void * interrupt_routines[MAX_INTERRUPT_ROUTINES] = {0};

/*
 * Rutina: interrupt_handler
 * Descripcion: Esta rutina recibe el control de la rutina de manejo de
 * interrupcion (IRQ) correspondiente. Las IRQ se encuentran definidas
 * en el archivo irq.S.
 * Entrada:
 *  struct interrupt_state  : Estructura de datos que contiene el estado de los registros
 *  			   del procesador
 * Salida:
 *	Ninguno.
 * */
void interrupt_handler(struct interrupt_state);

/*
 * Rutina: irq_remap
 * Descripcion: Al arranque, las IRQ 0 a 7 estan mapeadas a las interrupciones
 * 0x8 - 0xF. Estas entradas estan asignadas por defecto a las excepciones x86
 * (la entrada 8 es Double Fault, por ejemplo).
 * Por esta razon es necesario mapear las IRQ a otros numeros de interrupcion.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void irq_remap(void);


/* Macro para definir mas facilmente la referencia a las rutinas de manejo de
	IRQ  implementadas en  irq.S
	Este macro recibe como parametro el numero de IRQ y se expande a:

	extern void irq#(void);

	donde # es el numero de IRQ*/

#define IRQ(id) \
 extern void irq##id(void);

IRQ(0);
IRQ(1);
IRQ(2);
IRQ(3);
IRQ(4);
IRQ(5);
IRQ(6);
IRQ(7);
IRQ(8);
IRQ(9);
IRQ(10);
IRQ(11);
IRQ(12);
IRQ(13);
IRQ(14);
IRQ(15);

/* Implementacion de las rutinas */

/*
 * Rutina: irq_remap
 * Descripcion: Al arranque, las IRQ 0 a 7 estan mapeadas a las interrupciones
 * 0x8 - 0xF. Estas entradas estan asignadas por defecto a las excepciones x86
 * (la entrada 8 es Double Fault, por ejemplo).
 * Por esta razon es necesario mapear las IRQ a otros numeros de interrupcion.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void irq_remap(void) {
	/* Reprogramacion del PIC (ver archivo readme.txt) */

	/* Initialization Command Word 1 - ICW1
	Esta es la palabra primaria para inicializar el PIC.
	Para inicializar el PIC se requiere que los bits 0 y 4 de ICW1 esten en
	1 y los demas en 0. Esto significa que el valor de ICW1 es 0x11.
	ICW1 debe ser escrita en el  registro de comandos del PIC maestro
	(direccion de e/s 0x20). Si existe un PIC esclavo, ICW1 se debe enviar
	tambien su registro de comandos (0xA0)
	*/

	outb(MASTER_PIC_COMMAND_PORT, 0x11);
	outb(SLAVE_PIC_COMMAND_PORT, 0x11);

	/* Initialization Command Word 2 - ICW2
	Esta palabra permite definir la direccion base (inicial) en la tabla de
	descriptores de interrupcion que el PIC va a utilizar.

	Debido a que las primeras 32 entradas estan reservadas para las
	excepciones en la arquitectura x86, ICW2 debe contener un valor mayor o
	igual a 32 (0x20). Los valores  de ICW2 representan el numero de IRQ
	base que manejara el PIC

	Al utilizar los PIC en cascada, se debe enviar ICW2 a los dos
	controladores en su registro de datos (0x21 y 0xA1 para maestro y
	esclavo respectivamente), indicando la direccion en la IDT que va a ser
	utilizada por cada uno de ellos.

	Las primeras 8 IRQ van a ser manejadas por el PIC maestro y se mapearan
	a partir del numero 32 (0x20). Las siguientes 8 interrupciones las manejara
	el PIC esclavo, y se mapearan a la direccion 40 (0x28).
	*/

	outb(MASTER_PIC_DATA_PORT, 0x20);
	outb(SLAVE_PIC_DATA_PORT, 0x28);

	/* Initialization Control Word 3 - ICW3
	Esta palabra permite definir cuales lineas de IRQ van a ser compartidas
	por los PIC maestro y esclavo. Al igual que ICW2, ICW3 tambien se
	escribe en los 	registros de datos de los PIC (0x21 y 0xA1 para el PIC
	maestro y esclavo,respectivamente).

	Dado que en la arquitectura Intel el PIC maestro se conecta con el PIC
	esclavo por medio de la linea IRQ 2, el valor de ICW3 debe ser 00000100
	(0x04), que define el bit 3 (correspondiente a la linea IRQ2) en 1.

	Para el PIC esclavo, el numero de la linea se debe representar en
	notacion binaria. Por lo tanto, 000 corresponde a la linea de IRQ 0,
	001 corresponde a la linea de IRQ 1, 010 corresponde a la linea de
	IRQ 2, y asi sucesivamente.
	Debido a que se va a utilizar la linea de IRQ 2, el valor de ICW3
	para el PIC esclavo debe ser 00000010, (0x02).
	*/

	outb(MASTER_PIC_DATA_PORT, 0x04);
	outb(SLAVE_PIC_DATA_PORT, 0x02);

	/* Initialization Control Word 4 - ICW4
	Para ICW4 solo es necesario establecer su bit 0 (x86 mode) y escribirla
	en los registros de datos del PIC maestro y esclavo (0x21 y 0xA1).
	El valor de ICW4 debe ser entonces 00000001, es decir, 0x01.
	*/
	outb(MASTER_PIC_DATA_PORT, 0x01);
	outb(SLAVE_PIC_DATA_PORT, 0x01);
}

/*
 * Rutina: setup_interrupts
 * Descripcion: Esta rutina se encarga de crear las entradas en la IDT para
 * las interrupciones que se desean manejar. Por defecto configura las
 * interrupciones correspondientes a las IRQ de los dispositivos de entradda/
 * salida, que han sido mapeadas a los numeros 32..47.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void setup_interrupts(void) {
	int i;

	for (i=0; i<MAX_INTERRUPT_ROUTINES;i++) {
		interrupt_routines[i]=0;
	}

	/* Mapear las IRQ 0..15 a las entradas 32 .. 47 de la IDT */
	irq_remap();

/*Establecer las entradas 32 .. 47 con las rutinas de manejo de interrupcion*/
	setup_interrupt_gate(IRQ0_INTERRUPT, (unsigned int)irq0,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ1_INTERRUPT, (unsigned int)irq1,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ2_INTERRUPT, (unsigned int)irq2,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ3_INTERRUPT, (unsigned int)irq3,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ4_INTERRUPT, (unsigned int)irq4,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ5_INTERRUPT, (unsigned int)irq5,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ6_INTERRUPT, (unsigned int)irq6,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ7_INTERRUPT, (unsigned int)irq7,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ8_INTERRUPT, (unsigned int)irq8,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ9_INTERRUPT, (unsigned int)irq9,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ10_INTERRUPT, (unsigned int)irq10,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ11_INTERRUPT, (unsigned int)irq11,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ12_INTERRUPT, (unsigned int)irq12,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ13_INTERRUPT, (unsigned int)irq13,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ14_INTERRUPT, (unsigned int)irq14,
						KERNEL_CODE_SELECTOR, 0x8E);
	setup_interrupt_gate(IRQ15_INTERRUPT, (unsigned int)irq15,
						KERNEL_CODE_SELECTOR, 0x8E);
}

/*
 * Rutina: install_interrupt_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de IRQ dentro
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	int: numero de interrupcion a configurar, > 32
 *  void (*)(struct interrupt_state) : Rutina a manejar la interrupcion
 * Salida:
 * 	Ninguno. */
void install_interrupt_handler(int number, void (*handler)(struct interrupt_state r)){
	/* Simplemente sobre-escribir la rutina anterior, si existe.
	 * Se debe recordar que las IRQ se mapearon a los numeros 32..47, por esta
	 * razon es necesario restar IDT_INTERRUPT_OFFSET (32) al numero de la interrupcion
	 * */
	int index;

	if (number < IDT_INTERRUPT_OFFSET) {
		printf(" Error! minimum interrupt number must be %d\n", IDT_INTERRUPT_OFFSET);
	}

	index = number - IDT_INTERRUPT_OFFSET;

	interrupt_routines[index] = handler;
}

/*
 * Rutina: uninstall_interrupt_handler
 * Descripcion: Esta rutina permite quitar un  manejador de IRQ dentro
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	int: numero de interrupcion a configurar
 *  void (*)(struct interrupt_state) : Rutina a manejar la interrupcion
 * Salida:
 * 	Ninguno. */
void uninstall_interrupt_handler(int number) {
	/* Simplemente eliminar la rutina de manejo de interrupcion*/
	int index;
	index = number - IDT_INTERRUPT_OFFSET;
	interrupt_routines[index] = 0;
}

/*
 * Rutina: interrupt_handler
 * Descripcion: Esta rutina recibe el control de la rutina de manejo de
 * interrupcion (IRQ) correspondiente. Las IRQ se encuentran definidas
 * en el archivo irq.S.
 * Todas las rutinas de manejo de interrupcion invocan a esta rutina.
 * Dentro de la estructura de datos que recibe, se puede obtener el numero
 * de la interrupcion que ocurrio, asi como el estado del procesador.
 * Entrada:
 *  struct interrupt_state  : Estructura de datos que contiene el estado de los registros
 *  			   del procesador
 * Salida:
 *	Ninguno.
 * */
void interrupt_handler(struct interrupt_state r) {
	void (*handler)(struct interrupt_state r);

	int index;

	index = r.number - IDT_INTERRUPT_OFFSET; /* Determinar el numero de la IRQ */

	//printf("IRQ %d (interrupt %d) launched!\n", index, r.number);

	/*
	 * Enviar EOI al puerto de control del 8259 que lanzo la interrupcion
	 * */

	/* Si la IRQ es 8..15, se debe enviar EOI al 8259 esclavo tambien. */
	if (index >= 8) {
		outb(SLAVE_PIC_COMMAND_PORT, EOI);
	}
	outb(MASTER_PIC_COMMAND_PORT, EOI);

	/* Buscar la rutina que maneja la interrupcion */
	handler = interrupt_routines[index];

	/* Si la rutina existe, ejecutarla y pasarle como parametro los registros.*/
	if (handler != 0) {
			handler(r);
	}
}
