/*
 * irq.S
 * Este archivo contiene la definicion y la implementacion de las
 * rutinas de para el manejo de las 15 interrupciones del PIC (Programable
 * Interrupt Controller). Al PIC se conectan los dispositivos de E/S como
 * el timer, el teclado, el mouse, los discos, etc.
 */

.intel_syntax noprefix /* Usar sintaxis Intel, si prefijo para los registros */
.section .text		/* Segmento de texto */
.code32				/* 32 bits - Modo protegido */

#define ASM 1 /* Solo incluir las constantes del archivo pm.h*/
#include <pm.h>


/*
* Macro: irq_define
* Descripcion: Este macro permite crear las  rutinas para el manejo de las
*			   solicitudes de interrupcion IRQ0 - IRQ 15.
*			   Se debe tener en cuenta que las IRQ0 - IRQ15 se mapearon
*			   a las interrupciones numero 32..47. Es decir, la IRQ0
*			   lanzara la interrupcion numero 32, IRQ1 lanzara la interrupcion
*			   33 y asi sucesivamente.
* 			   Para estas IRQ se crea un marco de pila similar al que se creo
* 			   para manejar las excepciones del procesador Intel. Ver isr.S.
*			   El codigo de error enviado a la rutina de manejo de interrupcion
*			   siempre es cero.
*/
.macro define_irq id number
 .global irq\id
irq\id:
    /* Deshabilitar las interrupciones*/
 	cli
 	/* Ahora se crea un marco de pila estandar para invocar la rutina general
 	interrupt_handler. */
 	/* Codigo de error, siempre 0 para este tipo de interrupciones */
	push 0
	/* # de interrupcion generada */
	push \number
	/* Almacenar en la pila los registros de proposito general en el siguiente
	orden: eax, ecx, edx, ebx, esp original, ebp, esi, y edi */
	pusha
	/* Almacenar en la pila los registros de segmento de datos */
	push ds
	push es
	push fs
	push gs

	/*
	La pila luce asi:
	+--------------------------+
	| old ss                   | Estos valores son almacenados automaticamente
	|--------------------------| en la pila cuando ocurre una excepcion
	| old esp                  | old_ss y old_sp se almacenan solo si cambia PL
	|--------------------------| ..
	| eflags                   | ..
	|--------------------------| ..
	| old cs                   | ..
	|--------------------------| ..
	| old eip                  | ..
	|--------------------------| -------------------------------------------
	| 0 (codigo de error)      | push 0 (siempre codigo de error = 0)
	|--------------------------|
	| # de interrupcion        | push \number
	|--------------------------|
	| eax                      | pusha
	|--------------------------|
	| ecx                      |(recuerde que pusha almacena en la pila los
	|--------------------------|registros en el siguiente orden:
	| edx                      |eax, ecx, edx, ebx, esp original, ebp, esi,
	|--------------------------|edi)
	| ebx                      |
	|--------------------------|
	| esp antes de pusha       |
	|--------------------------|
	| ebp                      |
	|--------------------------|
	| esi                      |
	|--------------------------|
	| edi                      |
	|--------------------------|------------------------------------------
	| ds                       | ahora los registros de segmento de datos
	|--------------------------|
	| es                       |
	|--------------------------|
	| fs                       |
	|--------------------------|
	| gs                       |
	|--------------------------|<--esp
	*/

    /* Configurar los registros de segmento de datos para que contengan
	el selector de datos para el kernel definido en la GDT */
	movw ax, KERNEL_DATA_SELECTOR
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax

	/* interrupt_handler recibe como parametro una estructura de tipo regs,
	la cual se almaceno con los 'push' anteriores  */

	call interrupt_handler
	/* Ahora sacar los parametros enviados a la pila en orden inverso*/
	pop gs
	pop fs
	pop es
	pop ds
	/* los registros de proposito general */
	popa
	/* Codigo de error y excepcion generada */
	add esp, 8
	iret
.endm


/* Definir las rutinas  para el manejo de las IRQ0-IRQ15. A estas IRQ les
corresponden los numeros de interrupcion 32..46, luego de re-programar el
PIC. */

define_irq 0 32
define_irq 1 33
define_irq 2 34
define_irq 3 35
define_irq 4 36
define_irq 5 37
define_irq 6 38
define_irq 7 39
define_irq 8 40
define_irq 9 41
define_irq 10 42
define_irq 11 43
define_irq 12 44
define_irq 13 45
define_irq 14 46
define_irq 15 47
