/*
 * isr.S
 * Este archivo contiene la definicion y la implementacion de las
 * rutinas de servicio de interrupcion para las primeras 32 excepciones
 * del procesador Intel x86.
 */

.intel_syntax noprefix /* Usar sintaxis Intel, si prefijo para los registros */
.section .text		/* Segmento de texto */
.code32				/* 32 bits - Modo protegido */

#define ASM 1 /* Solo incluir las constantes del archivo pm.h*/
#include <pm.h>


/* Rutina: cli
* Descripcion: Esta rutina permite deshabilitar las interrupciones por medio
* 			   de la instruccion cli.
* Entrada:
*  Ninguno.
* Saluda:
*  Ninguno.
*/
.global cli
cli:
  cli
  ret

/* Rutina: sti
* Descripcion: Esta rutina permite habilitar las interrupciones por medio
* 			   de la instruccion sti.
* Entrada:
*  Ninguno.
* Saluda:
*  Ninguno.
*/
.global sti
sti:
  sti
  ret

/*
* Macro: isr_no_error_code
* Descripcion: Este macro permite crear las  rutinas de servicio de
*			   interrupcion (isr) para las excepciones del procesador
*			   Intel x86 que no generan codigo de error. Estas son:
*	           0 - 7, 9 y 16 en adelante.
*              Con el fin de mantener un marco de pila constante con las rutinas
*              de servicio de interrupcion que si generan codigo de error, se
* 			   inserta un '0' como codigo de error.
*/
.macro isr_no_error_code id
 .global isr\id
 isr\id:
    /* Deshabilitar las interrupciones*/
 	cli
 	/* Ahora se crea un marco de pila estandar para invocar la rutina general
 	falut_handler. */
 	/* Codigo de error, siempre 0 para este tipo de excepciones */
	push 0
	/* # de excepcion generada */
	push \id
	/* Almacenar en la pila los registros de proposito general en el siguiente
	orden: eax, ecx, edx, ebx, esp original, ebp, esi, y edi */
	pusha
	/* Almacenar en la pila los registros de segmento de datos */
	push ds
	push es
	push fs
	push gs

	/*
	La pila luce asi:
	+--------------------------+
	| old ss                   | Estos valores son almacenados automaticamente
	|--------------------------| en la pila cuando ocurre una excepcion
	| old esp                  | old_ss y old_sp se almacenan solo si cambia PL
	|--------------------------| ..
	| eflags                   | ..
	|--------------------------| ..
	| old cs                   | ..
	|--------------------------| ..
	| old eip                  | ..
	|--------------------------| -------------------------------------------
	| 0 (codigo de error)      | push 0 (siempre codigo de error = 0)
	|--------------------------|
	| # de excepcion generada  | push \id
	|--------------------------|
	| eax                      | pusha
	|--------------------------|
	| ecx                      |(recuerde que pusha almacena en la pila los
	|--------------------------|registros en el siguiente orden:
	| edx                      |eax, ecx, edx, ebx, esp original, ebp, esi,
	|--------------------------|edi)
	| ebx                      |
	|--------------------------|
	| esp antes de pusha       |
	|--------------------------|
	| ebp                      |
	|--------------------------|
	| esi                      |
	|--------------------------|
	| edi                      |
	|--------------------------|------------------------------------------
	| ds                       | ahora los registros de segmento de datos
	|--------------------------|
	| es                       |
	|--------------------------|
	| fs                       |
	|--------------------------|
	| gs                       |
	|--------------------------|<--esp
	*/

    /* Configurar los registros de segmento de datos para que contengan
	el selector de datos para el kernel definido en la GDT */
	movw ax, KERNEL_DATA_SELECTOR
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax

	/* fault_handler recibe como parametro una estructura de tipo regs,
	la cual se almaceno con los 'push' anteriores  */

	call fault_handler
	/* Ahora sacar los parametros enviados a la pila en orden inverso*/
	pop gs
	pop fs
	pop es
	pop ds
	/* los registros de proposito general */
	popa
	/* Codigo de error y excepcion generada */
	add esp, 8
	iret
.endm

/*
* Macro: isr_no_error_code
* Descripcion: Este macro permite crear las  rutinas de servicio de
*			   interrupcion (isr) para las excepciones del procesador
*			   Intel x86 que si generan codigo de error. Estas son:
*	           8 y 10-14.
*/
.macro isr_error_code id
 .global isr\id
 isr\id:
    /* Deshabilitar las interrupciones*/
 	cli
 	/* Ahora se crea un marco de pila estandar para invocar la rutina general
 	fault_handler. */
 	/* El codigo de error es almacenado  por el procesador en la pila
 	de forma automatica cuando ocurre la excepcion
 	*/
	/* # de excepcion generada */
	push \id
	/* Almacenar en la pila los registros de proposito general en el siguiente
	orden: eax, ecx, edx, ebx, esp original, ebp, esi, y edi */
	pusha
	/* Almacenar en la pila los registros de segmento de datos */
	push ds
	push es
	push fs
	push gs

	/*
	La pila luce asi:
	+--------------------------+
	| old ss                   | Estos valores son almacenados automaticamente
	|--------------------------| en la pila cuando ocurre una excepcion
	| old esp                  | old_ss y old_sp se almacenan solo si cambia PL
	|--------------------------| ..
	| eflags                   | ..
	|--------------------------| ..
	| old cs                   | ..
	|--------------------------| ..
	| old eip                  | ..
	|--------------------------|
	|   (codigo de error)      | El codigo de error se almacena automaticamente
	|--------------------------|---------------------------------------------
	| # de excepcion generada  | push \id
	|--------------------------|
	| eax                      | pusha
	|--------------------------|
	| ecx                      |(recuerde que pusha almacena en la pila los
	|--------------------------|registros en el siguiente orden:
	| edx                      |eax, ecx, edx, ebx, esp original, ebp, esi,
	|--------------------------|edi)
	| ebx                      |
	|--------------------------|
	| esp antes de pusha       |
	|--------------------------|
	| ebp                      |
	|--------------------------|
	| esi                      |
	|--------------------------|
	| edi                      |
	|--------------------------|------------------------------------------
	| ds                       | ahora los registros de segmento de datos
	|--------------------------|
	| es                       |
	|--------------------------|
	| fs                       |
	|--------------------------|
	| gs                       |
	|--------------------------|<--esp
	*/

    /* Configurar los registros de segmento de datos para que contengan
	el selector de datos para el kernel definido en la GDT */
	movw ax, KERNEL_DATA_SELECTOR
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax

	/* fault_handler recibe como parametro una estructura de tipo regs,
	la cual se almaceno con los 'push' anteriores  */

	call fault_handler
	/* Ahora sacar los parametros enviados a la pila en orden inverso*/
	pop gs
	pop fs
	pop es
	pop ds
	/* los registros de proposito general */
	popa
	/* Codigo de error y excepcion generada */
	add esp, 8


	/*
	Ahora la pila luce asi:
	+--------------------------+
	| old ss                   | Estos valores son almacenados automaticamente
	|--------------------------| en la pila cuando ocurre una excepcion
	| old esp                  | old_ss y old_sp se almacenan solo si cambia PL
	|--------------------------| ..
	| eflags                   | ..
	|--------------------------| ..
	| old cs                   | ..
	|--------------------------| ..
	| old eip                  | ..
	+--------------------------+
	*/
	/* Retornar de la interrupcion*/
	iret
.endm

/* Definir las rutinas de servicio de interrupcion. Se debe tener
en cuenta que las rutinas 0-7, 9, y 16 en adelante no generan codigo
de error, mientras que las rutinas 8, y 10-14 si generan codigo de error. */
isr_no_error_code 0 /* 0: Divide By Zero*/
isr_no_error_code 1 /* 1: Debug Exception */
isr_no_error_code 2 /* 2: Non Maskable Interrupt */
isr_no_error_code 3 /* 3: Int 3 Exception */
isr_no_error_code 4 /* 4: INTO Exception */
isr_no_error_code 5 /* 5: Out of Bounds Exception */
isr_no_error_code 6 /* 6: Invalid Opcode Exception */
isr_no_error_code 7 /* 7: Coprocessor Not Available */
isr_error_code 8	/* 8: Double Fault Exception */
isr_no_error_code 9 /* 9: Coprocessor Segment Overrun Exception */
isr_error_code 10	/* 10: Bad TSS Exception */
isr_error_code 11	/* 11: Segment Not Present Exception*/
isr_error_code 12	/* 12: Stack Fault Exception*/
isr_error_code 13	/* 13: General Protection Fault Exception*/
isr_error_code 14	/* 14: Page Fault Exception*/
isr_no_error_code 15 /* 15: Reserved Exception*/
isr_no_error_code 16 /* 16: Floating Point Exception*/
isr_no_error_code 17 /* 17: Alignment Check Exception*/
isr_no_error_code 18 /* 18: Machine Check Exception*/
isr_no_error_code 19 /* 19: Reserved Exception*/
isr_no_error_code 20 /* 20: Reserved Exception*/
isr_no_error_code 21 /* 21: Reserved Exception*/
isr_no_error_code 22 /* 22: Reserved Exception*/
isr_no_error_code 23 /* 23: Reserved Exception*/
isr_no_error_code 24 /* 24: Reserved Exception*/
isr_no_error_code 25 /* 25: Reserved Exception*/
isr_no_error_code 26 /* 26: Reserved Exception*/
isr_no_error_code 27 /* 27: Reserved Exception*/
isr_no_error_code 28 /* 28: Reserved Exception*/
isr_no_error_code 29 /* 29: Reserved Exception*/
isr_no_error_code 30 /* 30: Reserved Exception*/
isr_no_error_code 31 /* 31: Reserved Exception*/
