/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * Este proyecto implementa la funcionalidad necesaria para configurar el timer
 * (Ver readme.txt). El timer emitira aproximadamente 100 interrupciones por
 * segundo, es decir tendra una frecuencia de 100 hz.
 * La rutina de manejo del timer timer_handler (definida en timer.c) imprime
 * un '.' en la pantalla cada vez que el contador de ticks (interrupciones del
 * timer) sea multiplo de la frecuencia.
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <pm.h>
#include <idt.h>
#include <interrupt.h>
#include <timer.h>
#include <keyboard.h>

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */


void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Si no se encuentra memoria disponible, terminar */
	if (memory_length <= 0) {
		printf("No available memory found!\n");
		return;
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);

	/* Pasar a modo protegido, usando la GDT configurada en pm.c */
	setup_gdt();

	/* Configurar la IDT en idt.c */
	setup_idt();

	/* Configurar los manejadores de interrupcion por defecto */
	setup_interrupts();

	/* Configurar el timer para que emita aproximadamente 100 interrupciones
	 * por segundo, es decir una interrupcion cada 10 milisegundos */
	setup_timer();

	/* configurar la rutina de manejo de teclado */
	setup_keyboard();

	/* Y finalmente habilitar las interrupciones. Esto permite
	 * que el timer y los demas dispositivos de entrada/salida puedan
	 * emitir interrupciones. */


	sti();



	printf("Kernel started\n");


}
