/*
 * keyboard.c
 * En este archivo se implementan las funcionalidades para manejar el
 * controlador de teclado 8042
 */
#include <idt.h>
#include <interrupt.h>
#include <keyboard.h>
#include <asm.h>
#include <timer.h>

/* Puerto de datos del teclado: registro de salida de 8042 */
#define KEYBOARD_DATA_PORT 0x60

/* Puerto de datos del 8042: 0x64 read */
#define KEYBOARD_STATUS_PORT 0x64

/* Puerto de comandos del 8042: 0x64 Write*/
#define KEYBOARD_COMMAND_PORT KEYBOARD_STATUS_PORT


/* Constante para un break code */
#define BREAK_CODE 0x80

/* Constante para verificar el estado del bufer de salida */
#define KEYBOARD_OUTPUT_BUFFER_FULL 0x1

/*
 * Rutina: keyboard_handler
 * Descripcion: Esta rutina se encarga del manejo de interrupcion de teclado.
 * Entrada:
 *  struct interrupt_state : Estructura con el estado de los registros del procesador.
 * Salida:
 * 	Ninguno.
 *  */
void keyboard_handler(struct interrupt_state);

/*
 * Rutina: setup_keyboard
 * Descripcion: Esta rutina configura el manejador de interrupcion de teclado.
 * */
void setup_keyboard(void) {

	/* Antes de instalar la rutina de manejo de interrupcion,
	 * eliminar cualquier dato que se encuentre en el buffer de salida
	 * del teclado. */
	flush_keyboard_controller();

	/* Instalar la rutina 'keyboard_handler' para gestionar la interrupcion
		 * generada por la IRQ1 */
	install_interrupt_handler(IRQ1_INTERRUPT, &keyboard_handler);

	/* Intentar vaciar el buffer de teclado despues de instalar la
	 * rutina de manejo de interrupcion.*/
	flush_keyboard_controller();
}

/*
 * Rutina: keyboard_handler
 * Descripcion: Esta rutina se encarga del manejo de interrupcion de teclado.
 * Entrada:
 *  struct interrupt_state : Estructura con el estado de los registros del procesador.
 * Salida:
 * 	Ninguno.
 *  */
void keyboard_handler(struct interrupt_state r) {

	unsigned short c;
	unsigned short status;
	unsigned short code;
	int is_break;

	/* Esta rutina simplemente toma el valor almacenado en el puerto de
	 * salida del 8042. Verifica si es un codigo Make o Brake (tecla pulsada
	 * o soltada), y luego imprime el codigo en binario, hexadecimal y decimal*/

	//status = inb(KEYBOARD_STATUS_PORT);
	//printf("Status: %b\n", status);

	/* Si no existen datos en el buffer de salida del teclado, retornar */

	c = inb(KEYBOARD_DATA_PORT);
	code = c;

	is_break = 0;
	if (code & BREAK_CODE) {
		code &= ~(BREAK_CODE); /* Eliminar el break code */
		is_break = 1;
		printf("Break ");
	}else {
		printf("Make ");
	}
	/* code contiene el codigo de la tecla, igual si es break o make. */
	printf("%b %x [%d]", code, code, code);
	printf(" EOI\n");

}

/*
 * Rutina: flush_keyboard_controller
 * Descripcion: Esta rutina permite descartar todos los datos que
 * se encuentren en el buffer de salida del teclado (0x60).
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void flush_keyboard_controller(void) {

	unsigned char status, data;
	int i=0;

	//printf("Waiting for keyboard output buffer to be empty...\n");
	do {
		status = inb(KEYBOARD_STATUS_PORT);
		//printf("%b\n", status);
		delay(20); /* Esperar 10 milisegundos */
		data=inb(KEYBOARD_DATA_PORT);
		delay(20);
	}while (status & KEYBOARD_OUTPUT_BUFFER_FULL);
	//printf("ok");
}
