/*
 * pm.c
 * Este archivo contiene la variables y la implementacion de las rutinas
 * necesarias para el paso a modo protegido desde codigo en C
 */

#include <pm.h>
#include <stdio.h>
#include <stdlib.h>

/* Tabla global de descriptores (GDT). Debe estar alineado 8 bytes (Intel).
 * La GDT es un arreglo en el cual cada entrada es un descriptor de segmento.
 * El formato de los descriptores de segmento se define en el include/pm.h
 * */

segment_descriptor gdt[MAX_GDT_ENTRIES] __attribute__((aligned(8)));


/* El apuntador al GDT que se utiliza en la instruccion lgdt. */
gdt_ptr gdt_pointer;

/* Tiene el siguiente formato:
 * 47           31                    0
 * +----------------------------------+
 * |  limite    |    base             |
 * +----------------------------------+
 *
 * en donde:
 * base = direccion lineal del gdt, que corresponde a la direccion
 * de memoria de la variable gdt.
 *
 * limite = tamanio de la GDT.
 * */

/*
 * Rutina: setup_gdt
 * Descripcion: Esta rutina se encarga de configurar la tabla global de
 * descriptores (GDT) y cargarla mediante la instruccion LGDT. Luego pasa de
 * nuevo a modo protegido.
 * Dado a que la carga de la GDT solo se puede realizar en codigo ensamblador,
 * se utiliza inline assembly.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_gdt(void) {
	int i;
	/* Variables para verificar los valores pasados por inline assembly */
	unsigned int gdt_ptr_location;
	short cs_value, ds_value;

	//printf("Setting up the GDT...\\n");

	/* Primero inicializar el registro del GDT que se va a cargar mediante
	 * la directiva de ensamblador lgdt */

	/* Tamano del GDT - 1 en bytes */
	gdt_pointer.limit = sizeof(segment_descriptor)*MAX_GDT_ENTRIES - 1;

	/* Direccion lineal de 32 bits en donde se encuentra la GDT,
	 * es decir la direccion de memoria del arreglo gdt. */
	gdt_pointer.base = (unsigned int) &gdt;

	//printf("Location of GDT pointer: %u\\n", &gdt_pointer);
    //printf("GDT base address: %u\\n", gdt_pointer.base);
    //printf("GDT limit: %u\n", gdt_pointer.limit);

	for (i=0; i<MAX_GDT_ENTRIES; i++) {
			setup_null_descriptor(i);
	}

	/* Segmento de codigo del kernel */
	gdt[KERNEL_CODE_SELECTOR >> 3] = code32_segment_descriptor(0, 0xFFFFFFFF,
			RING0_DPL);

	/* Segmento de datos del kernel. */
	gdt[KERNEL_DATA_SELECTOR >> 3] = data32_segment_descriptor(0, 0xFFFFFFFF,
			RING0_DPL);

	/*
	 * Cargar la GDT: Se realiza con Inline Assembly.
	 * Cantidad de parametros de inline assembly: 7
	 * Importante: Aunque el paso a modo protegido se puede realizar
	 *  con menos parametros, se especifican 7 para luego verificar si se
	 *  pasaron correctamente por inline assembly.
	 * Parametros de salida (para verificacion)
	 *  0 : apuntador a la localizacion de gdt_pointer en ax -> gdt_ptr_location
	 *  1 : Valor de cs en bx -> cs_value
	 *  2 : Valor de ds en cx -> ds_value
	 * Parametros de entrada
	 *  3 : Selector de segmento de codigo (inmediato)
	 *  4 : Direccion lineal en memoria de gdt_pointer -> eax
	 *  5 : KERNEL_CODE_SELECTOR -> ebx (para verificacion)
	 *  6 : KERNEL_DATA_SELECTOR -> ecx
	 *
	 *
	 *  Tenga en cuenta que un cargador compatible con la Especificacion
	 *  Multiboot ya ha activado el bit PE de CR0. No obstante, para que
	 *  el codigo sea mas consistente con los pasos sugeridos por Intel
	 *  para pasar a modo protegido, se activa de nuevo este bit.
	 *
	 * */

	inline_assembly(".att_syntax prefix	 	\n\t\
			lgdtl (%%eax)						 	\n\t\
			movl %%cr0, %%edx						\n\t\
			orl $1, %%edx							\n\t\
			movl %%edx, %%cr0						\n\t\
			ljmp %3, $1f	 			\n\t\
1:			 						\n\t\
.intel_syntax noprefix								\n\t\
			mov ds, cx	 							\n\t\
			mov es, cx	 							\n\t\
			mov fs, cx	 							\n\t\
			mov gs, cx	 							\n\t\
			mov ss, cx	 							\n\t\
			.att_syntax prefix"
			: "=a"(gdt_ptr_location), "=b"(cs_value), "=c"(ds_value)
			: "i" (KERNEL_CODE_SELECTOR), "am"(&gdt_pointer),
			  "b"(KERNEL_CODE_SELECTOR),
			  "c"(KERNEL_DATA_SELECTOR)
			:
			);

	//printf("GDT loaded successfully!\\n");
	/*
	 * Ahora verificar que los operandos de salida corresponden a los
	 * operandos de entrada enviados a inline assembly
	 * */

	/* Deberia imprimir el mismo valor: la direccion de memoria de
	 * gdt_pointer */
	//printf("Returned GDT pointer location: %u = %u\\n", gdt_ptr_location,
									//					&gdt_pointer);

	/* Deberia imprimir un valor igual a KERNEL_CODE_SELECTOR */
	//printf("CS = %x\\n", cs_value);

	/* Deberia imprimir un valor igual a KERNEL_DATA_SELECTOR */
	//printf("DS = %x\n", ds_value);
}

/*
 * Rutina: setup_segment_descriptor
 * Descripcion: Esta rutina permite  configurar un descriptor de segmento dentro
 * de la tabla global de descriptores, GDT
 * Entrada:
 * 	int 			: Entrada de la GDT que se desea configurar
 *	unsigned int	: Base del segmento
 *	unsigned int	: Limite del segmento
 *	unsigned char	: Tipo del segmento
 *	unsigned char	: Nivel de privilegio del segmento
 * Salida:
 * 	Ninguno.
 * En caso de error, imprime un mensaje y bloquea el kernel.
 *
 * */
void setup_segment_descriptor(int index,
		unsigned int base,
		unsigned int limit,
		unsigned char type,
		unsigned char dpl) {

	/* Verificar que se este configurando una entrada valida */
	if (index >= MAX_GDT_ENTRIES) {
		printf("[%s] FATAL: Entry %d out of range!\n",
						__func__, index);
		while(1);
	}

	/* Configurar el limite */
	gdt[index].limit_low = (limit & 0x0000FFFF); /* 16 bits */
	gdt[index].limit_high = (limit >> 16 & 0x0F); /* 4 bits */

	/* Configurar la base */
	gdt[index].base_low = (base & 0x0000FFFF); /* 16 bits */
	gdt[index].base_middle = (base >> 16 & 0xFF); /* 8 bits */
	gdt[index].base_high = (base >> 24 & 0xFF); /* 8 bits*/

	/* Configurar los demas atributos */

	gdt[index].type = type; /* tipo de segmento */
	gdt[index].code_or_data = 1; /* Segmento de codigo o datos */
	gdt[index].dpl = dpl;
	gdt[index].present = 1;
	gdt[index].reserved = 0;
	gdt[index].big = 1; /* 0 = 16 bits, 1 = 32 bits*/
	gdt[index].granularity = 1; /* 4K - 4 GB*/
}

/*
 * Rutina: setup_null_descriptor
 * Descripcion: Esta rutina permite establecer en nulo un descriptor de segmento
 *  dentro de la tabla global de descriptores, GDT
 * Entrada:
 * 	int 			: Entrada de la GDT que se desea establecer en nulo
 * Salida:
 * 	Ninguno.
 * En caso de error, imprime un mensaje y bloquea el kernel.
 *
 * */
void setup_null_descriptor(int index) {
	/* Verificar que se este configurando una entrada valida */
	if (index >= MAX_GDT_ENTRIES) {
		printf("[%s] FATAL: GDT entry %d out of range\n",
						__func__, index);
		while(1);
	}

	/* Configurar el limite */
	gdt[index].limit_low = 0; /* 16 bits */
	gdt[index].limit_high = 0; /* 4 bits */

	/* Configurar la base */
	gdt[index].base_low = 0; /* 16 bits */
	gdt[index].base_middle = 0; /* 8 bits */
	gdt[index].base_high = 0; /* 8 bits*/

	/* Configurar los demas atributos */

	gdt[index].type = 0; /* tipo de segmento */
	gdt[index].code_or_data = 0; /* Segmento de codigo o datos */
	gdt[index].dpl = 0;
	gdt[index].present = 0;
	gdt[index].reserved = 0;
	gdt[index].big = 0; /* 0 = 16 bits, 1 = 32 bits*/
	gdt[index].granularity = 0; /* 4K - 4 GB*/
}

/*
 * Rutina: print_gdt_descriptor
 * Descripcion: Imprime un descriptor de la GDT.
 * Entrada:
 * 	int : Entrada a imprimir
 * Salida:
 * 	Ninguno.
 * */
void print_gdt_descriptor(int index) {
	/* Verificar que se va a imprimir una entrada valida */
	if (index >= MAX_GDT_ENTRIES) {
		printf("[%s] FATAL: Entry %d out of range!\n",
						__func__, index);
		while(1);
	}
	printf("Global Descriptor table. Entry %d\n", index);
	printf("limit low: %x\n", gdt[index].limit_low);
	printf("limit high: %x\n", gdt[index].limit_high);

	printf("base low: %x\n", gdt[index].base_low);
	printf("Base middle: %x\n", gdt[index].base_middle);
	printf("Base high: %x\n", gdt[index].base_high);

	printf("Type : %b\n", gdt[index].type);
	printf("Code, data or system: %u\n", gdt[index].code_or_data);
	printf("DPL: %x\n", gdt[index].dpl);
	printf("Present: %u\n", gdt[index].present);
	printf("Available: %u\n", gdt[index].available);
	printf("Reserved: %u\n", gdt[index].reserved);
	printf("D/B (big) bit: %u\n", gdt[index].big);
	printf("Granularity: %u\n", gdt[index].granularity);
	printf("Size of descriptor: %d\n", sizeof(segment_descriptor));
}


