/*
 * mm.h
 * Este archivo contiene la definicion de las primitivas para la gestion de
 * memoria.
 * Ver explicacion al final de este archivo.
 */

#ifndef MM_H_
#define MM_H_

#include "generic_list.h"

/* Limite inferior de regiones de memoria libres. Por encima de este valor
 * la pila se contrae cuando se divide la region libre que se
 * encuentra en el tope del heap. */
#define FREE_MEMREGS_LIMIT 64

/* Minimo numero de bytes permitido para una region de memoria . Debe ser
potencia de 2 */
#define MEMREG_GRANULARITY 4 

/* Numero de bytes preferidos para solicitar. Debe ser potencia de 2, y 
multiplo de MEMREG_GRANULARITY  */
#define AVG_ALLOC_SIZE 0x100 

/* Estructuras de datos para gestionar la memoria disponible */

struct memreg_header;

/* Encabezado de una region de memoria */
typedef struct memreg_header {
	unsigned int base;		/* Inicio del area de datos */
	unsigned int limit;	/* Tamanio de la region de memoria */
	int used;		/* 1 = region asignada, 0 = region libre */
	DEFINE_GENERIC_LIST_LINKS; /*Links genericos */
}memreg_header_t;

/* Definicion de las primitivas para gestionar listas de tipo
 * memreg_header_t*/
DEFINE_GENERIC_LIST_TYPE(memreg_header_t);

/* Funcion para comparar dos regiones de memoria */
int compare_memreg_header_t(memreg_header_t * , memreg_header_t *);

/* Pie de una region de memoria*/
typedef struct memreg_footer {
	unsigned int base;			/* Inicio del area de datos */
	memreg_header_t * header;	/* Apuntador al encabezado */
}memreg_footer_t;

/* Tipo para lista enlazada de encabezados. */
typedef struct memreg_header_list {
	memreg_header_t * head; /* Cabeza de la lista */
	memreg_header_t * tail;
	int count;
} memreg_header_list_t;

/* Tipo para la gestion de memoria dinamica
 * Para gestionar la memoria  se requiere conocer la direccion de inicio de la
 * region para asignacion dinamica, su tamanio, y las listas de regiones
 * libres y asignadas.
 * */

typedef struct heap {
	unsigned int base; /* Inicio de la region de memoria para asignacion
							dinamica */
	unsigned int top; /* Posicion actual del heap */
	unsigned int limit;/* Tamanio maximo de la region de memoria para
							asignacion dinamica */
	list_memreg_header_t * free; /* Lista de regiones de memoria libres */
	list_memreg_header_t * used; /* Lista de regiones de memoria usadas */
}heap_t;


/* Macros para tamanos */

/* Tamano del encabezado de region */
#define MEMREG_HEADER_SIZE sizeof(memreg_header_t)

/* Tamano del pie de la region */
#define MEMREG_FOOTER_SIZE sizeof(memreg_footer_t)

/* Tamano minimo de una region de memoria */
#define MEMREG_MIN_SIZE MEMREG_HEADER_SIZE + MEMREG_GRANULARITY \
		+ MEMREG_FOOTER_SIZE

#define MEMREG_SIZE(size) \
	MEMREG_HEADER_SIZE + size + MEMREG_HEADER_SIZE

/* Tamano minimo del heap */
#define HEAP_MIN_SIZE sizeof(heap_t) + MEMREG_MIN_SIZE

/*
 * Rutina: mm_init
 * Descripcion: Esta rutina se encarga de inicializar el area de memoria
 * para asignacion dinamica, y las estructuras de datos requeridas para su
 * gestion.
 * Entrada:
 * 	void * : Puntero al inicio de la memoria disponible para asignacion dinamica
 *  unsigned int : Tamanio de la memoria disponible
 * Salida:
 * 	Ninguno.
 * */
void mm_init(void *, unsigned int);

/*
 * Rutina: create_heap
 * Descripcion: Permite crear una region de memoria para asignacion dinamica.
 * Importante: Esta rutina asume que la region de memoria que se desea
 * crear esta disponible. Si no se utiliza adecuadamente, puede danar regiones
 * de memoria creadas anteriormente.
 * Entrada:
 * 	unsigned int : Direccion lineal de inicio del heap
 *  unsigned int : Tamanio de la region de memoria para asignacion dinamica.
 * Salida:
 *  heap_t * : Apuntador al nuevo heap
 * */

heap_t * create_heap(unsigned int, unsigned int);

/*
 * Rutina: expand_heap
 * Descripcion: Permite expandir el heap en un tamanio especificado.
 * Entrada:
 *  heap_t * : Heap que se desea expandir
 *  unsigned int : Tamanio a expandir.
 * Salida:
 *  Puntero al encabezado de la nueva region de memoria, 0 si no es
 *  posible expandir el heap.
 * */
memreg_header_t * expand_heap( heap_t *, unsigned int );

/*
 * Rutina: alloc_from_heap
 * Descripcion: Solicita la asignacion de memoria dentro de un heap.
 * Primero se busca dentro de las regiones libres, una que tenga
 * un tamanio cercano al buscado. Si no existe una region, expande el heap.
 * Si existe suficiente espacio adicional al buscado dentro de la region, esta
 * region se divide en dos: una region asignada, que se retorna, y una
 * region libre con el espacio sobrante.
 * Entrada:
 *  heap_t * Heap de la cual se desea obtener la memoria
 * 	unsigned int : Cantidad de memoria a asignar
 * Salida:
 *  void * : Apuntador a la region de memoria asignada. 0 si no se puede
 *  		 asignar memoria.
 * */

void * alloc_from_heap(heap_t *, unsigned int);

/*
 * Rutina: free_from_heap
 * Descripcion: Libera una region de memoria dentro de un heap.
 * Entrada:
 *  heap_t * Heap de la cual se desea liberar la region
 *  memreg_header_t * : Region de memoria a liberar
 * Salida:
 *  Ninguno.
 * */

void free_from_heap(heap_t *, memreg_header_t *);

/*
 * Rutina: mmalloc
 * Descripcion: Solicita asignacion de memoria dentro del heap del kernel.
 * Entrada:
 *  unsigned int: Tamanio a asignar
 * Salida:
 *  void * : Puntero a la base de la region de memoria asignada, 0 si
 *  		 no es posible asignar memoria.
 */
void * mmalloc(unsigned int);

/*
 * Rutina: mfree
 * Descripcion: Solicita liberar una region de memoria dentro del heap
 * del kernel
 * Entrada:
 *  void * : Puntero a la base de la region de memoria a liberar
 * Salida:
 * 	Ninguno.
 */
void  mfree(void * ptr);

/*
 * Rutina: memreg_is_valid
 * Descripcion: Determina si una region de memoria dada es valida dentro de un
 * heap.
 * Entrada:
 * 	heap_t * : Apuntador al heap en el cual se encuentra la region
 *  header_t * : Apuntador al encabezado de la region a validar
 * Salida:
 *  int : 1 = region valida, 0 = region no valida
 * */
int memreg_is_valid(heap_t *, memreg_header_t *);

/*
 * Rutina: print_region
 * Descripcion: Imprime la informacion de una region de memoria
 * Entrada:
 * 	mmreg_header_t * : Apuntador al encabezado de la region
 * Salida:
 *	Ninguno.
 * */
void print_memory_region(memreg_header_t * );

/*
 * Rutina: print_heap
 * Descripcion: Imprime el estado de un heap, incluyendo las regiones
 * que se encuentren definidas.
 * Entrada:
 * 	heap_t * : heap a imprimir
 * Salida:
 *	Ninguno.
 * */
void print_heap(heap_t *);

#endif /* MM_H_ */
