/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion dentro de la rutina cmain.
 * Este proyecto implementa la funcionalidad necesaria para configurar y
 * manejar las 32 excepciones del procesador Intel.
 *
 * Vea el archivo readme.txt para mas detalles.
 *
 * Las estructuras de datos y las rutinas necesarias se definen en el archivo
 * include/exception.h y se implementan en el archivo src/exception.c
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <pm.h>
#include <idt.h>
#include <tss.h>
#include <exception.h>
#include <irq.h>


/* Manejador de prueba.
 * Esta funcion recibe el control cuando se invoca la IRQ a la
 * cual se ha configurado para atender. */
void test_handler(interrupt_state * state) {
	printf("Handler: IRQ %d (INT %d) generated!\n",
			state->number - IDT_IRQ_OFFSET, state->number);
}


/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */


void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	char * linea;

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Si no se encuentra memoria disponible, terminar */
	if (memory_length <= 0) {
		printf("No available memory found!\n");
		return;
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);

	/* Pasar a modo protegido, usando la GDT configurada en pm.c */
	setup_gdt();

	/* Configurar la IDT en idt.c */
	setup_idt();

	/* Ahora configurar el TSS. El TSS es un segmento que se utiliza para
	 * realizar el cambio de contexto de ejecucion de tareas.
	 * Posteriormente se implementara la logica para utilizarlo.
	 * */

	setup_tss();

	/* configurar el manejador por defecto de excepciones */
	setup_exceptions();

	/* configurar el manejador por defecto de IRQ*/
	setup_irq();

	/* configurar un manejador de IRQ de prueba */
	install_irq_handler(0, test_handler);

	/* Y lanzar la excepcion como una interrupcion por software */
	inline_assembly(" int $32");

	/* Esta tarea nunca termina!. Por esta razon, debe entrar en un ciclo
		infinito. */
	while(1);

	/* Este mensaje nunca deberia aparecer, ya que se entra en un ciclo
	 * infinito.*/
	printf("Kernel finished\n");

}
