/*
 * timer.h
 * Este archivo contiene las definiciones necesarias para el manejo del
 * Programmable Interrupt Timer (PIT) 8253/8254.
 */

#ifndef TIMER_H_
#define TIMER_H_

#include <asm.h>

/* Numero de IRQ para el timer: En la arquitectura x86 le corresponde
 * la IRQ 0*/
#define TIMER_IRQ 0

/*
 * Rutina: setup_timer
 * Descripcion: Esta rutina configura la frecuencia con la cual el timer
 * lanzara la interrupcion, y ademas define la rutina de manejo de IRQ
 * del timer.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_timer();

/*
 * Rutina: current_tick
 * Descripcion: Retorna el valor del contador de ticks de reloj del kernel.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	unsigned int:	Numero de ticks de reloj desde el arranque del sistema
 * */
unsigned int current_tick();

/*
 * Rutina: sdelay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de segundos. Para realizar la demora, usa la funcion auxiliar delay_loop,
 * que realiza una espera activa. Su precision depende del calculo del numero de
 * veces que se puede ejecutar la funcion delay_loop en un tick de reloj. Debido
 * a su baja precision, se limita a esperas menores o iguales a 256.
 * Entrada:
 * 	unsigned char : Numero de segundos (0-255)
 * Salida:
 * 	Ninguno.
 * */
void sdelay(unsigned char);
/*
 * Rutina: delay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de milisegundos. Para realizar la demora, usa la funcion auxiliar delay_loop,
 * que realiza una espera activa. Su precision depende del calculo del numero de
 * veces que se puede ejecutar la funcion delay_loop en un tick de reloj.
 * Entrada:
 * 	unsigned char : Numero de milisegundos
 * Salida:
 * 	Ninguno.
 * */
void delay(unsigned int);

/*
 * Rutina: udelay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de microsegundos. Para realizar la demora, usa la funcion auxiliar
 * delay_loop, que realiza una espera activa. Su precision depende del calculo
 * del numero de veces que se puede ejecutar la funcion delay_loop en un tick de
 * reloj.
 * Entrada:
 * 	unsigned char : Numero de microsegundos
 * Salida:
 * 	Ninguno.
 * */
void udelay(unsigned int);

/*
 * Rutina: ndelay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de nanosegundos. Para realizar la demora, usa la funcion auxiliar
 * delay_loop, que realiza una espera activa. Su precision depende del calculo
 * del numero de veces que se puede ejecutar la funcion delay_loop en un tick de
 * reloj.
 * Entrada:
 * 	unsigned char : Numero de nanosegundos
 * Salida:
 * 	Ninguno.
 * */
void ndelay(unsigned int);

/* Frecuencia por defecto en la cual se programa el PIT. Este es el
 * numero de ticks de reloj (IRQ, interrupciones) que se generan en un
 * segundo */
#define TIMER_FREQUENCY 1000 /* ticks / segundo*/


#endif /* TIMER_H_ */
