/* Archivo que contiene algunas constantes necesarias para el bootsector y el
 kernel */

#ifndef MULTIBOOT_H_
#define MULTIBOOT_H_

/* Direccion fisica del kernel en memoria */
#define KERNADDR 0x100000

/* Estas FLAGS se pasan a GRUB. Ver especificacion Multiboot. */

/* Alinear el kernel y los modulos cargados a limites de 4 KB*/
#define MULTIBOOT_PAGE_ALIGN 1<<0
/* Suministrar informacion de la memoria disponible*/
#define MULTIBOOT_MEMORY_INFO 1<<1
/* Suministrar informacion de los modos de video soportados (no se utiliza)*/
#define MULTIBOOT_VIDEO_INFO 1<<2
/* Suministrar informacion de las secciones del archivo binario (no se usa)*/
#define MULTIBOOT_AOUT_KLUDGE 1<<16

/* Numero magico que debe estar en el encabezado del archivo*/
#define MULTIBOOT_HEADER_MAGIC 0x1BADB002

/* Construir el campo 'flags' que se pasa como parametro a GRUB*/
#define MULTIBOOT_HEADER_FLAGS MULTIBOOT_PAGE_ALIGN | MULTIBOOT_MEMORY_INFO | \
							   MULTIBOOT_AOUT_KLUDGE

/* Checksum: GRUB utiliza este valor para verificar un encabezado valido */
#define MULTIBOOT_CHECKSUM -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)

/* Numero magico pasado por GRUB al kernel. Sirve para que el kernel compruebe
 * si fue cargado por un cargador compatible con la Especificacion Multiboot.*/
#define MULTIBOOT_BOOTLOADER_MAGIC 0x2BADB002

/* No incluir de aqui en adelante si se incluye este archivo desde codigo
 * en ensamblador */
#ifndef ASM


/* Encabezado multiboot */
typedef struct multiboot_header_struct {
	unsigned int magic;
	unsigned int flags;
	unsigned int checksum;
	unsigned int header_addr;
	unsigned int kernel_start;
	unsigned int data_end;
	unsigned int bss_end;
	unsigned int entry_point;
} multiboot_header_t;

/* Tabla de simbolos (kernel en formato a.out)*/
typedef struct aout_symbol_table
{
	unsigned int tabsize;
	unsigned int strsize;
	unsigned int addr;
	unsigned int reserved;
} aout_symbol_table_t;

/* Tabla de encabezados de seccion (kernel en formato elf)*/
typedef struct elf_section_header_table
{
	unsigned int num;
	unsigned int size;
	unsigned int addr;
	unsigned int shndx;
}elf_section_header_table_t;

/* "The memory map. Be careful that the offset 0 is base_addr_low
  but no size". */
typedef struct memory_map
{
 unsigned int entry_size;
 unsigned int base_addr_low;
 unsigned int base_addr_high;
 unsigned int length_low;
 unsigned int length_high;
 unsigned int type;
} memory_map_t;

typedef struct mod_info  {
	unsigned int mod_start;
	unsigned int mod_end;
	char * string;
	unsigned int always0;
}mod_info_t;

typedef struct {
	unsigned int flags;
	unsigned int mem_lower;	/* Presente si flags[0] = 1 */
	unsigned int mem_upper;	/* Presente si flags[0] = 1 */
	unsigned int boot_device;	/* Presente si flags[1] = 1 */
	unsigned int cmdline;		/* Presente si flags[2] = 1 */
	unsigned int mods_count;	/* Presente si flags[3] = 1 */
	unsigned int mods_addr;	/* Presente si flags[3] = 1 */
	union {
		aout_symbol_table_t aout_symbol_table;
		elf_section_header_table_t elf_section_table;
	}syms;						/* Presente si flags[4] = 1 o flags[5] = 1*/

	unsigned int mmap_length;	/* Presente si flags[6] = 1 */
	unsigned int mmap_addr;	/* Presente si flags[6] = 1 */

	unsigned int drives_length;/* Presente si flags[7] = 1 */
	unsigned int drives_addr;	/* Presente si flags[7] = 1 */

	unsigned int config_table;		/* Presente si flags[8] = 1 */
	unsigned int boot_loader_name;	/* Presente si flags[9] = 1 */

	unsigned int apm_table;	/* Presente si flags[10] = 1 */

	unsigned int vbe_control_info;	/* Presente si flags[11] = 1*/
	unsigned int vbe_mode_info;	/* Presente si flags[11] = 1 */
	unsigned int vbe_mode;			/* Presente si flags[11] = 1 */

	unsigned short vbe_interface_seg;	/* Presente si flags[11] = 1 */
	unsigned short vbe_interface_off;	/* Presente si flags[11] = 1 */
	unsigned short vbe_interface_len;	/* Presente si flags[11] = 1 */

}multiboot_info_t;

/*
 * Rutina: get_available_memory
 * Descripcion: Determina la region de memoria de mayor tamanio que se
 * encuentra por encima del kernel. Usa la informacion enviada por el
 * cargador de arranque compatible con la especificacion multiboot.
 * Entrada:
 * 	multiboot_header_t : Variable que contiene el encabezado multiboot
 *  multiboot_info_t * : Apuntador a la estructura de informacion
 *  					 multiboot pasada al kernel por el cargador de arranque
 *  unsigned int *    : Referencia a la variable en la cual se almacenara la
 *  				     direccion de memoria de la region disponible
 *  unsigned int *    : Referencia a la variable en la cual se almacenara
 *						 el tamanio de la region disponible encontrada
 * Salida:
 * 	Ninguno.
 * En caso de no encontrar una region disponible, los dos parametros
 * unsigned int * se establecen a cero.
 * */
void get_available_memory(multiboot_header_t ,
		multiboot_info_t *,
		unsigned int * , unsigned int * );

#endif

#endif
