/*
 * pm.h
 * Este archivo contiene las definicion de la GDT y las rutinas para su
 * manejo.
 * La GDT es la principal estructura de datos que controla la ejecucion de
 * un procesador x86 en modo protegido. La GDT contiene una serie de entradas,
 * llamadas descriptores de segmentos. Cada descriptor de segmento define
 * las direcciones base y limite de un segmento, su tipo y sus opciones de
 * acceso.
 */

#ifndef PM_H_
#define PM_H_

/* Definiciones para la Tabla Global de Descriptores (GDT) */

/*Este valor define el numero de entradas de la GDT */
#define MAX_GDT_ENTRIES  1024

/* Tipo de segmento de codigo:  = 1010 : Code, Non-Conformant,  Readable,
 * Not Accessed*/
#define CODE_SEGMENT_TYPE 0x0A

/* Tipo de segmento de datos:  = 0011 :Data, Read / Write, Accessed */
#define DATA_SEGMENT_TYPE 0x03

/* Desplazamiento en el GDT en el cual se encuentra el selector del segmento de
codigo del kernel */
#define KERNEL_CODE_SELECTOR 0x08

/* Desplazamiento en el GDT en el cual se encuentra el selector del segmento de
datos del kernel */
#define KERNEL_DATA_SELECTOR 0x10

/* Definir los niveles de privilegio para los segmentos */
#define RING0_DPL 0
#define RING1_DPL 1
#define RING2_DPL 2
#define RING3_DPL 3

#define KERNEL_STACK_TOP 0x9FC00

/* Dado que este archivo puede ser incluido desde codigo en Assembler, incluir
 * solo las constantes definidas anteriormente. */

#ifndef ASM

/* Cada entrada de la Tabla Global de Descriptores (GDT) tiene los siguientes
 * campos (Ver documento de Intel: en la seccion General Segment Descriptor
 * Format) */

 struct segment_descriptor {
	unsigned limit_low			: 16, /* limite 0..15*/
		     base_low			: 16, /* base 0..16 */
		     base_middle		: 8,  /* base 16..23*/
		     type 				: 4,  /* Informacion de acceso */
		     code_or_data		: 1, /* 1 = codigo o datos, 0 = sistema */
		     dpl				: 2, /* Nivel de privilegio */
		     present			: 1, /* Presente */
		     limit_high			: 4, /* limite 16..19*/
		     available			: 1, /* Disponible */
		     reserved			: 1, /* Reservado */
		     big				: 1, /* 0 = 16 bits, 1 = 32 bits */
		     granularity		: 1, /* 0 = bytes, 1 = 4096 bytes */
		     base_high			: 8; /* base 24..31*/
}__attribute__((packed)); /* Evitar posible alineacion del compilador */

typedef struct segment_descriptor segment_descriptor;

/* Variables definidas en pm.c para  los selectores de los descriptores
 * de codigo y datos para el kernel */

extern unsigned short kernel_data_selector;
extern unsigned short kernel_code_selector;

/* Estructura de datos para el registro GDTR (puntero a la GDT) */
struct gdt_pointer_t {
	unsigned short limit; /* Tamanio de la GDT */
	unsigned int base; /* Direccion lineal del inicio de la GDT */
} __attribute__ ((packed));

typedef struct gdt_pointer_t gdt_ptr;

/*
 * Rutina: setup_gdt
 * Descripcion: Esta rutina se encarga de configurar la tabla global de
 * descriptores (GDT) y cargarla mediante la instruccion LGDT. Luego pasa de
 * nuevo a modo protegido.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_gdt(void);

/*
 * Rutina: allocate_gdt_entry
 * Descripcion: Esta rutina permite obtener un indice de descriptor de segmento
 * libre dentro de la GDT, diferente al descriptor nulo.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	int : indice de la entrada disponible en el GDT
 * */
int allocate_gdt_entry(void);


/* Rutina: code32_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor de segmento de
 * codigo de 32 bits.
 * Entrada:
 * 	unsigned int: Base del segmento
 *  unsigned int: Limite del segmento
 *  unsigned char: Nivel de privilegios
 * Salida:
 *  segment_descriptor : Descriptor de segmento. */
static __inline__ segment_descriptor code32_segment_descriptor(
		unsigned int base,
		unsigned int limit,
		unsigned char dpl) {

		return (segment_descriptor) {
			(limit & 0x0000FFFF),	/* limite 0..15 */
			(base & 0x0000FFFF), 	/* base 0..15 */
			(base >> 16 & 0xFF), 	/* base 16..23 */
			CODE_SEGMENT_TYPE,	 	/* Informacion de acceso */
			1,						/*  Code or data */
			dpl & 0x03, 			/* Nivel de privilegio */
			1, 						/* Presente */
			(limit >> 16 & 0x0F),	/* limite 16..19*/
			0,						/* Disponible */
			0, 						/* Reservado */
			1, 						/* 0 = 16 bits, 1 = 32 bits */
			1, 						/* 0 = bytes, 1 = 4096 bytes */
			(base >> 24 & 0xFF) 	/* base 24..31*/
		};
}

/* Rutina: data32_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor de segmento de
 * datos de 32 bits.
 * Entrada:
 * 	unsigned int: Base del segmento
 *  unsigned int: Limite del segmento
 *  unsigned char: Nivel de privilegios
 * Salida:
 *  segment_descriptor : Descriptor de segmento. */
static __inline__ segment_descriptor data32_segment_descriptor(
		unsigned int base,
		unsigned int limit,
		unsigned char dpl) {

		return (segment_descriptor) {
			(limit & 0x0000FFFF),	/* limite 0..15 */
			(base & 0x0000FFFF), 	/* base 0..15 */
			(base >> 16 & 0xFF), 	/* base 16..23 */
			DATA_SEGMENT_TYPE,	 	/* Informacion de acceso */
			1,						/*  Code or data */
			dpl & 0x03, 			/* Nivel de privilegio */
			1, 						/* Presente */
			(limit >> 16 & 0x0F),	/* limite 16..19*/
			0,						/* Disponible */
			0, 						/* Reservado */
			1, 						/* 0 = 16 bits, 1 = 32 bits */
			1, 						/* 0 = bytes, 1 = 4096 bytes */
			(base >> 24 & 0xFF) 	/* base 24..31*/
		};
}

/* Rutina: null_segment_descriptor
 * Descripcion: Esta rutina permite configurar un descriptor desegmento nulo.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ segment_descriptor null_segment_descriptor(void) {

		return (segment_descriptor) {
			0,	/* limite 0..15 */
			0, 	/* base 0..15 */
			0, 	/* base 16..23 */
			0,	/* Informacion de acceso */
			0,	/*  Code or data */
			0, 	/* Nivel de privilegio */
			0, 	/* Presente */
			0,	/* limite 16..19*/
			0,	/* Disponible */
			0, 	/* Reservado */
			0, 	/* 0 = 16 bits, 1 = 32 bits */
			0, 	/* 0 = bytes, 1 = 4096 bytes */
			0 	/* base 24..31*/
		};
}

/*
 * Rutina: gdt_selector
 * Descripcion: Construye un selector de la GDT, a partir del indice de la
 * entrada y el nivel de privilegios especificado.
 * Entrada:
 * 	unsigned short: indice de la entrada de la GDT
 *  unsigned char:  nivel de privilegio del selector
 * Salida:
 *  unsigned short : selector.
 * */
static __inline__ unsigned short gdt_selector(unsigned short index,
		unsigned char rpl) {
	unsigned short ret;


	/*
	 Un selector tiene el siguiente formato:
15                        4 3    0
+-------------------------+-+---+
| 						  |T|   |
| 		INDEX 			  | |RPL|
| 						  |I|   |
+-------------------------+-+---+
	 *
	 * Para el caso de un selector de la GDT, TI siempre es 0.
	 * */

	/* Enmascarar los bits de acuerdo con el formato del selector */
	ret = (index << 3) | rpl & 0x03;
	return ret;
}

/*
 * Rutina: setup_gdt_descriptor
 * Descripcion: Esta rutina publica permite establecer una entrada en la GDT
 * con un nuevo descriptor, diferente a la entrada nula y a las entradas para
 * los descriptores de segmento de codigo y datos para el kernel.
 * Entrada:
 * 	unsigned short: indice del descriptor a configurar
 * Salida:
 * 	unsigned short: selector del descriptor configurado.
 * */
 unsigned short setup_gdt_descriptor(unsigned short, segment_descriptor );

/*
 * Rutina: print_gdt_descriptor
 * Descripcion: Imprime un descriptor de la GDT.
 * Entrada:
 * 	int : Entrada a imprimir
 * Salida:
 * 	Ninguno.
 * */
void print_gdt_descriptor(int );

#endif /* ASM */

#endif /* PM_H_ */
