/*
 * irq.c
 * Este archivo contiene la implementacion de las rutinas necesarias para
 * manejar las solicitudes de interrupcion (IRQ) de los dispositivos de
 * entrada / salida.
 * */

#include <idt.h>
#include <irq.h>
#include <stdio.h>
#include <stdlib.h>

/* Primitivas basicas para el manejo de IRQ */
/* OCW2 (End of Interrupt): Codigo para escribir en el puerto de comandos del
 * PIC para indicar que se ha recibido la interrupcion.
 */
#define EOI 0x20

/* Direccion del puerto de comandos del PIC  maestro*/
#define MASTER_PIC_COMMAND_PORT 0x20

/* Direccion del puerto de comandos del PIC esclavo */
#define SLAVE_PIC_COMMAND_PORT 0xA0

/* Direccion del puerto de datos del PIC maestro*/
#define MASTER_PIC_DATA_PORT 0x21

/* Direccion del puerto de datos del PIC esclavo */
#define SLAVE_PIC_DATA_PORT 0xA1

/* Arreglo que contiene los apuntadores a las rutinas de manejo de interrupcion
 */
irq_handler irq_routines[MAX_IRQ_ROUTINES];

/*
 * Rutina: irq_dispatcher
 * Descripcion: Esta rutina recibe el control de la rutina de manejo de
 * interrupcion y la canaliza hacia el manejador de IRQ correspondiente
 * a partir del numero de interrupcion.
 * Entrada:
 * 	Ninguno.
 * Salida:
 *	Ninguno.
 * */
void irq_dispatcher();

/*
 * Rutina: irq_remap
 * Descripcion: Al arranque, las IRQ 0 a 7 estan mapeadas a las interrupciones
 * 0x8 - 0xF. Estas entradas estan asignadas por defecto a las excepciones x86
 * (la entrada 8 es Double Fault, por ejemplo).
 * Por esta razon es necesario mapear las IRQ a otros numeros de interrupcion.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void irq_remap(void);


/* Implementacion de las rutinas */

/*
 * Rutina: irq_remap
 * Descripcion: Al arranque, las IRQ 0 a 7 estan mapeadas a las interrupciones
 * 0x8 - 0xF. Estas entradas estan asignadas por defecto a las excepciones x86
 * (la entrada 8 es Double Fault, por ejemplo).
 * Por esta razon es necesario mapear las IRQ a otros numeros de interrupcion.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void irq_remap(void) {
	/* Reprogramacion del PIC (ver archivo readme.txt) */

	/* Initialization Command Word 1 - ICW1
	Esta es la palabra primaria para inicializar el PIC.
	Para inicializar el PIC se requiere que los bits 0 y 4 de ICW1 esten en
	1 y los demas en 0. Esto significa que el valor de ICW1 es 0x11.
	ICW1 debe ser escrita en el  registro de comandos del PIC maestro
	(direccion de e/s 0x20). Si existe un PIC esclavo, ICW1 se debe enviar
	tambien su registro de comandos (0xA0)
	*/

	outb(MASTER_PIC_COMMAND_PORT, 0x11);
	outb(SLAVE_PIC_COMMAND_PORT, 0x11);

	/* Initialization Command Word 2 - ICW2
	Esta palabra permite definir la direccion base (inicial) en la tabla de
	descriptores de interrupcion que el PIC va a utilizar.

	Debido a que las primeras 32 entradas estan reservadas para las
	excepciones en la arquitectura x86, ICW2 debe contener un valor mayor o
	igual a 32 (0x20). Los valores  de ICW2 representan el numero de IRQ
	base que manejara el PIC

	Al utilizar los PIC en cascada, se debe enviar ICW2 a los dos
	controladores en su registro de datos (0x21 y 0xA1 para maestro y
	esclavo respectivamente), indicando la direccion en la IDT que va a ser
	utilizada por cada uno de ellos.

	Las primeras 8 IRQ van a ser manejadas por el PIC maestro y se mapearan
	a partir del numero 32 (0x20). Las siguientes 8 interrupciones las manejara
	el PIC esclavo, y se mapearan a partir de la interrupcion 40 (0x28).
	*/

	outb(MASTER_PIC_DATA_PORT, IDT_IRQ_OFFSET);
	outb(SLAVE_PIC_DATA_PORT, IDT_IRQ_OFFSET + 8);

	/* Initialization Control Word 3 - ICW3
	Esta palabra permite definir cuales lineas de IRQ van a ser compartidas
	por los PIC maestro y esclavo. Al igual que ICW2, ICW3 tambien se
	escribe en los 	registros de datos de los PIC (0x21 y 0xA1 para el PIC
	maestro y esclavo,respectivamente).

	Dado que en la arquitectura Intel el PIC maestro se conecta con el PIC
	esclavo por medio de la linea IRQ 2, el valor de ICW3 debe ser 00000100
	(0x04), que define el bit 3 (correspondiente a la linea IRQ2) en 1.

	Para el PIC esclavo, el numero de la linea se debe representar en
	notacion binaria. Por lo tanto, 000 corresponde a la linea de IRQ 0,
	001 corresponde a la linea de IRQ 1, 010 corresponde a la linea de
	IRQ 2, y asi sucesivamente.
	Debido a que se va a utilizar la linea de IRQ 2, el valor de ICW3
	para el PIC esclavo debe ser 00000010, (0x02).
	*/

	outb(MASTER_PIC_DATA_PORT, 0x04);
	outb(SLAVE_PIC_DATA_PORT, 0x02);

	/* Initialization Control Word 4 - ICW4
	Para ICW4 solo es necesario establecer su bit 0 (x86 mode) y escribirla
	en los registros de datos del PIC maestro y esclavo (0x21 y 0xA1).
	El valor de ICW4 debe ser entonces 00000001, es decir, 0x01.
	*/
	outb(MASTER_PIC_DATA_PORT, 0x01);
	outb(SLAVE_PIC_DATA_PORT, 0x01);

	/* Se han mapeado las IRQ!.
	 * Las IRQ 0-7 seran atendidas por el PIC maestro, y las IRQ 8-15
	 * por el PIC esclavo. Las IRQ0-15 estaran mapeadas en la IDT a partir
	 * de la entrada 32 hasta la 47.*/
}

/*
 * Rutina: setup_irq
 * Descripcion: Esta rutina se encarga de crear los manejadores de
 * interrupcion para las 16 IRQ en los procesadores x86. Estas IRQ
 * se re-mapean a las interrupciones con vector 32 .. 47.
 * Todas ellas son manejadas por la rutina 'irq_dispacther', que se encarga
 * de invocar la rutina de manejo de IRQ correspondiente, si se encuentra
 * definida.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void setup_irq(void) {
	int i;

	for (i=0; i<MAX_IRQ_ROUTINES;i++) {
		irq_routines[i] = NULL_INTERRUPT_HANDLER;
	}

	/* Mapear las IRQ 0..15 a las entradas 32 .. 47 de la IDT */
		irq_remap();

	/* Ahora configurar el manejador para las interrupciones re-mapeadas
	 * (32..47)
	 * Todas estas interrupciones son manejadas por la rutina irq_dispatcher */

	for (i=0; i<MAX_IRQ_ROUTINES; i++) {
		install_interrupt_handler(i + IDT_IRQ_OFFSET, irq_dispatcher);
	}
}

/*
 * Rutina: install_irq_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de IRQ
 * Entrada:
 * 	int: numero de irq a configurar
 *  irq_handler : Rutina a manejar la irq
 * Salida:
 * 	Ninguno. */
void install_irq_handler(int number, irq_handler handler){
	/* Simplemente sobre-escribir la rutina anterior, si existe.
	 * */
	if (number > MAX_IRQ_ROUTINES) {
		return;
	}
	irq_routines[number] = handler;
}

/*
 * Rutina: uninstall_irq_handler
 * Descripcion: Esta rutina permite quitar un  manejador de IRQ.
 * Entrada:
 * 	int: numero de irq a quitar
 * Salida:
 * 	Ninguno. */
void uninstall_irq_handler(int number) {

	if (number < MAX_IRQ_ROUTINES) {
		irq_routines[number] = NULL_INTERRUPT_HANDLER;
	}
}

/*
 * Rutina: irq_dispatcher
 * Descripcion: Esta rutina recibe el control de la rutina de manejo de
 * interrupcion y canaliza esta solicitud a la rutina de manejo de IRQ
 * correspondiente, si se encuentra definida.
 * Dentro de la estructura de datos que recibe, se puede obtener el numero
 * de la interrupcion que ocurrio, asi como el estado del procesador.
 * Con el numero de la interrupcion y restando IDT_IRQ_OFFSET se puede
 * obtener el numero de IRQ.
 * Entrada:
 * 	interrupt_state * : Apuntador al marco de interrupcion actual.
 * Salida:
 *	Ninguno.
 * */
void irq_dispatcher(interrupt_state * state) {

	irq_handler handler;

	int index;

	/* Determinar el numero de la IRQ */
	index = state->number - IDT_IRQ_OFFSET;

	//printf("IRQ %d (interrupt %d) launched!\n", index, state->number);

	/*
	 * Enviar EOI al puerto de control del 8259 que lanzo la interrupcion
	 * */

	/* Si la IRQ es 8..15, se debe enviar EOI al 8259 esclavo tambien. */
	if (index >= 8) {
		outb(SLAVE_PIC_COMMAND_PORT, EOI);
	}
	outb(MASTER_PIC_COMMAND_PORT, EOI);

	/* Buscar la rutina que maneja la interrupcion */
	handler = irq_routines[index];

	/* Si la rutina existe, ejecutarla y pasarle como parametro los
	 * registros.*/
	if (handler != NULL_INTERRUPT_HANDLER) {
			handler(state);
			printf(" EOI");
	}else {
		printf(" Warning! unhandled IRQ %d (INT %d)\n", index, state->number);
	}
}
