/*
 * timer.c
 * Este archivo implementa las rutinas necesarias para el manejo del
 * Programmable Interrupt Timer (PIT) 8253/8254.
 */

#include <idt.h>
#include <irq.h>
#include <timer.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <task.h>

/* El PIT oscila por defecto a 1193180 (0x1234dc) Hz. */
#define CLOCK_TICK_RATE 0x1234DC

/* Puerto para el contador 0 del timer (reloj) */
#define TIMER_COUNTER_0_PORT 0x40

/* Puerto de control del modo del timer */
#define TIMER_CONTROL_PORT 0x43

/*  Palabra de control para inicializar el timer
 *
 *  0x34 = 00110100 (Select counter 0, read LSB first then MSB, Mode 2,
 *  binary counter 16-bits */
#define TIMER_CONTROL_WORD 0x34

/* Macro para calcular el divisor de frecuencia */
#define TIMER_DIVISOR(x)  ((CLOCK_TICK_RATE+(x)/2)/(x))

/* Numero de ticks por segundo = frecuencia del timer */
#define TICKS_PER_SEC TIMER_FREQUENCY

/* Variable para almacenar el divisor de frecuencia configurado en el
 * timer.*/
unsigned short timer_divisor = 0;

/* Variable para almacenar la frecuencia configurada en el timer, por
 * defecto 100 hz = 10 interrupciones por segundo */
unsigned short timer_frequency = 0;

/* Variable que almacena el numero de ticks (interrupciones de reloj)*/

unsigned int kernel_ticks = 0;

/*
 * Rutina: timer_handler
 * Descripcion: Esta rutina se ejecuta cada vez que se recibe la IRQ0
 * (interrupcion 32), que corresponde al timer.
 * */
void timer_handler(interrupt_state *);

/*
 * Rutina: delay_loop
 * Descripcion: Esta rutina realiza un ciclo simple. Se usa para calibrar
 * la velocidad aproximada del procesador.
 * Entrada:
 * 	unsigned int : Numero de ciclos a ejecutar
 * Salida:
 * 	Ninguno.
 * */

void delay_loop(unsigned int );

/* Variable que se utiliza para determinar el numero de veces que se
 * puede ejecutar la funcion delay_loop en un tick de reloj. Su valor
 * es determinado en la rutina calibrate_timer */
unsigned int loops_per_tick;

/* Cantidad de veces que se puede ejecutar la funcion delay_loop en un segundo.
 * Depende del valor de loops_per_tick
 * */
unsigned int loops_per_sec;

/* Cantidad de veces que se puede ejecutar la funcion delay_loop en un
 * milisegundo. Depende del valor de loops_per_tick */
unsigned int loops_per_msec;

/*
 * Cantidad de veces que se puede ejecutar la funcion delay_loop en un
 * microsegundo. Depende del valor de loops_per_tick
 * */
unsigned int loops_per_usec;

/*
 * Cantidad de veces que se puede ejecutar la funcion delay_loop en un
 * nanosegundo. Depende del valor de loops_per_tick
 * */
unsigned int loops_per_nsec;

/* Suponer que la rutina delay_loop se puede ejecutar por lo menos
 * una vez en cada precision. Dentro de la funcion calibrate_timer se establecen
 * los valores reales para cada precision. */

int msec_precision = 1; /* Precision en milisegundos */
int usec_precision = 1; /* Precision en microsegundos */
int nsec_precision = 1; /* Precision en nanosegundos */


/*
 * Rutina: calibrate_timer
 * Descripcion: Esta rutina determina el numero aproximado de veces que
 * se puede ejecutar la rutina 'delay_loop' en un tick de reloj.
 * Con ello se actualiza la variable global 'loops_per_tick'
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void calibrate_timer();

/*
 * Rutina: setup_timer
 * Descripcion: Esta rutina configura la frecuencia con la cual el timer
 * lanzara la interrupcion, y ademas define la rutina de manejo de interrupcion
 * del timer.
 * Entrada:
 * 	Nunguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_timer() {
  /* Configuracion del timer
  * Primero se establece la frecuencia con la cual el timer lanzara la
  * interrupcion */

  /* enviar  0x34 = 00110100  al puerto de comandos del PIT (0x43) */
  outb(TIMER_CONTROL_PORT, TIMER_CONTROL_WORD);

  timer_frequency = TIMER_FREQUENCY;

  timer_divisor = TIMER_DIVISOR(timer_frequency);

  /* Enviar el byte menos significativo al puerto 0x40 (counter 0) del pit */
  outb(TIMER_COUNTER_0_PORT,  timer_divisor & 0xFF);
  /* Enviar el byte mas significativo al puerto 0x40 (counter 0) del pit */
  outb(TIMER_COUNTER_0_PORT,( timer_divisor >> 8 ) & 0xFF);

  /* Ahora configurar la rutina de manejo de interrupcion del timer.
   * Se debe recordar que la IRQ0 (asociada al timer) se mapeo a la interrupcion
   * numero 32.*/
  install_irq_handler(TIMER_IRQ, timer_handler);

  /* Importante! se asume que en este punto las interrupciones se encuentran
   * deshabilitadas */
  sti();

  /* Calibrar el timer */

  calibrate_timer();

  cli();

}

/*
 * Rutina: current_tick
 * Descripcion: Retorna el valor del contador de ticks de reloj del kernel.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	unsigned int:	Numero de ticks de reloj desde el arranque del sistema
 * */
unsigned int current_tick() {
	return kernel_ticks;
}

/*
 * Rutina: timer_handler
 * Descripcion: Esta funcion recibe el control de la rutina de manejo
 * de interrupcion del timer. Cada vez que se ejecuta, aumenta en 1 el contador
 * para el numero de ticks (interrupciones de reloj).
 * Entrada:
 * 	interrupt_state : Estructura de datos que contiene el estado de los
 * 					  registros del procesador.
 * Salida:
 *  Ninguno.
 * */
void timer_handler(interrupt_state * state) {
	extern task * current_task;
	kernel_ticks++;

	/*
	if (kernel_ticks % timer_frequency == 0) {
		printf("%u ", kernel_ticks / timer_frequency);
	}
	*/

	/* Si existe una tarea en ejecucion, verificar si se debe pasar el
	 * control a otra tarea. */
	if (current_task != NULL_TASK) {
		current_task->timeslice++;
		task_dispatcher();
	}
}

/*
 * Rutina: delay_loop
 * Descripcion: Esta rutina realiza un ciclo simple. Se usa para calibrar
 * la velocidad aproximada del procesador.
 * Entrada:
 * 	unsigned int : Numero de ciclos a ejecutar
 * Salida:
 * 	Ninguno.
 * */
void delay_loop(unsigned int loops) {
	inline_assembly(
			"	or %1, %1	\n"
			"	jz 2f		\n"
			"	jmp 1f		\n"
			".align 16		\n"
			"1: inc %0		\n"
			"	cmp %0, %1	\n"
			"	jnz 1b		\n"
			"2: nop"
			:
			:"b"(0), "c"(loops)
			:"0", "1"
			);

}

/*
 * Rutina: calibrate_timer
 * Descripcion: Esta rutina determina el numero aproximado de veces que
 * se puede ejecutar la rutina 'delay_loop' en un tick de reloj.
 * Con ello se actualiza la variable global 'loops_per_tick'
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void calibrate_timer() {
	unsigned int start_tick;
	unsigned int end_tick;
	unsigned int remainder;

	/* Determinar la mayor potencia de 2 de loops_per_tick.
	 * Maximo: 32 iteraciones para un procesador de 32 bits
	 * Se inicia con 1 = 2^0*/
	loops_per_tick = 1;

	do {
		/* Esperar al inicio de un tick */
		start_tick = kernel_ticks;
		while (start_tick == kernel_ticks);

		/* Medir desde el inicio del tick */
		start_tick = kernel_ticks;
		/* Ejecutar la funcion auxiliar delay_loop*/
		delay_loop(loops_per_tick);
		end_tick = kernel_ticks;

		/* Verificar si ejecutar la rutina delay_loop loops_per_tick veces
		 * toma mas de un tick*/
		if (start_tick != end_tick) {
			break;
		}
	}while ((loops_per_tick *= 2)!=0 );

	/* Tomar la anterior potencia de 2, que es la que se aproxima por debajo
	 * al numero de loops por tick */
	loops_per_tick /= 2;

	/* Tratar de aproximar loops_per_tick sumando potencias de
	 * 2 menores o iguales a loops_per_tick / 2 */
	remainder = loops_per_tick / 2;

	do {
		loops_per_tick += remainder;
		/* Esperar al inicio de un tick */
		start_tick = kernel_ticks;
		while (start_tick == kernel_ticks);

		start_tick = kernel_ticks;
		delay_loop(loops_per_tick);
		end_tick = kernel_ticks;

		if (start_tick != end_tick) {
			loops_per_tick -= remainder;
		}
		remainder /= 2;
	}while (remainder > 0);

	/* loops_per_tick contiene el valor aproximado del numero de veces
	 * que se puede ejecutar la funcion delay_loop en un tick de reloj. */

	//printf("loops per tick: %u\n", loops_per_tick);

	/* Ahora calcular los valores del numero de loops por segundo,
	 *  por milisegundo, microsegundo y nanosegundo. Tambien se debe
	 *  calcular la resolucion minima para cada unidad. */

	loops_per_sec =	TICKS_PER_SEC * loops_per_tick;
	/* La funcion delay_loop se deberia ejecutar al menos una vez en un
	 * segundo, ya que TICKS_PER_SEC es mayor que 1 */
	if (loops_per_sec == 0) {
		printf(" Error! cannot calculate loops per second!");
		return;
	}

	loops_per_msec = loops_per_sec / 1000;
	/* No alcanza a ejecutarse un loop en un milisegundo? */
	if (loops_per_msec == 0) {
		msec_precision =loops_per_sec / 100;
		if (msec_precision == 0) {
			msec_precision =loops_per_sec / 10;
			/* No se puede calcular la resolucion en milisegundos*/
			if (msec_precision == 0) {
				msec_precision = 1000;
			}else {
				msec_precision++;
			}
		}else {
			msec_precision++;
		}
	}
	//printf("milisec precision: %d\n", msec_precision);

	loops_per_usec = loops_per_sec / 1000000;
	/* No alcanza a ejecutarse un loop en un microsegundo? */
	if (loops_per_usec == 0) {
		usec_precision = loops_per_sec / 100000;
		if (usec_precision == 0) {
			usec_precision = loops_per_sec / 10000;
			if (usec_precision == 0) {
				usec_precision = 1000;
			}else {
				usec_precision++;
			}
		}else {
			usec_precision++;
		}
	}
	//printf("microsec precision: %d\n", msec_precision);

	loops_per_nsec = loops_per_sec / 1000000000;
	/* No alcanza a ejecutarse un loop en un nanosegundo? */
	if (loops_per_nsec == 0) {
		nsec_precision = loops_per_sec / 100000000;
		if (nsec_precision == 0) {
			nsec_precision = loops_per_sec / 10000000;
			if (nsec_precision == 0) {
				nsec_precision = 1000;
			}else {
				nsec_precision++;
			}
		}else {
			nsec_precision++;
		}
	}
	//printf("nanosec precision: %d\n", nsec_precision);

}

/*
 * Rutina: sdelay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de segundos. Para realizar la demora, usa la funcion auxiliar delay_loop,
 * que realiza una espera activa. Su precision depende del calculo del numero de
 * veces que se puede ejecutar la funcion delay_loop en un tick de reloj. Debido
 * a su baja precision, se limita a esperas menores o iguales a 256.
 * Entrada:
 * 	unsigned char : Numero de segundos (0-255)
 * Salida:
 * 	Ninguno.
 * */
void sdelay(unsigned char secs) {
	unsigned int loops;

	/* Calcular el numero de loops que se deben ejecutar */
	loops = secs * TICKS_PER_SEC; /* segundos * (ticks/segundo) */

	while (loops > 0) { /* Iterar el numero de ticks necesario */
		delay_loop(loops_per_tick); /* Demorarse un tick */
		loops--;
	}
}

/*
 * Rutina: delay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de milisegundos. Para realizar la demora, usa la funcion auxiliar delay_loop,
 * que realiza una espera activa. Su precision depende del calculo del numero de
 * veces que se puede ejecutar la funcion delay_loop en un tick de reloj.
 * Entrada:
 * 	unsigned char : Numero de milisegundos
 * Salida:
 * 	Ninguno.
 * */
void delay(unsigned int milisecs){

	int i;
	unsigned int loops;

	if (milisecs >= 1000) {
		sdelay(milisecs / 1000);
		milisecs = milisecs % 1000;
	}

	if (milisecs == 0) {return;}
	if (milisecs < msec_precision) {
		milisecs = msec_precision;
	}
	/* Calcular el numero de ciclos que se deben ejecutar en los
	 * milisegundos especificados */


	/* Calcular el numero de loops que se deben ejecutar */
	loops = (milisecs + msec_precision) * loops_per_msec;

	delay_loop(loops); // Demorarse el numero de loops calculados

}

/*
 * Rutina: udelay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de microsegundos. Para realizar la demora, usa la funcion auxiliar
 * delay_loop, que realiza una espera activa. Su precision depende del calculo
 * del numero de veces que se puede ejecutar la funcion delay_loop en un tick de
 * reloj.
 * Entrada:
 * 	unsigned char : Numero de microsegundos
 * Salida:
 * 	Ninguno.
 * */
void udelay(unsigned int usecs){

	int i;
	unsigned int loops;

	if (usecs >= 1000) {
		delay(usecs / 1000);
		usecs = usecs % 1000;
	}

	if (usecs == 0) {return;}
	if (usecs < usec_precision) {
		usecs = usec_precision;
	}
	/* Calcular el numero de ciclos que se deben ejecutar en los
	 * microsegundos especificados */


	loops = (usecs  + usec_precision) * loops_per_usec;

	delay_loop(loops); // Demorarse el numero de loops calculados

}

/*
 * Rutina: ndelay
 * Descripcion: Esta rutina permite realizar demoras en un numero determinado
 * de nanosegundos. Para realizar la demora, usa la funcion auxiliar
 * delay_loop, que realiza una espera activa. Su precision depende del calculo
 * del numero de veces que se puede ejecutar la funcion delay_loop en un tick de
 * reloj.
 * Entrada:
 * 	unsigned char : Numero de nanosegundos
 * Salida:
 * 	Ninguno.
 * */
void ndelay(unsigned int nsecs){

	int i;
	unsigned int loops;

	if (nsecs >= 1000) {
		udelay(nsecs / 1000);
		nsecs = nsecs % 1000;
	}

	if (nsecs == 0) {return;}
	if (nsecs < nsec_precision) {
		nsecs = nsec_precision;
	}
	/* Calcular el numero de ciclos que se deben ejecutar en los
	 * nanosegundos especificados */


	loops = (nsecs + nsec_precision) * loops_per_nsec ;
	//printf("Loops: %u lpt: %u", loops, loops_per_tick);


	delay_loop(loops); // Demorarse el numero de loops calculados

}
