/*
 * task.h
 * Este archivo contiene las definiciones de las estructuras de datos y las
 * rutinas para gestionar tareas con diferentes niveles de privilegios.
 *
 * kernel.
 */

#ifndef TASK_H_
#define TASK_H_

#include <pm.h>
#include <idt.h>

#define TASK_STACK_SIZE 4096
#define TASK_HEAP_SIZE 8192

/* Estados de una tarea */
typedef enum task_state {
	TASK_AVAILABLE = 0,
	TASK_READY = 1,
	TASK_RUNNING = 2,
	TASK_BLOCKED = 3,
	TASK_FINISHED = 4,
	TASK_SCHEDULED = 5
} task_state;

/*
 * Quantum: numero de ticks de reloj que se asignan a una tarea en forma
 * continua. Luego de este numero de ticks, la CPU se asigna a otra tarea.
 * */
/* Las tareas de kernel tienen un quantum de 20 ticks de reloj */
#define KERNEL_TASK_QUANTUM 50
/* Las tareas de usuario tienen un quantum de 10 ticks de reloj.*/
#define USER_TASK_QUANTUM 20


/* Estructura de datos para las tareas. Los campos que referencian a la
 * memoria se crean como apuntadores, para facilitar las operaciones
 * de memoria.
 *  */
typedef struct task {

	/* Identificador de la tarea */
	unsigned short pid;

	/* Identificador de la tarea padre */
	unsigned short ppid;

	/* Estado actual de la tarea */
	unsigned char state;

	/* Nivel de privilegio de la tarea */
	unsigned char pl;

	/* Desplazamiento de la rutina inicial de la tarea */
	unsigned int entry;

	/* Direccion lineal de memoria de inicio de la tarea */
	char * base;

	/* Direccion lineal de memoria del fin del codigo y los datos de la tarea*/
	char * end;

	/* Tamanio de la tarea. Incluye el espacio de heap y de pila. */
	unsigned int limit;

	/* Variable que contiene el valor del selector de pila */
	unsigned short current_ss;

	/* Variable que contiene el valor del apuntador al tope de la pila
	 * en el cual se encuentra el marco actual de interrupcion. El tope de la
	 * pila es relativo a la tarea, sin importar su PL. */
	unsigned int current_esp;

	/* Variable en la cual se almacena el numero de ticks de reloj que
	 * la tarea tiene la CPU */
	unsigned int timeslice;

	/* Variable que almacena el total de ticks asignados a la tarea */
	unsigned int total_ticks;

}task;

/* Referencia para una tarea nula */
#define NULL_TASK 0

/* funcion auxiliar para una tarea inicializada en 0 */
static __inline__ task null_task() {
	return (task) {0};
}

/* Numero maximo de tareas que pueden existir en un momento dado */
#define TASK_LIMIT 1024

/* Rutina: create_task
 * Descripcion: Crea una nueva tarea, a partir de un archivo ELF que se
 * encuentra leido en memoria.
 * Entrada:
 * char * : Argumentos que se deben pasar a la tarea
 * 	unsigned int : Direccion de inicio del archivo ELF en memoria
 * 	unsigned int : Direccion de fin del archivo ELF en memoria
 *  unsigned char : Nivel de privilegio de la tarea
 * */
task * create_task(char *, unsigned int, unsigned int, unsigned char);

/*
 * Rutina: sampe_dpl
 * Descripcion: Esta rutina permite determinar si dos tareas
 * se encuentran en el mismo nivel de privilegios.
 * Entrada:
 * 	task *, task *: Tareas a comparar
 * Salida:
 *  int: 1 = las tareas tienen el mismo PL, 0 = las tareas tienen diferente PL.
 * */
static __inline__ int same_pl(task * task1, task * task2) {
	return ((task1 != NULL_TASK && task2 != NULL_TASK) &&
			(task1->pl == task2->pl));
}


/*
 * Rutina: switch_to_task
 * Descripcion: Permite continuar la ejecucion de una tarea especificada.
 * Realiza los ajustes para cambiar de nivel de privilegio si es necesario.
 * Entrada:
 * 	task * : Tarea a convertirse en la tarea actual.
 * Salida:
 * 	Nunguno. Esta rutina no retorna, ya que la ejecucion continua en la
 * tarea especificada.
 * */
void switch_to_task(task *);


/*
 * Rutina: switch_to_idle_task
 * Descripcion: Esta rutina retorna el control a una tarea vacia del kernel,
 * cuyo objetivo es detener el procesador hasta que ocurra el siguiente tick de
 * reloj.
 * 	Entrada:
 * 	Ninguno.
 * Salida:
 * 	Nunguno. Esta rutina no retorna, ya que la ejecucion continua en la
 * rutina 'idle_task'.
 * */
void switch_to_idle_task(void);

/*
 * Rutina: task_dispatcher
 * Descripcion: Esta rutina busca la siguiente tarea que se debe ejecutar
 * y le entrega el control. Si no existe una tarea disponible,
 * le retorna el control a la rutina 'idle_task' como una tarea.
 * Entrada:
 * 	Ninguno.
 * Salida:
 *  Ninguno.
 * */
void task_dispatcher(void);

/*
 * Rutina: destroy_task
 * Descripcion: Esta rutina marca la tarea como terminada, y luego invoca
 * al despachador de tareas.
 * Entrada:
 * 	task * : Tarea a destruir
 * Salida:
 * 	Ninguno.
 * */
void destroy_task(task *);

#endif /* TASK_H_ */
