/*
 * exception.c
 * Este archivo implementa las primitivas necesarias para el manejo
 * de excepciones.
 */

#include <exception.h>


/* Estructura de datos para almacenar las rutinas que manejaran las
 * excepciones*/

exception_handler exception_handlers[MAX_EXCEPTIONS];

/*
 * Rutina: exception_dispatcher
 * Descripcion: Esta rutina recibe el control de la interrupt_dispatcher.
 * Su trabajo consiste en determinar el vector de interrupcion a partir del
 * estado que recibe como parametro, y de invocar la rutina de manejo de
 * excepcion adecuada, si existe.
 * Entrada:
 * 	Ninguno.
 * Salida:
 *	Ninguno.
 * */
void exception_dispatcher();

/* Excepciones dele procesador Intel*/
unsigned char *exceptions[] = {
	"Division By Zero",
	"Debug",
	"Non Maskable Interrupt",
	"Breakpoint",
	"Into Detected Overflow",
	"Out of Bounds",
	"Invalid Opcode",
	"No Coprocessor",
	"Double Fault",
	"Coprocessor Segment Overrun",
	"Bad TSS",
	"Segment Not Present",
	"Stack Fault",
	"General Protection Fault",
	"Page Fault",
	"Unknown Interrupt",
	"Coprocessor Fault",
	"Alignment Check",
	"Machine Check",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved",
	"Reserved"
};

/*
 * Rutina: setup_exceptions
 * Descripcion: Esta rutina crea un manejador de interrupcion para las
 * 32 excepciones x86 e inicializa la tabla de manejadores de excepcion.
 * Para todas las excepciones se establece la rutina 'exception_dispatcher'
 * como la rutina de manejo de interrupcion.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void setup_exceptions(void) {
	int i;

	for (i=0; i<MAX_EXCEPTIONS; i++) {
		exception_handlers[i] = NULL_INTERRUPT_HANDLER;
	}

	/* En la arquitectura Intel las excepciones tienen vectores (numero) 0..31
	 * Asi, el vector es el mismo numero de la excepcion*/

	for (i=0; i<MAX_EXCEPTIONS; i++) {
		install_interrupt_handler(i, exception_dispatcher);
	}
}


/*
 * Rutina: exception_dispatcher
 * Descripcion: Esta rutina recibe el control de la interrupt_dispatcher.
 * Su trabajo consiste en determinar el vector de interrupcion a partir del
 * contexto actual de interrupcion, y de invocar la rutina de manejo de
 * excepcion adecuada, si existe.
 * Entrada:
 * Ninguno.
 * Salida:
 *	Ninguno.
 * */
void exception_dispatcher(interrupt_state * state) {
	exception_handler handler;

	/* Buscar la rutina que maneja la excepcion. El numero
	 * de la interrupcion (0..31) es el mismo sub-indice en la
	 * tabla de manejadores de excepcion*/
	handler = exception_handlers[state->number];

	/* Si la rutina existe, ejecutarla y pasarle como parametro los registros.*/
	if (handler != NULL_INTERRUPT_HANDLER) {
			handler(state);
	} else {
		/* En caso contrario, informar que ocurrio una interrupcion
		 * que no tiene un manejador asociado.*/
		printf("x86 Exception [%d]: %s. System Halted!\n", state->number,
				exceptions[state->number]);
		/* Mostrar el estado de la interrupcion. */
		dump_interrupt_state(state);

		/* Bloquear el kernel cuando ocurre una excepcion que no tiene
		 * manejador asociado.
		 * Recuerde que esta rutina se esta ejecutando con las interrupciones
		 * deshabilitadas, por lo cual no existe forma de romper el
		 * ciclo infinito que se define a continuacion.*/
		for (;;)
			;
	}
}

/*
 * Rutina: install_exception_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de excepcion
 * para una de las excepciones de los procesadores x86.
 * Entrada:
 * 	unsigned char: vector (numero) de excepcion a configurar el manejador
 *  exception_handler : Rutina para manejar la excepcion especificada
 * Salida:
 * 	Ninguno. */
int install_exception_handler(unsigned char i, exception_handler handler) {
	if (exception_handlers[i] != NULL_INTERRUPT_HANDLER) {
		printf("Error! handler for exception %d was already set!\n", i);
		return -1;
	}
	exception_handlers[i] = handler;
}

/*
 * Rutina: uninstall_exception_handler
 * Descripcion: Esta rutina permite quitar un  manejador de excepcion
 * Entrada:
 * 	unsigned char: vector (numero) de excepcion de la cual se desea quitar
 * su manejador
 * Salida:
 * 	Ninguno. */
void uninstall_exception_handler(unsigned char i) {
	/* Simplemente quitar la referencia a la rutina de manejo de interrupcion.*/
	if (i > MAX_EXCEPTIONS) {
		return;
	}
	exception_handlers[i] = NULL_INTERRUPT_HANDLER;

}

