/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion dentro de la rutina cmain.
 *
 * En este proyecto se implementa la funcionalidad para que las tareas
 * puedan invocar llamadas al sistema (servicios del kernel).
 *
 * Vea el archivo readme.txt para mas detalles.
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>
#include <pm.h>
#include <idt.h>
#include <tss.h>
#include <exception.h>
#include <irq.h>
#include <timer.h>
#include <task.h>
#include <elf.h>
#include <string.h>


/*
 * Rutina: init_modules
 * Descripcion:
 * Inicializa los modulos (programas) cargados junto con el kernel.
 * Cada modulo cargado se transforma en una tarea que debe ser ejecutada.
 * Entrada:
 * 	multiboot_info_t * : Apuntador a la estructura de datos que contiene la
 * informacion multiboot, de la cual se pueden extractar los modulos cargados.
 * Salida:
 * 	Ninguno.
 * */
void init_modules(multiboot_info_t *);

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */
void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	char * linea;

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Si no se encuentra memoria disponible, terminar */
	if (memory_length <= 0) {
		printf("No available memory found!\n");
		return;
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);
	//printf("Memory start: %u\n", memory_start);

	/* Pasar a modo protegido, usando la GDT configurada en pm.c */
	setup_gdt();

	/* Configurar la IDT en idt.c */
	setup_idt();

	/* Ahora configurar el segmento de estado de tarea (TSS). Este segmento
	 * se utiliza por la arquitectura Intel para implementar la multitarea.
	 * */

	setup_tss();

	/* configurar el manejador por defecto de excepciones */
	setup_exceptions();

	/* configurar el manejador por defecto de IRQ*/
	setup_irq();

	/* configurar el manejador de IRQ para el timer */
	setup_timer();


	/* Inicializar el entorno de multitarea */
	setup_multitasking();

	/*crear las primeras tareas de kernel (PL=0) con los modulos adicionales
	 * cargados junto con el kernel.
	 * Consulte el archivo readme.txt y el archivo boot/grub/menu.lst para
	 * mas detalles de los modulos cargados.
	 */
	init_modules(info);

	/* Finalmente invocar a la rutina 'task_dispatcher'  para que busque
	 * la primera tarea a ejecutar. */
	task_dispatcher();

	/* No es necesario habilitar las interrupciones, ya que el kernel le
	 * transfiere el control a  la tarea a ejecutar y habilita las
	 * interrupciones automaticamente para esta tarea. */

}


/*
 * Rutina: init_modules
 * Descripcion:
 * Inicializa los modulos (programas) cargados junto con el kernel.
 * Cada modulo cargado se transforma en una tarea que debe ser ejecutada.
 * Estas tareas se ejecutan con un nivel de privilegio de 0.
 * Entrada:
 * 	multiboot_info_t * : Apuntador a la estructura de datos que contiene la
 * informacion multiboot, de la cual se pueden extractar los modulos cargados.
 * Salida:
 * 	Ninguno.
 * */
void init_modules(multiboot_info_t *info) {
		int i;
	/* si flags[3] = 1, se especificaron modulos que deben ser cargados junto
		 * con el kernel*/
	if (test_bit(info->flags, 3)) {
		mod_info_t * mod_info;
		int mod_count;
		unsigned int mod_start;
		unsigned int mod_end;
		char mod_program[80];
		/*
		printf("Modules available!. Start: %u Count: %u\n", info->mods_addr,
				info->mods_count);
		*/
		for (mod_info = (mod_info_t*)info->mods_addr, mod_count=0;
				mod_count <info->mods_count;
				mod_count++, mod_info++) {

			/* Se supone que los modulos cargados por GRUB son archivos ELF
			 * leidos en memoria, que inician en mod_start y finalizan en
			 * mod_end. */

			mod_start = mod_info->mod_start;
			mod_end = mod_info->mod_end;
			nexttok(mod_info->string, mod_program, 0);

			/* Crear la tarea con base en el codigo y los datos del
			 * modulo. El punto de entrada se toma del encabezado del
			 * modulo.
			 */
			//printf("Creating task %s\n", mod_info->string);
			create_task(mod_info->string,
					mod_start, mod_end, RING3_DPL);
		}
	}else {
		printf("Warning! no modules specified!\n");
	}
}
