/* task.c
 *
 * Este archivo implementa las rutinas para gestionar tareas con diferentes
 * niveles de privilegios.
 * Para ello, se deben crear dos nuevos descriptores de segmento en la GDT
 * para describir los segmentos de codigo y datos de las tareas.
 * Igualmente se debe crear un TSS y un descriptor de segmento de TSS que
 * tambien se almacena en la GDT.
 *
 * Las entradas en la GDT para las tareas se configuran cada vez
 * que ocurre un cambio de contexto. Esto permite aislar las tareas para
 * prevenir  que tengan acceso al espacio de otras tareas o del kernel.
 */

#include <pm.h>
#include <stdio.h>
#include <stdlib.h>
#include <task.h>
#include <tss.h>
#include <mm.h>
#include <timer.h>
#include <elf.h>
#include <idt.h>
#include <string.h>

/* Referencia a la primera tarea que se crea en el kernel. */
extern void init(void);

/* Tabla de tareas del kernel.
 * En esta estructura de datos se almacenan las tareas que son gestionadas
 * por el kernel. */
task tasks[TASK_LIMIT];

/* Variable para optimizar la busqueda de la siguiente entrada disponible
 * en la tabla de tareas */
static int next_available_task = 0;

/* Referencia a la tarea actual */
task * current_task = 0;

/* Variables que almacenan los selectores a los segmentos de codigo y datos
 * para las tareas.
 *  */
unsigned short task_code_selector;
unsigned short task_data_selector;

unsigned int kernel_esp;

/*
 * Rutina: return_from_interrupt
 * Descripcion: A partir de un marco de interrupcion, continua con la ejecucion
 * de una tarea.
 * Implementada en isr.S
 */
extern void return_from_interrupt(void);

extern unsigned short current_ss;
extern unsigned int current_esp;

/*
 * Rutina: idle_task
 * Descripcion: Esta rutina recibe el control del kernel, cuando no existe
 * una tarea definida.
 * Su tarea consiste en mantener ocupado el procesador.
 * */
void idle_task(void);

/*
 * Rutina: get_current_task_esp
 * Descripcion: Esta rutina permite obtener un apuntador a la posicion
 * del apuntador de la pila (esp) de la tarea actual.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	unsigned int * : Apuntador a la pila de la tarea, en el momento anterior
 * 					  a que esta invocara la interrupcion.
 * */
unsigned int * get_current_task_esp(void);

/*
 * Rutina: current_quantum
 * Descripcion: Esta rutina determina el quantum asignado a la tarea actual.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	int : quantum actual.
 * */
 int current_quantum() {
	if (current_task == NULL_TASK) {
		return USER_TASK_QUANTUM;
	}
	if (current_task->pl == RING0_DPL) {
		return KERNEL_TASK_QUANTUM;
	}else {
		return USER_TASK_QUANTUM;
	}
}

/*
 * Rutina: set_current_task
 * Descripcion: Establece la variable del kernel current_task a la
 * tarea especificada.
 * Entrada:
 * 	task * : Apuntador a la nueva tarea que se convierte en la tarea actual.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ set_current_task(task * t) {
	current_task = t;
}

/*
 * Rutina: task_handler
 * Descripcion: Esta rutina permite manejar la interrupcion invocada por
 * las tareas.
 * Entrada:
 * 	interrupt_state * : Apuntador al marco de interrupcion de la tarea actual
 * Salida:
 * 	Ninguno.
 *  */
void task_handler(interrupt_state * state) {
	int count;
	unsigned int parameter;
	unsigned int * esp;


	//printf("Task handler called!\n");

	/* Obtener la posicion que tenia esp antes de que ocurriera la
	 * interrupcion */
	esp = get_current_task_esp();

	count = 0;
	do {
		printf(" [%d] : %d", count,	*esp);
		esp++;
		count++;
	}while (count <255 &&
			((unsigned int)esp <= (unsigned int)current_task->base
									+ current_task->limit));

	/* Simplemente finalizar la tarea que invoco la interrupcion. */
	destroy_task(current_task);

}

/*
 * Rutina: allocate_task
 * Descripcion: Busca una entrada libre en la tabla de tareas del kernel.
 * Se considera una entrada libre aquella cuyo atributo state  sea igual
 * a TASK_AVAILABLE
 * */
static __inline__ task * allocate_task();

/*
 * Rutina: set_current_task
 * Descripcion: Establece la variable del kernel current_task a la
 * tarea especificada.
 * Entrada:
 * 	task * : Apuntador a la nueva tarea que se convierte en la tarea actual.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ set_current_task(task *);

/*
 * Rutina: print_task_context
 * Descripcion: Imprime el marco de interrupcion de una tarea,
 * apuntado por su atributo current_esp
 * Entrada:
 * 	task * : Apuntador a la tarea
 * Salida:
 * 	Ninguno.
 * */
void print_task_context(task *);

/*
 * Rutina: push_into_task_frame
 * Descripcion: Permite insertar valores en el marco de interrupcion
 * de una tarea.
 * Importante: Esta rutina se debe ejecutar solo cuando el selector de datos
 * (ds) se encuentra apuntando a un descriptor de segmento cuya base sea
 * la direccion lineal 0.
 * */
static __inline__ push_into_task_frame(task *t, unsigned int value) {
	if (t == NULL_TASK) {
		printf("Warning! pushing a value into a null task\n");
		return;
	}

	if (t->current_esp == 0) {
		printf("Warning! pushing a value into stack with top=0\n");
		return;
	}

	/* Decrementar el tope de la pila de la tarea */
	t->current_esp -= 4;

	unsigned int * ptr = (unsigned int*) (t->base + t->current_esp);

	*ptr = value;

}

/*
 * Rutina: push_into_task_stack
 * Descripcion: Permite insertar valores en la pila de una tarea.
 * Entrada:
 * 	task * : Tarea a la cual se insertara el valor en su pila.
 *  unsigned int : Valor a insertar en la pila de la tarea
 * Salida:
 * 	Ninguno.
 * Importante: Esta rutina supone que se ha configurado una pila
 * valida para la tarea, cuyo tope se encuentra en (t->base + t->current_esp).
 * */
static __inline__ push_into_task_stack(task *t, unsigned int value) {
	if (t == NULL_TASK) {
		printf("Warning! pushing a value into a null task\n");
		return;
	}

	if (t->current_esp == 0) {
		printf("Warning! pushing a value into stack with top=0\n");
		return;
	}

	/* Decrementar el tope de la pila de la tarea */
	t->current_esp -= 4;

	unsigned int * ptr = (unsigned int*) (t->base + t->current_esp);

	*ptr = value;
}

/*
 * Rutina: setup_multitasking
 * Descripcion: Inicializa las estructuras de datos necesarias para
 * que el kernel pueda administrar multiples tareas.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void setup_multitasking(void) {
	int i;

	/* Referencia a la IDT, para modificarla y permitir el acceso a una
	 * interrupcion por software de las tares con cualquier PL*/
	extern idt_descriptor * idt;

	/* Valores para almacenar el numero de entrada de la GDT para los
	 * segmentos de codigo y datos de las tareas*/
	int code_index;
	int data_index;

	/* Variable para almacenar el indice dentro del GDT que se asignara al
	 * descriptor de TSS para las tareas*/
	//unsigned short tss_index;

	//printf("Setting up multitasking...");

	/* Inicializar todas las tareas a nulo. Esto las marca con estado
	 * TASK_AVAILABLE */
	for (i = 0; i < TASK_LIMIT; i++) {
		tasks[i] = null_task();
	}

	/* Antes de crear la primera tarea de kernel, se deben configurar dos
	 * nuevos descriptores dentro de la GDT.
	 *
	 * Primero se crea un descriptor de segmento de codigo vacio.
	 * Luego se crea un descriptor de datos vacio.
	 * Estos dos descriptores se configuran adecuadamente cada vez que el kernel
	 * le pase el control a una tarea. */

	code_index = allocate_gdt_entry();

	if (!code_index) {
		printf("Unable to allocate a GDT entry for the user code segment!\n");
		return;
	}

	/* allocate_gdt_entry solo marca los descriptores como 'Presente'. Sus
	 * demas bits se establecen a 0. Esto es valido por ahora, ya que
	 * la entrada realmente se configura cada vez que se va a realizar
	 * un cambio de contexto. No obstante se debe reservar la entrada en
	 * la GDT y configurarla como vacia.*/

	task_code_selector = code_index << 3;

	//printf("User code selector: %x\n", task_code_selector);

	/* Reservar y establecer en nulo la entrada en la GDT para el
	 * descriptor de segmento de datos para la tarea */

	data_index = allocate_gdt_entry();

	if (!data_index) {
		printf("Unable to allocate a GDT entry for the user data segment!\n");
		return;
	}

	task_data_selector = data_index << 3;

	//printf("User data selector: %x\n", task_data_selector);
	set_current_task(NULL_TASK);

	/* Instalar el manejador de interrupcion para permitir que las
	 * tares soliciten servicios al kernel. En este caso se instala
	 * un manejador para la interrupcion 128 (0x80).*/
	install_interrupt_handler(128, task_handler);
}

/*
 * Rutina: allocate_task
 * Descripcion: Busca una entrada libre en la tabla de tareas del kernel.
 * Se considera una entrada libre aquella cuyo atributo state sea igual
 * a TASK_AVAILABLE
 * */
static __inline__ task * allocate_task() {
	int i;
	int start = next_available_task;
	i = start;
	do {
		if (tasks[i].state == TASK_AVAILABLE) {
			/* Tomar esta tarea */
			tasks[i].state = TASK_READY;
			tasks[i].pid = i;
			if (current_task != NULL_TASK) {
				tasks[i].ppid = current_task->pid;
			} else {
				tasks[i].ppid = -1;
			}
			next_available_task = (next_available_task + 1) % TASK_LIMIT;
			break;
		}
		i = (i + 1) % TASK_LIMIT;
	} while (i != next_available_task);
	//printf("Allocated task: %d start: %d\n", i, start);
	/* Si no se pudo encontrar una entrada valida, retornar NULL_TASK*/
	if (i == next_available_task) {
		return NULL_TASK;
	} else {
		return &tasks[i];
	}
}

/* Rutina: create_task
 * Descripcion: Crea una nueva tarea, a partir de un archivo ELF que se
 * encuentra leido en memoria.
 * Entrada:
 * char * : Argumentos que se deben pasar a la tarea
 * 	unsigned int : Direccion de inicio del archivo ELF en memoria
 * 	unsigned int : Direccion de fin del archivo ELF en memoria
 *  unsigned char : Nivel de privilegio de la tarea
 * */
task * create_task(char * command, unsigned int start, unsigned int end,
		unsigned char pl) {

	char * task_location;
	unsigned int size; /* Tamanio del codigo y los datos de la tarea */
	unsigned int limit; /* Tamanio de la tarea, incluyendo pila y heap */
	unsigned int entry;
	unsigned int data_offset; /* Desplazamiento de la seccion de datos de la
	 tarea */

	unsigned short code_selector;
	unsigned short data_selector;

	int i;

	/* Variables utilizadas para copiar los argumentos que se pasaran como
	 * parametro a la rutina main() de la tarea.
	 * De antemano se supone que maximo se pueden pasar 255 parametros
	 * a la tarea. */
	int argc;
	char * argv[255];

	int command_offset;
	int argument_offset;
	int char_count;
	char token[255];

	task * ret;

	/* Si ocurre cualquier error, retornar una tarea nula. */
	ret = NULL_TASK;

	/* Tratar de leer el encabezado del modulo: Se supone que cada modulo
	 * es un archivo en formato ELF. Si no lo es, retornar.
	 * Leer la documentacion del formato ELF para mas detalles.*/

	/* Los archivos con formato ELF comienzan con un encabezado, en el cual
	 * se encuentra informacion descriptiva.*/
	elf_header * header = (elf_header *) start;

	if (header == 0) {
		printf("Warning! module has no valid ELF header\n");
		return ret;
	}

	/* Si es un archivo ELF valido, los campos ident[0..3]
	 * deben contener 0x7f, 'E', 'L', 'F', respectivamente. */
	if (header->ident[0] != 0x7f || header->ident[1] != 'E' || header->ident[2]
			!= 'L' || header->ident[3] != 'F') {
		printf("Warning! module has no valid ELF header\n");
		return ret;
	}

	//printf("Program: %s Command: %s\n", program, command_line);

	entry = header->entry;

	/* El encabezado ELF es valido, obtener el mapa de las secciones
	 * de programa */
	program_header * ph;
	//printf(" ELF at %u\n", start);
	int count;
	size = 0;
	for (ph = (program_header*) (start + header->program_header_offset), count
			= 0; count < header->program_header_count; ph++, count++) {
		/* Buscar los segmentos validos, para determinar  el tamano del
		 * codigo y los datos de la tarea en memoria. */
		if (ph->type == ELF_PT_LOAD && ph->virtual_address <= end - start
				&& ph->file_size <= ph->memory_size) {
			/*
			 printf("Program header at %u\n", ph);
			 printf("align: %d filesize: %u flags: %b memory_size: %u "
			 "offset: %u phys: %u virt: %u, type=%d\n", ph->align,
			 ph->file_size, ph->flags, ph->memory_size, ph->offset,
			 ph->physical_address, ph->virtual_address, ph->type);
			 */

			if (ph->align == 0 || ph->align == 1) {
				size = size + ph->memory_size;
			} else {
				size = size + ph->align;
			}
		}
	}

	/* Redondear a un limite de 4096 bytes */
	size = size + (size % 4096);

	/* Agregar el espacio requerido para el heap y la pila de la tarea */
	limit = size + TASK_HEAP_SIZE + TASK_STACK_SIZE;

	//printf("Executable size: %u Task size: %u\n", size, limit);

	/* Reservar memoria para la tarea */
	task_location = (char *) mmalloc(limit);
	memset(task_location, 0, limit);
	//printf("task location: %u\n", (unsigned int)task_location);

	/* Y ahora copiar la informacion desde el archivo ELF leido en memoria
	 * al espacio de la tarea. Para ello se debe recorrer de nuevo los
	 * encabezados de programa (program_header) */
	for (ph = (program_header*) (start + header->program_header_offset), count
			= 0; count < header->program_header_count; ph++, count++) {
		/* Solo copiar los segmentos de programa que sean validos:
		 * de tipo ELF_PT_LOAD y cuya direccion virtual se encuentre
		 * dentro del limite del archivo ejecutable.
		 * Igualmente, de acuerdo con la especificacion ELF, el atributo
		 * file_size no debe ser mayor que el atributo memory_size
		 * */
		if (ph->type == ELF_PT_LOAD && ph->virtual_address <= end - start
				&& ph->file_size <= ph->memory_size) {
			/*
			 printf("Program header at %u\n", ph);
			 printf("align: %d filesize: %u flags: %b memory_size: %u "
			 "offset: %u phys: %u virt: %u, type=%d\n", ph->align,
			 ph->file_size, ph->flags, ph->memory_size, ph->offset,
			 ph->physical_address, ph->virtual_address, ph->type);
			 */

			/* Copiar a la posicion de la tarea*/
			/*
			 printf("Copying to %x from %u %d bytes\n",
			 (char*)((unsigned int)task_location +
			 ph->virtual_address),
			 (char *)(start + ph->offset), ph->file_size);
			 */
			//printf("flags: %b type: %b ", ph->flags, ph->type);
			/* Si el program header referencia la seccion de datos
			 * del ejecutable, almacenar su desplazamiento
			 * desde el inicio.*/
			if (ph->type == ELF_PT_LOAD && (ph->flags & (ELF_PH_WRITE))
					&& (ph->flags & (ELF_PH_READ))) {
				data_offset = ph->virtual_address;
				//printf("data offset at: %u\n", data_offset);
			}

			memcpy(
					(char*) ((unsigned int) task_location
							+ ph->virtual_address), (char *) (start
							+ ph->offset), ph->file_size);
		}
	}

	/* Inicializar la seccion de argumentos de la tarea en 0.
	 * Esta seccion se ubica siempre al inicio de su seccion de datos,
	 * y tiene un tamanio predefinido de 512 bytes. Ver archivo start.S,
	 * en su seccion .bootdata*/

	memset((void *) (task_location + data_offset), 0, 512);

	/* Ahora copiar los argumentos que se pasaron a la tarea en su seccion
	 * de argumentos. */

	/* Siempre la tarea recibe al menos un argumento, el nombre de
	 * programa. Este siempre es el primer argumento.*/
	argc = 0;
	command_offset = 0;
	argument_offset = 0;
	char_count = 0;

	//printf("Copying arguments to: %u\n", task_location + data_offset);

	do {
		/* La rutina nexttok automaticamente retorna una cadena terminada
		 * en nulo. */
		char_count = nexttok(command, token, command_offset);
		if (char_count > 0) {
			strcpy(task_location + data_offset + argument_offset, token);
			argv[argc] = (char*) (data_offset + argument_offset);
			command_offset = command_offset + char_count;
			argument_offset = argument_offset + char_count;
			//printf("Argument %u: %s\n", argv[argc], argv[argc]);
			argc++;
		}
	} while (char_count > 0);

	if (argc == 0) {
		printf("No argument copied!\n");
	}

	/* Buscar una entrada disponible dentro de la tabla de tareas
	 * del kernel */
	ret = allocate_task();

	//printf("Allocated task: %d\n", ret->pid);

	/* Si no existe una entrada disponible, terminar! */
	if (ret == NULL_TASK) {
		return ret;
	}

	ret->end = ret->base + size;

	ret->limit = limit;

	/* Configurar los atributos de la tarea. */

	/* La direccion base de la tarea */
	ret->base = task_location;

	/* El final del codigo y los datos de la tarea */
	ret->end = ret->base + size;

	/* Desplazamiento a partir de la direccion base en el cual
	 *  se encuentra la rutina a la cual se pasa el control inicialmente. */
	ret->entry = entry;

	/*printf("Task entry at %x (%d)\n", ret->base + ret->entry,
			ret->base + ret->entry);*/

	/* Direccion en la cual termina el espacio ocupado por el codigo, los
	 * datos y la seccion BSS de la tarea */

	ret->pl = pl;

	/* Definir los selectores de codigo y datos para la tarea */

	code_selector = (task_code_selector >> 3) << 3 | ret->pl;
	data_selector = (task_data_selector >> 3) << 3 | ret->pl;

	ret->current_ss = data_selector;
	ret->current_esp = ret->limit;


	/* Reservar el tope de la pila para uso del kernel.
	 * Para tareas con PL != 0: El TSS se configuro de forma tal
	 * que cada vez que ocurre una interrupcion, el kernel apunta
	 * automaticamente al tope de la pila de la tarea interrumpida. Debido
	 * a que el kernel debe almacenar el marco de interrupcion, se debe
	 * reservar el espacio equivalente al tamanio de la estructura
	 * interrupt_state */
	/* Reservar el espacio para el manejo de las interrupciones */
	ret->current_esp -= sizeof(interrupt_state);

	/* La pila de la tarea ahora luce asi:
	 +--------------------------+ <- ret->base + ret->limit (tope de la pila)
	 |                          |
	 |                          |  Espacio reservado para que cada vez
	 |  espacio vacio en el     |  que ocurra una interrupcion, el kernel
	 |  tope de la pila de la   |  automaticamente apunte ss:esp al tope de
	 |  tarea:                  |  la pila de la tarea.
	 |  Reservado para el kernel|
	 |  (tareas con PL !=0)     |
	 |  Para tareas con PL = 0, |  Para tareas con PL = 0, no ocurre un
	 |  este espacio no se usa. |  cambio de pila. No obstante se reserva el
	 |                          |  espacio para mantener un marco de pila
	 |                          |  estandar.
	 |--------------------------|<-- ret->current_esp
	 */

	/* Copiar los argumentos que se pasaran a la rutina principal de la
	 * tarea (main) en su pila.*/

	/* Almacenar los argumentos en la pila.
	 * Estos se almacenan de atras hacia adelante, es decir primero el
	 * argumento n-1, luego el argumento n-2 ... hasta el argumento 0. */
	for (i = 1; i <= argc; i++) {
		push_into_task_stack(ret, (unsigned int) argv[argc - i]);
	}

	/* y almacenar el numero de argumentos (argc) */
	push_into_task_stack(ret, argc);

	/* La pila de la tarea ahora luce asi:
	 +--------------------------+ <- ret->base + ret->limit (tope de la pila)
	 |                          |
	 |                          |  Espacio reservado para que cada vez
	 |  espacio vacio en el     |  que ocurra una interrupcion, el kernel
	 |  tope de la pila de la   |  automaticamente apunte ss:esp al tope de
	 |  tarea:                  |  la pila de la tarea.
	 |  Reservado para el kernel|
	 |  (tareas con PL !=0)     |
	 |  Para tareas con PL = 0, |  Para tareas con PL = 0, no ocurre un
	 |  este espacio no se usa. |  cambio de pila. No obstante se reserva el
	 |                          |  espacio para mantener un marco de pila
	 |                          |  estandar.
	 |--------------------------|
	 | ...                      |
	 |--------------------------| Argumentos a la rutina 'main' de la tarea
	 |argv[1]                   |
	 |--------------------------|
	 |argv[0]                   |
	 |--------------------------|
	 |argc (# de argumentos)    |
	 |--------------------------|
	 */


	/* Configurar el contexto inicial de la tarea. El contexto se configura
	 * para que parezca que la tarea ha sido interrumpida y su estado se ha
	 * almacenado en su pila. */


	/* Si el PL de la tarea es != 0, almacenar el apuntador actual al tope
	 * de la pila. De esta forma, cuando se retorne de la interrupcion,
	 * en el tope de la pila se encuentran los argumentos de la rutina 'main'.
	 *
	 * Para tareas con PL = 0, estos valores no se insertan en la pila.
	 *  */
	if (ret->pl != RING0_DPL) {
			push_into_task_stack(ret, ret->current_ss); /* old ss */
			push_into_task_stack(ret, ret->current_esp); /* old esp */
	}

	push_into_task_stack(ret, IF_ENABLE); /* old_eflags */
	push_into_task_stack(ret, code_selector); /* old cs */
	push_into_task_stack(ret, entry); /* old eip  = entry point del ejecutable*/
	push_into_task_stack(ret, 0); /* codigo de error */
	/* Valores almacenados en el mismo orden que las rutinas de
	 * servicio de interrupcion */
	push_into_task_stack(ret, 0); /* interrupcion generada */
	push_into_task_stack(ret, 0); /* eax *//* pusha*/
	push_into_task_stack(ret, 0); /* ecx */
	push_into_task_stack(ret, 0); /* edx */
	push_into_task_stack(ret, 0); /* ebx */
	push_into_task_stack(ret, 0); /* esp antes de pusha, ignorado*/
	push_into_task_stack(ret, 0); /* ebp */
	push_into_task_stack(ret, 0); /* esi */
	push_into_task_stack(ret, 0); /* edi */

	push_into_task_stack(ret, data_selector); /* ds */
	push_into_task_stack(ret, data_selector); /* es */
	push_into_task_stack(ret, data_selector); /* fs */
	push_into_task_stack(ret, data_selector); /* gs */

	/*
	 Ahora la pila para la nueva tarea se encuentra asi:
	 +--------------------------+ <- ret->base + ret->limit (tope de la pila)
	 |                          |
	 |                          |  Espacio reservado para que cada vez
	 |  espacio vacio en el     |  que ocurra una interrupcion, el kernel
	 |  tope de la pila de la   |  automaticamente apunte ss:esp al tope de
	 |  tarea:                  |  la pila de la tarea.
	 |  Reservado para el kernel|
	 |  (tareas con PL !=0)     |
	 |  Para tareas con PL = 0, |  Para tareas con PL = 0, no ocurre un
	 |  este espacio no se usa. |  cambio de pila. No obstante se reserva el
	 |                          |  espacio.
	 |--------------------------|
	 | ...                      |
	 |--------------------------| Argumentos a la rutina 'main' de la tarea
	 |argv[1]                   |
	 |--------------------------|
	 |argv[0]                   |
	 |--------------------------|
	 |argc (# de argumentos)    |
	 |--------------------------| <-+
	 | old ss                   |   | (Estos valores solo se almacenan para
	 |--------------------------|   |  tareas con PL != 0)
	 | old esp                  | --+
	 |--------------------------|
	 | eflags                   |   IF = 1
	 |--------------------------|
	 | old cs                   | Selector de codigo de la tarea
	 |--------------------------|
	 | old eip                  | Rutina inicial de la tarea (main)
	 |--------------------------|
	 | 0 (codigo de error)      | 0
	 |--------------------------|
	 | # de excepcion generada  | 0
	 |--------------------------|
	 | eax                      | 0
	 |--------------------------|
	 | ecx                      | 0
	 |--------------------------|
	 | edx                      | 0
	 |--------------------------|
	 | ebx                      | 0
	 |--------------------------|
	 | esp antes de pusha       | (ignorado)
	 |--------------------------|
	 | ebp                      | 0
	 |--------------------------|
	 | esi                      | 0
	 |--------------------------|
	 | edi                      | 0
	 |--------------------------|------------------------------------------
	 | ds                       | Selector de datos de la tarea
	 |--------------------------|
	 | es                       | Selector de datos de la tarea
	 |--------------------------|
	 | fs                       | Selector de datos de la tarea
	 |--------------------------|
	 | gs                       | Selector de datos de la tarea
	 |--------------------------|<-- ret->current_esp
	 */

	/* Inicializar las variables para la planificacion. */
	ret->timeslice = 0;
	ret->total_ticks = 0;

	return ret;
}

task * find_next_task() {
	int i;
	int start_index;
	task * ret;
	//printf("!");

	ret = NULL_TASK;

	/* Si no existe una tarea actual, buscar la siguiente tarea
	 * que se encuentre en estado de listo. */

	if (current_task != NULL_TASK) {
		start_index = current_task->pid;
	}else {
		start_index = 0;
	}
	i = start_index;
	do {
		if (tasks[i].state == TASK_READY) {
			ret = &tasks[i];
			break;
		}
		i = (i + 1) % TASK_LIMIT;
	} while (i != start_index);
	return ret;
}

/*
 * Rutina: task_dispatcher
 * Descripcion: Esta rutina busca la siguiente tarea que se debe ejecutar
 * y le entrega el control. Si no existe una tarea disponible,
 * le retorna el control a la rutina 'idle_task' como una tarea.
 * Entrada:
 * 	Ninguno.
 * Salida:
 *  Ninguno.
 * */
void task_dispatcher(void) {

	int i;
	task * new_task;

	unsigned short * ptr;
	/*if (current_task != NULL_TASK) {
		ptr = (unsigned short *)VIDEO_ADDR;
		ptr += current_task->pid;
		*ptr = (0x7f << 8 | ' ');
	}*/

	/* Si la tarea actual ha finalizado, informar. */
	if (current_task != NULL_TASK && current_task->state == TASK_FINISHED) {
		current_task->total_ticks += current_task->timeslice;
		/*printf("Task %d finished! total ticks: %u total secs: %d\n",
				current_task->pid,
				current_task->total_ticks,
				current_task->total_ticks / TIMER_FREQUENCY);*/
		current_task == NULL_TASK;
	}

    /* Si existe una tarea actual y su estado es 'en ejecucion',
	 * retornar a esa tarea siempre y cuando su quantum no haya terminado. */
	if (current_task != NULL_TASK && current_task->state == TASK_RUNNING) {
		/* Si la tarea actual aun tiene quantum, retornar a ella.*/
		if (current_task->timeslice < current_quantum()) {
			/* Establecer el estado de la tarea a 'listo', para poder invocar
			 * la rutina switch_to_task */
			current_task->state = TASK_READY;
			switch_to_task(current_task);
		}else {
			/* Buscar la siguiente tarea a ejecutar */
			new_task = find_next_task();

			/* Antes de pasarle el control a la nueva tarea, actualizar
			 * su estado y timeslice */
			current_task->state = TASK_READY;
			current_task->total_ticks += current_task->timeslice;
			current_task->timeslice = 0;

			/* Si existe otra tarea, pasarle el control. */
			if (new_task != NULL_TASK) {
				/* Pasar el control a la nueva tarea*/
				new_task->timeslice = 0;
				switch_to_task(new_task);
			}else {
				/* No existe otra tarea, retornar el control a la tarea
				 * actual */
				switch_to_task(current_task);
			}
		}
	}

	/* No existe una tarea actual, buscar una nueva tarea para pasarle el
	 * control */
	/* Buscar la siguiente tarea a ejecutar */
	new_task = find_next_task();
	/* Si existe otra tarea, pasarle el control. */
	if (new_task != NULL_TASK) {
		/* Pasar el control a la nueva tarea*/
		new_task->timeslice = 0;
		switch_to_task(new_task);
	}

	/* Si no se encuentra una tarea a la cual se debe pasar el control,
	 * pasar el control a la rutina 'idle_task()' como una tarea.*/
	switch_to_idle_task();
}

/*
 * Rutina: switch_to_task
 * Descripcion: Permite continuar la ejecucion de una tarea especificada.
 * Realiza los ajustes para cambiar de nivel de privilegio si es necesario.
 * Entrada:
 * 	task * : Tarea a convertirse en la tarea actual.
 * Salida:
 * 	Nunguno. Esta rutina no retorna, ya que la ejecucion continua en la
 * tarea especificada.
 * Importante: Esta rutina asume que los atributos current_ss y current_esp de
 * la tarea apuntan al inicio de un marco de interrupcion valido.
 * */
void switch_to_task(task * t) {

	extern tss kernel_tss;
	extern unsigned short kernel_tss_selector;
	/* Esta variable definida en isr.S contiene el valor del apuntador
	 * actual al tope de la pila (esp)*/
	extern unsigned int current_esp;
	/* Esta variable definida en isr.S contiene el valor del selector de
	 * pila (ss) */
	extern unsigned short current_ss;

	/* Si la tarea no existe o no se encuentra en estado de listo, retornar. */
	if (t == NULL_TASK || t->state != TASK_READY) {
		return;
	}

	//set_current_task(t);
	t->state = TASK_RUNNING;

	/* Configurar el descriptor de segmento de codigo de tarea para
	 * que apunte al espacio de la tarea  */
	task_code_selector
			= setup_gdt_descriptor(task_code_selector >> 3,
					code32_segment_descriptor((unsigned int) t->base,
					(unsigned int) t->limit + TASK_HEAP_SIZE
					+ TASK_STACK_SIZE, t->pl));

	/* Configurar el descriptor de segmento de datos de tarea para
	 * que apunte al espacio de la tarea  */
	task_data_selector
			= setup_gdt_descriptor(task_data_selector >> 3,
					data32_segment_descriptor((unsigned int) t->base,
							(unsigned int) t->limit + TASK_HEAP_SIZE
									+ TASK_STACK_SIZE, t->pl));

	/* Re-iniciar el registro de estado de tareas (TSS) del kernel. */
	kernel_tss = NULL_TSS;

	/* El TSS debe estar marcado como 'BUSY' para que sea valido
	 * retornar a la tarea como si hubiera sido interrumpida. */

	/* Configurar el apuntador de la pila, para que cuando suceda una
	 * interrupcion se configure automaticamente ss y esp con estos valores */

	/* Cuando una tarea con PL !=0 es interrumpida, el procesador toma
	 * los valores de ss y esp (el apuntador a la pila) de los atributos
	 * ss0 y esp0 del TSS actual.*/

	/* ss cuando ocurre una interrupcion: selector de datos de la tarea */
	kernel_tss.ss0 = kernel_data_selector;

	/* esp cuando ocurre una interrupcion: Tope de la pila de la tarea.
	 * Para facilitar el manejo de la pila desde el manejador de interrupcion,
	 * se toma una direccion lineal absoluta en el segmento de pila del kernel.
	 */
	kernel_tss.esp0 = (unsigned int)t->base + t->limit;

	/* Configurar de nuevo el selector de TSS del kernel, para que
	 * apunte a la tarea actual con el nivel de privilegio requerido. */
	kernel_tss_selector = setup_gdt_descriptor(kernel_tss_selector >> 3,
			tss_descriptor_32(&kernel_tss, t->pl));

	inline_assembly("ltr %0"::"a"(kernel_tss_selector));
	/* Establecer la variable global 'current_task' a la tarea especificada */
	set_current_task(t);
	unsigned short * ptr;
	/*
	if (current_task != NULL_TASK) {
		ptr = (unsigned short *)VIDEO_ADDR;
		ptr += current_task->pid;
		*ptr = (0x7f << 8 | '>');
	}*/



	/* Apuntar al marco de interrupcion de la tarea, para modificar
	 * el registro EFLAGS.
	 * En el momento en que ocurre la interrupcion, la isr correspondiente
	 * deshabilita las interrupciones y luego almacena el marco de
	 * interrupcion.
	 * Se requiere que al retornar a la tarea, las interrupciones
	 * se encuentren habilitadas. Por esta razon se debe modificar
	 * el valor de old_eflags en el marco de interrupcion de la tarea. */

	interrupt_state * state = (interrupt_state *)((unsigned int)t->base
			+ t->current_esp);

	/* Activar las interrupciones en el registro EFLAGS para la tarea a la
	 * cual se va a pasar. */
	state->old_eflags |= IF_ENABLE;

	/* Imprimir el id de la tarea a la cual se le pasa el control */
	//printf(" %d ", t->pid);

	/* Retornar al marco de interrupcion de la tarea.
	 * Para tareas con PL diferente de cero, se debe apuntar a una
	 * pila relativa al kernel.
	 * Para tareas con PL = 0, se apunta al marco de pila
	 * de la tarea.
	 * Las variables current_ss y current_esp se usan en la rutina
	 * return_from_interrupt() para apuntar al tope de la pila en la
	 * cual se encuentar el marco de interrupcion de la tarea a retornar el
	 * control.*/
	if (t->pl != RING0_DPL) {
		current_ss = kernel_data_selector;
		current_esp = (unsigned int)t->base + t->current_esp;
	}else{
		current_ss = task_data_selector;
		current_esp = t->current_esp;
	}

	/* La rutina return_from_interrupt hace uso de las variables current_ss
	 * y current_esp establecidas anteriormente, para apuntar ss:esp al
	 * marco de interrupcion de la tarea a la cual se debe retornar el
	 * control. */
	return_from_interrupt();

	/* Esta rutina no retorna, ya que return_from_interrupt() le paso
	 * el control a la tarea especificada.*/
}

/*
 * Rutina: switch_to_idle_task
 * Descripcion: Esta rutina retorna el control a una tarea vacia del kernel,
 * cuyo objetivo es detener el procesador hasta que ocurra el siguiente tick de
 * reloj.
 * 	Entrada:
 * 	Ninguno.
 * Salida:
 * 	Nunguno. Esta rutina no retorna, ya que la ejecucion continua en la
 * rutina 'idle_task'.
 * */
void switch_to_idle_task(void) {
	unsigned int * ptr;
	/* Crear un marco de interrupcion que apunta a la rutina 'idle_task'.
	 * Este marco de pila se crea en el tope de la pila del kernel. */
	set_current_task(NULL_TASK);

	/* Apuntar kerne_esp al tope de la pila del kernel, en el cual se
	 * dejo un espacio para almacenar un marco de interrupcion*/
	kernel_esp = KERNEL_STACK_TOP;

	ptr = (unsigned int*)kernel_esp;

	/* Ahora crear el marco de pila. */

	/* Dado que no hay cambio de contexto (esta rutina se ejecuta
	 * con DPL = 0), no es necesario almacenar los valores de old ss y old esp.
	 * */

	/* Importante: Se debe pasar el control a esta rutina, habilitando las
	 * interrupciones. En caso de no habilitar las interrupciones, el sistema
	 * se detiene. */
	*(--ptr) = IF_ENABLE; /* old_eflags , con interrupciones habilitadas */
	*(--ptr) = kernel_code_selector; /* old cs */
	*(--ptr) = (unsigned int)idle_task; /* old eip  = idle_task */
	*(--ptr) = 0; /* codigo de error */
	/* Valores almacenados en el mismo orden que las rutinas de
	 * servicio de interrupcion */
	*(--ptr) = 0; /* interrupcion generada */
	*(--ptr) = 0; /* eax *//* pusha*/
	*(--ptr) = 0; /* ecx */
	*(--ptr) = 0; /* edx */
	*(--ptr) = 0; /* ebx */
	*(--ptr) = 0; /* esp antes de pusha, ignorado*/
	*(--ptr) = 0; /* ebp */
	*(--ptr) = 0; /* esi */
	*(--ptr) = 0; /* edi */
	*(--ptr) = kernel_data_selector; /* ds */
	*(--ptr) = kernel_data_selector; /* es */
	*(--ptr) = kernel_data_selector; /* fs */
	*(--ptr) = kernel_data_selector; /* gs */

	/* Actualizar la variable kernel_esp que se utiliza al retornar de la
	 * interrupcion. */
	kernel_esp = (unsigned int)ptr;

	/* Establecer los valores de current_ss y current_esp para que
	 * apunten al marco de interrupcion */
	current_ss = kernel_data_selector;
	current_esp = kernel_esp;

	/* Retornar de la interrupcion.  Dado que no existe una tarea definida,
	 * se apunta al marco de interrupcion que se creo anteriormente. */
	return_from_interrupt();

}

/*
 * Rutina: print_task_context
 * Descripcion: Imprime el marco de interrupcion de una tarea,
 * apuntado por su atributo current_esp
 * Entrada:
 * 	task * : Apuntador a la tarea
 * Salida:
 * 	Ninguno.
 * */
void print_task_context(task *t) {

	interrupt_state * state;

	state = (interrupt_state *) ((unsigned int)t->base + t->current_esp);

	printf("[%d] gs: %x fs: %x es: %x ds: %x ", t->state, state->gs,
			state->fs, state->es, state->ds);

	printf("edi: %u esi: %u ebp: %u esp: %u ebx: %u edx: %u ecx: %u eax: %u ",
			state->edi, state->esi, state->ebp, state->esp, state->ebx,
			state->ebx, state->edx, state->ecx, state->eax);
	printf("Number: %d Error code: %d old eip: %u old cs: %x ", state->number,
			state->error_code, state->old_eip, state->old_cs);

	printf("Old eflags: %b old esp: %u old ss: %x\n", state->old_eflags,
			state->old_esp, state->old_ss);
}

/*
 * Rutina: idle_task
 * Descripcion: Esta rutina recibe el control del kernel, cuando no existe
 * una tarea definida.
 * Esta tarea se ejecuta en el contexto del kernel, con las interrupciones
 * habilitadas. Su proposito consiste en  detener el procesador, hasta que
 * ocurra una nueva interrupcion.
 * Dado que el control a esta tarea se pasa por medio de return_to_interrupt()
 * (ver rutina switch_to_idle_task) y dentro del marco de interrupcion para
 * retornar a ella se encuentran habilitadas las interrupciones, se asume
 * que en algun momento el timer ejecutara  una interrupcion que active
 * de nuevo al kernel.
 * */
void idle_task(void) {
	while (1) {
		/* Detener el procesador hasta que ocurra otro tick de reloj. */
		inline_assembly("hlt");
	}
}

/*
 * Rutina: destroy_task
 * Descripcion: Esta rutina marca la tarea como terminada, y luego invoca
 * al despachador de tareas.
 * Entrada:
 * 	task * : Tarea a destruir
 * Salida:
 * 	Ninguno.
 * */
void destroy_task(task * t){
	/* Marcar la tarea como finalizada. */
	t->state = TASK_FINISHED;

	task_dispatcher();
}

/*
 * Rutina: get_current_task_esp
 * Descripcion: Esta rutina permite obtener un apuntador a la posicion
 * del apuntador de la pila (esp) de la tarea actual.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	unsigned int * : Apuntador a la pila de la tarea, en el momento anterior
 * 					  a que esta invocara la interrupcion.
 * */
unsigned int * get_current_task_esp(void) {

	unsigned int * ret;
	unsigned int ptr;
	interrupt_state * state;

	/* Validar si la tarea actual esta definida */
	if (current_task == NULL_TASK){
		return 0;
	}

	/*
	 * Para tareas con PL = RING0_DPL, el marco de interrupcion se almaceno
	 * en la posicion actual de la pila
	 * Para tareas con PL != RING0_DPL, el marco de interrupcion se almaceno
	 * en el tope de la pila.
	 * */

	/* Obtener la direccion lineal de la posicion actual de la pila */

	/* Apuntar a la direccion de inicio de la tarea en memoria*/
	ptr = (unsigned int)current_task->base;

	/* Hallar el apuntador a la pila que la tarea tenia justo antes de que
	 * ocurriera la interrupcion */
	if (current_task->pl == RING0_DPL) {

	/* Para tareas con PL = RING0_DPL, el marco de interrupcion se
	 * almaceno en el tope actual de la pila. De esta forma se debe
	 * sumar a ptr el tamanio en bytes de  un marco de
	 * interrupcion, y restarle el tamanio de old_ss y old_esp ya que
	 * a las tareas de tipo 0 estos valores no se almacenan
	 * en el marco de interrupcion. */

	/* actual + marco de interrupcion - (tamanio (old_ss) + tamanio(old_esp))*/
		ptr += current_task->current_esp + sizeof(interrupt_state)
				- (sizeof(unsigned int) * 2);

	}else {
		/*
		 * Para tareas con PL != RING0_DPL, existen dos alternativas:
		 * a) el marco de interrupcion siempre se almacena en el limite de
		 * la pila:
		 * +----------------------+ <- current_task->limit
		 * | old ss               |
		 * +----------------------+
		 * | old esp              | <- Valor de esp a obtener
		 * +----------------------+
		 * | ....                 | (resto del marco de interrupcion)
		 * +----------------------+
		 * b) current_task->current_esp apunta al marco de interrupcion.
		 * De este marco de interrupcion se puede obtener el valor de old_esp.
		 * */

		/* Obtener un apuntador al marco de interrupcion actual. */
		interrupt_state * state = (interrupt_state *)(
				(unsigned int)current_task->base + current_task->current_esp
		);

		/* A partir del marco de interrupcion, obtener el atributo old_esp
		 * old_esp es relativo a la tarea, por lo cual se debe sumar
		 * a current_task->base para obtener un apuntador lineal.
		 * */
		ptr += state->old_esp;
	}

	ret = (unsigned int *)ptr;

	/* El primer valor encontrado en la pila es la direccion a la cual se debe
	 * retornar en el espacio de la tarea para continuar la ejecucion.
	 * Avanzar en la pila, para apuntar a los parametros que se encuentran
	 * en la pila de la tarea.*/
	ret++;


	return ret;

}
